@@ -19,6 +19,7 @@ except ImportError:
 
 from collections import defaultdict
 
+from plaso.analysis import definitions
 from plaso.engine import queue
 from plaso.lib import timelib
 from plaso.lib import errors
@@ -35,7 +36,7 @@ class AnalysisPlugin(queue.ItemQueueConsumer):
   # The name of the plugin. This is the name that is matched against when
   # loading plugins, so it is important that this name is short, concise and
   # explains the nature of the plugin easily. It also needs to be unique.
-  NAME = 'Plugin'
+  NAME = u'analysis_plugin'
 
   # A flag indicating whether or not this plugin should be run during extraction
   # phase or reserved entirely for post processing stage.
@@ -47,13 +48,6 @@ class AnalysisPlugin(queue.ItemQueueConsumer):
   # should be able to run during the extraction phase.
   ENABLE_IN_EXTRACTION = False
 
-  # All the possible report types.
-  TYPE_ANOMALY = 1    # Plugin that is inspecting events for anomalies.
-  TYPE_STATISTICS = 2   # Statistical calculations.
-  TYPE_ANNOTATION = 3    # Inspecting events with the primary purpose of
-                         # annotating or tagging them.
-  TYPE_REPORT = 4    # Inspecting events to provide a summary information.
-
   # A flag to indicate that this plugin takes a long time to compile a report.
   LONG_RUNNING_PLUGIN = False
 
@@ -64,7 +58,7 @@ class AnalysisPlugin(queue.ItemQueueConsumer):
       incoming_queue: A queue that is used to listen to incoming events.
     """
     super(AnalysisPlugin, self).__init__(incoming_queue)
-    self.plugin_type = self.TYPE_REPORT
+    self.plugin_type = definitions.PLUGIN_TYPE_REPORT
 
   def _ConsumeItem(self, item, analysis_mediator=None, **kwargs):
     """Consumes an item callback for ConsumeItems.
@@ -174,7 +168,7 @@ class HashTaggingAnalysisPlugin(AnalysisPlugin):
 
     self._analyzer = analyzer_class(self.hash_queue, self.hash_analysis_queue)
 
-  def _GenerateTextLine(self, analysis_mediator, pathspec, tag_string):
+  def _GenerateTextLine(self, analysis_mediator, pathspec, tag_strings):
     """Generates a line of text regarding the plugins findings.
 
     Args:
@@ -182,39 +176,43 @@ class HashTaggingAnalysisPlugin(AnalysisPlugin):
                          AnalysisMediator).
       pathspec: The pathspec (instance of dfvfs.PathSpec) whose hash was
                 looked up by the plugin.
-      tag_string: A string describing the plugin's results for a given pathspec.
+      tag_strings: A list of strings describing the plugin's results for a given
+                   pathspec.
     """
     display_name = analysis_mediator.GetDisplayName(pathspec)
-    return u'{0:s}: {1:s}'.format(display_name, tag_string)
+    return u'{0:s}: {1:s}'.format(display_name, u', '.join(tag_strings))
 
   @abc.abstractmethod
-  def GenerateTagString(self, hash_information):
-    """Generates a string to tag events with.
+  def GenerateTagStrings(self, hash_information):
+    """Generates a list of strings to tag events with.
 
     Args:
       hash_information: An object that encapsulates the result of the
                         analysis of a hash, as returned by the Analyze() method
                         of the analyzer class associated with this plugin.
+
+    Returns:
+      A list of string tags to apply to events.
     """
 
-  def _CreateTag(self, event_uuid, tag_string):
+  def _CreateTag(self, event_uuid, tag_strings):
     """Creates an event tag.
 
     Args:
       event_uuid: The UUID of the event that should be tagged.
-      tag_string: The string that the event should be tagged with.
+      tag_strings: A list of strings that the event should be tagged with.
     """
     event_tag = event.EventTag()
     event_tag.event_uuid = event_uuid
     event_tag.comment = u'Tag applied by {0:s} analysis plugin'.format(
         self.NAME)
-    event_tag.tags = [tag_string]
+    event_tag.tags = tag_strings
     return event_tag
 
   def _HandleHashAnalysis(self, hash_analysis):
     """Deals with a the results of the analysis of a hash.
 
-    This method ensures that a tag string is generated for the hash,
+    This method ensures that tag strings are generated for the hash,
     then tags all events derived from files with that hash.
 
     Args:
@@ -224,21 +222,21 @@ class HashTaggingAnalysisPlugin(AnalysisPlugin):
     Returns:
       A tuple of:
         pathspecs: A list of pathspecs that had the hash value looked up.
-        tag_string: The string that corresponds to the hash value that was
-                    looked up.
-        tags: A list of EventTags for the events that were extracted from the
+        tag_strings: A list of strings that corresponds to the hash value that
+                     was looked up.
+        tags: A list of EventTags for all events that were extracted from the
               pathspecs.
     """
     tags = []
     event_uuids = []
-    tag_string = self.GenerateTagString(hash_analysis.hash_information)
+    tag_strings = self.GenerateTagStrings(hash_analysis.hash_information)
     pathspecs = self._hash_pathspecs[hash_analysis.subject_hash]
     for pathspec in pathspecs:
       event_uuids.extend(self._event_uuids_by_pathspec[pathspec])
     for event_uuid in event_uuids:
-      tag = self._CreateTag(event_uuid, tag_string)
+      tag = self._CreateTag(event_uuid, tag_strings)
       tags.append(tag)
-    return pathspecs, tag_string, tags
+    return pathspecs, tag_strings, tags
 
   def _EnsureRequesterStarted(self):
     """Checks if the analyzer is running and starts it if not."""
@@ -342,12 +340,12 @@ class HashTaggingAnalysisPlugin(AnalysisPlugin):
         # The result queue is empty, but there could still be items that need
         # to be processed by the analyzer.
         continue
-      pathspecs, tag_string, new_tags = self._HandleHashAnalysis(
+      pathspecs, tag_strings, new_tags = self._HandleHashAnalysis(
           hash_analysis)
       tags.extend(new_tags)
       for pathspec in pathspecs:
         text_line = self._GenerateTextLine(
-            analysis_mediator, pathspec, tag_string)
+            analysis_mediator, pathspec, tag_strings)
         lines_of_text.append(text_line)
     self._analyzer.SignalAbort()
 
@@ -388,7 +386,7 @@ class HashAnalyzer(threading.Thread):
       hashes_per_batch: The number of hashes to analyze at once. The default
                         is 1.
       wait_after_analysis: The number of seconds to wait after each batch is
-                           analyzed. The default is 0.
+                           analyzed.
     """
     super(HashAnalyzer, self).__init__()
     self._abort = False
@@ -459,7 +457,7 @@ class HashAnalyzer(threading.Thread):
 
 
 class HTTPHashAnalyzer(HashAnalyzer):
-  """A class that provides a useful interface for hash plugins using HTTP"""
+  """A class that provides a useful interface for hash plugins using HTTP(S)"""
 
   def __init__(self, hash_queue, hash_analysis_queue, **kwargs):
     """Initializes a HTTP hash analyzer.
