@@ -258,8 +258,7 @@ class StorageMediaTool(tools.CLITool):
     Raises:
       BadConfigOption: if the options are invalid.
     """
-    filter_file = getattr(options, u'file_filter', None)
-
+    filter_file = self.ParseStringOption(options, u'file_filter')
     if not filter_file:
       return
 
@@ -450,7 +449,13 @@ class StorageMediaTool(tools.CLITool):
       if credential_type == u'skip':
         break
 
-      credential_data = getpass.getpass(u'Enter credential data: ')
+      getpass_string = u'Enter credential data: '
+      if sys.platform.startswith(u'win') and sys.version_info[0] < 3:
+        # For Python 2 on Windows getpass (win_getpass) requires an encoded
+        # byte string. For Python 3 we need it to be a Unicode string.
+        getpass_string = self._EncodeString(getpass_string)
+
+      credential_data = getpass.getpass(getpass_string)
       self._output_writer.Write(u'\n')
 
       if credential_type in self._BINARY_DATA_CREDENTIAL_TYPES:
@@ -577,7 +582,7 @@ class StorageMediaTool(tools.CLITool):
       if print_header:
         self._output_writer.Write(
             u'The following Volume Shadow Snapshots (VSS) were found:\n'
-            u'Identifier\tVSS store identifier\t\t\tCreation Time\n')
+            u'Identifier\t\tCreation Time\n')
 
         for volume_identifier in volume_identifiers:
           volume = volume_system.GetVolumeByIdentifier(volume_identifier)
@@ -586,14 +591,19 @@ class StorageMediaTool(tools.CLITool):
                 u'Volume missing for identifier: {0:s}.'.format(
                     volume_identifier))
 
-          vss_identifier = volume.GetAttribute(u'identifier')
           vss_creation_time = volume.GetAttribute(u'creation_time')
           vss_creation_time = timelib.Timestamp.FromFiletime(
               vss_creation_time.value)
           vss_creation_time = timelib.Timestamp.CopyToIsoFormat(
               vss_creation_time)
-          self._output_writer.Write(u'{0:s}\t\t{1:s}\t{2:s}\n'.format(
-              volume.identifier, vss_identifier.value, vss_creation_time))
+
+          if volume.HasExternalData():
+            external_data = u'\tWARNING: data stored outside volume'
+          else:
+            external_data = u''
+
+          self._output_writer.Write(u'{0:s}\t\t\t{1:s}{2:s}\n'.format(
+              volume.identifier, vss_creation_time, external_data))
 
         self._output_writer.Write(u'\n')
 
@@ -744,7 +754,7 @@ class StorageMediaTool(tools.CLITool):
       volume_scan_node: the volume scan node (instance of dfvfs.ScanNode).
 
     Raises:
-      SourceScannerError: if a VSS sub scan node scannot be retrieved.
+      SourceScannerError: if a VSS sub scan node cannot be retrieved.
     """
     if not self._process_vss:
       return
@@ -787,7 +797,7 @@ class StorageMediaTool(tools.CLITool):
                       argparse._ArgumentGroup).
     """
     argument_group.add_argument(
-        u'--credential', action=u'append', default=[], type=unicode,
+        u'--credential', action=u'append', default=[], type=str,
         dest=u'credentials', metavar=u'TYPE:DATA', help=(
             u'Define a credentials that can be used to unlock encrypted '
             u'volumes e.g. BitLocker. The credential is defined as type:data '
@@ -806,7 +816,7 @@ class StorageMediaTool(tools.CLITool):
     """
     argument_group.add_argument(
         u'-f', u'--file_filter', u'--file-filter', dest=u'file_filter',
-        action=u'store', type=unicode, default=None, help=(
+        action=u'store', type=str, default=None, help=(
             u'List of files to include for targeted collection of files to '
             u'parse, one line per file path, setup is /path|file - where each '
             u'element can contain either a variable set in the preprocessing '
@@ -886,31 +896,10 @@ class StorageMediaTool(tools.CLITool):
     self._ParseVSSProcessingOptions(options)
     self._ParseCredentialOptions(options)
 
-    self._source_path = getattr(options, self._SOURCE_OPTION, None)
+    self._source_path = self.ParseStringOption(options, self._SOURCE_OPTION)
     if not self._source_path:
       raise errors.BadConfigOption(u'Missing source path.')
 
-    if isinstance(self._source_path, str):
-      encoding = sys.stdin.encoding
-
-      # Note that sys.stdin.encoding can be None.
-      if not encoding:
-        encoding = self.preferred_encoding
-
-      # Note that the source path option can be an encoded byte string
-      # and we need to turn it into an Unicode string.
-      try:
-        self._source_path = unicode(
-            self._source_path.decode(encoding))
-      except UnicodeDecodeError as exception:
-        raise errors.BadConfigOption((
-            u'Unable to convert source path to Unicode with error: '
-            u'{0:s}.').format(exception))
-
-    elif not isinstance(self._source_path, unicode):
-      raise errors.BadConfigOption(
-          u'Unsupported source path, string type required.')
-
     self._source_path = os.path.abspath(self._source_path)
 
   def ScanSource(self):
