@@ -3,6 +3,7 @@
 
 import logging
 import os
+import re
 
 from dfvfs.analyzer import analyzer
 from dfvfs.lib import definitions as dfvfs_definitions
@@ -18,6 +19,7 @@ from plaso.engine import queue
 from plaso.lib import definitions
 from plaso.lib import errors
 from plaso.hashers import manager as hashers_manager
+from plaso.parsers import interface as parsers_interface
 from plaso.parsers import manager as parsers_manager
 
 
@@ -25,15 +27,50 @@ class BaseEventExtractionWorker(queue.ItemQueueConsumer):
   """Class that defines the event extraction worker base.
 
   This class is designed to watch a queue for path specifications of files
-  and directories (file entries) for which events need to be extracted.
+  and directories (file entries) and data streams for which events need to
+  be extracted.
 
   The event extraction worker needs to determine if a parser suitable
-  for parsing a particular file is available. All extracted event objects
-  are pushed on a storage queue for further processing.
+  for parsing a particular file entry or data stream is available. All
+  extracted event objects are pushed on a storage queue for further processing.
   """
 
   _DEFAULT_HASH_READ_SIZE = 4096
 
+  # TSK metadata files that need special handling.
+  _METADATA_FILE_LOCATIONS_TSK = frozenset([
+      # NTFS
+      u'/$AttrDef',
+      u'/$BadClus',
+      u'/$Bitmap',
+      u'/$Boot',
+      u'/$Extend/$ObjId',
+      u'/$Extend/$Quota',
+      u'/$Extend/$Reparse',
+      u'/$Extend/$RmMetadata/$Repair',
+      u'/$Extend/$RmMetadata/$TxfLog/$Tops',
+      u'/$Extend/$UsnJrnl',
+      u'/$LogFile',
+      u'/$MFT',
+      u'/$MFTMirr',
+      u'/$Secure',
+      u'/$UpCase',
+      u'/$Volume',
+      # HFS+/HFSX
+      u'/$ExtentsFile',
+      u'/$CatalogFile',
+      u'/$BadBlockFile',
+      u'/$AllocationFile',
+      u'/$AttributesFile',
+  ])
+
+  # TODO: make this filtering solution more generic. Also see:
+  # https://github.com/log2timeline/plaso/issues/467
+  _CHROME_CACHE_DATA_FILE_RE = re.compile(r'^[fF]_[0-9]{6}$')
+  _FIREFOX_CACHE_DATA_FILE_RE = re.compile(r'^[0-9a-fA-F]{5}[dm][0-9]{2}$')
+  _FIREFOX_CACHE2_DATA_FILE_RE = re.compile(r'^[0-9a-fA-F]{40}$')
+  _FSEVENTSD_FILE_RE = re.compile(r'^[0-9a-fA-F]{16}$')
+
   def __init__(
       self, identifier, path_spec_queue, event_queue_producer,
       parse_error_queue_producer, parser_mediator, resolver_context=None):
@@ -51,13 +88,15 @@ class BaseEventExtractionWorker(queue.ItemQueueConsumer):
                                   ItemQueueProducer).
       parser_mediator: A parser mediator object (instance of ParserMediator).
       resolver_context: Optional resolver context (instance of dfvfs.Context).
-                        The default is None.
+                        The default is None which will use the built in context
+                        which is not multi process safe. Note that every thread
+                        or process must have its own resolver context.
     """
     super(BaseEventExtractionWorker, self).__init__(path_spec_queue)
     self._compressed_stream_path_spec = None
     self._current_display_name = u''
     self._current_file_entry = None
-    self._enable_debug_output = False
+    self._enable_debug_mode = False
     self._identifier = identifier
     self._identifier_string = u'Worker_{0:d}'.format(identifier)
     self._file_scanner = None
@@ -71,6 +110,7 @@ class BaseEventExtractionWorker(queue.ItemQueueConsumer):
     self._produced_number_of_path_specs = 0
     self._resolver_context = resolver_context
     self._specification_store = None
+    self._usnjrnl_parser_object = None
 
     self._event_queue_producer = event_queue_producer
     self._parse_error_queue_producer = parse_error_queue_producer
@@ -85,6 +125,95 @@ class BaseEventExtractionWorker(queue.ItemQueueConsumer):
     self._profiling_sample = 0
     self._profiling_sample_rate = 1000
 
+  def _CanProcessFileEntryWithParser(self, file_entry, parser_object):
+    """Determines if a parser can process a file entry.
+
+    Args:
+      file_entry: the file entry relating to the data to be hashed (instance of
+                  dfvfs.FileEntry)
+      parser_object: a parser object (instance of BaseParser).
+
+    Returns:
+      A boolean value that indicates a match.
+    """
+    for filter_object in parser_object.FILTERS:
+      if filter_object.Match(file_entry):
+        return True
+
+    return False
+
+  def _CanSkipContentExtraction(self, file_entry):
+    """Determines if content extraction of a file entry can be skipped.
+
+    Args:
+      file_entry: the file entry relating to the data to be hashed (instance of
+                  dfvfs.FileEntry)
+
+    Returns:
+      A boolean value to indicate the content extraction can be skipped.
+    """
+    # TODO: make this filtering solution more generic. Also see:
+    # https://github.com/log2timeline/plaso/issues/467
+    location = getattr(file_entry.path_spec, u'location', None)
+    if not location:
+      return False
+
+    data_stream_name = getattr(file_entry.path_spec, u'data_stream', None)
+    if data_stream_name:
+      return False
+
+    file_system = file_entry.GetFileSystem()
+
+    path_segments = file_system.SplitPath(location)
+
+    if self._CHROME_CACHE_DATA_FILE_RE.match(path_segments[-1]):
+      location_segments = path_segments[:-1]
+      location_segments.append(u'index')
+      location = file_system.JoinPath(location_segments)
+      index_path_spec = path_spec_factory.Factory.NewPathSpec(
+          file_entry.type_indicator, location=location,
+          parent=file_entry.path_spec.parent)
+
+      if file_system.FileEntryExistsByPathSpec(index_path_spec):
+        # TODO: improve this check if "index" is a Chrome Cache index file.
+        return True
+
+    elif self._FIREFOX_CACHE_DATA_FILE_RE.match(path_segments[-1]):
+      location_segments = path_segments[:-4]
+      location_segments.append(u'_CACHE_MAP_')
+      location = file_system.JoinPath(location_segments)
+      cache_map_path_spec = path_spec_factory.Factory.NewPathSpec(
+          file_entry.type_indicator, location=location,
+          parent=file_entry.path_spec.parent)
+
+      if file_system.FileEntryExistsByPathSpec(cache_map_path_spec):
+        # TODO: improve this check if "_CACHE_MAP_" is a Firefox Cache
+        # version 1 cache map file.
+        return True
+
+    elif self._FIREFOX_CACHE2_DATA_FILE_RE.match(path_segments[-1]):
+      location_segments = path_segments[:-2]
+      location_segments.append(u'index')
+      location = file_system.JoinPath(location_segments)
+      index_path_spec = path_spec_factory.Factory.NewPathSpec(
+          file_entry.type_indicator, location=location,
+          parent=file_entry.path_spec.parent)
+
+      if file_system.FileEntryExistsByPathSpec(index_path_spec):
+        # TODO: improve this check if "index" is a Firefox Cache version 2
+        # index file.
+        return True
+
+    elif self._FSEVENTSD_FILE_RE.match(path_segments[-1]):
+      if len(path_segments) == 2 and path_segments[0].lower() == u'.fseventsd':
+        return True
+
+    elif len(path_segments) == 1 and path_segments[0].lower() in (
+        u'hiberfil.sys', u'pagefile.sys', u'swapfile.sys'):
+      return True
+
+    return False
+
   def _ConsumeItem(self, path_spec, **unused_kwargs):
     """Consumes an item callback for ConsumeItems.
 
@@ -102,33 +231,24 @@ class BaseEventExtractionWorker(queue.ItemQueueConsumer):
       self._ProcessPathSpec(self._compressed_stream_path_spec)
       self._compressed_stream_path_spec = None
 
-  def _GetSignatureMatchParserNames(self, file_entry):
-    """Determines if a file matches one of the known signatures.
+  def _DebugProcessPathSpec(self):
+    """Callback for debugging path specification processing failures."""
+    return
+
+  def _GetSignatureMatchParserNames(self, file_object):
+    """Determines if a file-like object matches one of the known signatures.
 
     Args:
-      file_entry: A file entry object (instance of dfvfs.FileEntry).
+      file_object: the file-like object whose contents will be checked
+                   for known signatures.
 
     Returns:
       A list of parser names for which the file entry matches their
       known signatures.
-
-    Raises:
-      IOError: if scanning for signatures failed.
     """
     parser_name_list = []
     scan_state = pysigscan.scan_state()
-
-    file_object = file_entry.GetFileObject()
-    try:
-      self._file_scanner.scan_file_object(scan_state, file_object)
-    except IOError as exception:
-      raise IOError(
-          u'Unable to scan for signatures with error: {0:s}'.format(exception))
-    finally:
-      file_object.close()
-
-      # Make sure frame.f_locals does not keep a reference to file_entry.
-      file_entry = None
+    self._file_scanner.scan_file_object(scan_state, file_object)
 
     for scan_result in scan_state.scan_results:
       format_specification = (
@@ -140,66 +260,83 @@ class BaseEventExtractionWorker(queue.ItemQueueConsumer):
 
     return parser_name_list
 
-  def _HashFileEntry(self, file_entry):
-    """Hashes the contents of a file entry.
+  def _HashDataStream(self, file_entry, data_stream_name=u''):
+    """Hashes the contents of a specific data stream of a file entry.
 
-    The resulting digest hashes are set in the parser mediator as attribute
-    to add to event objects.
+    The resulting digest hashes are set in the parser mediator as attributes
+    that are added to produced event objects. Note that some file systems
+    allow directories to have data streams, e.g. NTFS.
 
     Args:
-      file_entry: The file entry object to be hashed (instance of
+      file_entry: the file entry relating to the data to be hashed (instance of
                   dfvfs.FileEntry)
+      data_stream_name: optional data stream name. The default is
+                        an empty string which represents the default
+                        data stream.
     """
-    if not file_entry.IsFile() or not self._hasher_names:
+    if not self._hasher_names:
       return
 
-    logging.debug(u'[HashFileEntry] hashing file: {0:s}'.format(
+    logging.debug(u'[HashDataStream] hashing file: {0:s}'.format(
         self._current_display_name))
 
-    hasher_objects = hashers_manager.HashersManager.GetHasherObjects(
-        self._hasher_names)
+    self._status = definitions.PROCESSING_STATUS_HASHING
+
+    file_object = file_entry.GetFileObject(data_stream_name=data_stream_name)
+    if not file_object:
+      return
+
+    # Make sure frame.f_locals does not keep a reference to file_entry.
+    file_entry = None
 
-    file_object = file_entry.GetFileObject()
     try:
-      file_object.seek(0, os.SEEK_SET)
-
-      # We only do one read, then pass it to each of the hashers in turn.
-      data = file_object.read(self._DEFAULT_HASH_READ_SIZE)
-      while data:
-        for hasher in hasher_objects:
-          hasher.Update(data)
-        data = file_object.read(self._DEFAULT_HASH_READ_SIZE)
-
+      digest_hashes = hashers_manager.HashersManager.HashFileObject(
+          self._hasher_names, file_object,
+          buffer_size=self._DEFAULT_HASH_READ_SIZE)
     finally:
       file_object.close()
 
-      # Make sure frame.f_locals does not keep a reference to file_entry.
-      file_entry = None
-
-    # Get the digest values for every active hasher.
-    digests = {}
-    for hasher in hasher_objects:
-      digests[hasher.NAME] = hasher.GetStringDigest()
-      logging.debug(
-          u'[HashFileEntry] digest {0:s} calculated for file: {1:s}.'.format(
-              hasher.GetStringDigest(), self._current_display_name))
-
     if self._enable_profiling:
       self._ProfilingSampleMemory()
 
-    for hash_name, digest in iter(digests.items()):
+    for hash_name, digest_hash_string in iter(digest_hashes.items()):
       attribute_name = u'{0:s}_hash'.format(hash_name)
-      self._parser_mediator.AddEventAttribute(attribute_name, digest)
+      self._parser_mediator.AddEventAttribute(
+          attribute_name, digest_hash_string)
 
-    logging.debug(u'[HashFileEntry] completed hashing file: {0:s}'.format(
-        self._current_display_name))
+      logging.debug(
+          u'[HashDataStream] digest {0:s} calculated for file: {1:s}.'.format(
+              digest_hash_string, self._current_display_name))
 
-  def _ParseFileEntryWithParser(self, parser_object, file_entry):
+    logging.debug(
+        u'[HashDataStream] completed hashing file: {0:s}'.format(
+            self._current_display_name))
+
+  def _IsMetadataFile(self, file_entry):
+    """Determines if the file entry is a metadata file.
+
+    Args:
+      file_entry: a file entry object (instance of dfvfs.FileEntry).
+
+    Returns:
+      A boolean value indicating if the file entry is a metadata file.
+    """
+    if (file_entry.type_indicator == dfvfs_definitions.TYPE_INDICATOR_TSK and
+        file_entry.path_spec.location in self._METADATA_FILE_LOCATIONS_TSK):
+      return True
+
+    return False
+
+  def _ParseFileEntryWithParser(
+      self, parser_object, file_entry, file_object=None):
     """Parses a file entry with a specific parser.
 
     Args:
-      parser_object: A parser object (instance of BaseParser).
-      file_entry: A file entry object (instance of dfvfs.FileEntry).
+      parser_object: a parser object (instance of BaseParser).
+      file_entry: a file entry object (instance of dfvfs.FileEntry).
+      file_object: optional file-like object to parse. If not set the parser
+                   will use the parser mediator to open the file entry's
+                   default data stream as a file-like object
     """
     self._parser_mediator.ClearParserChain()
 
@@ -210,7 +347,13 @@ class BaseEventExtractionWorker(queue.ItemQueueConsumer):
       self._parsers_profiler.StartTiming(parser_object.NAME)
 
     try:
-      parser_object.UpdateChainAndParse(self._parser_mediator)
+      if isinstance(parser_object, parsers_interface.FileEntryParser):
+        parser_object.Parse(self._parser_mediator)
+      elif isinstance(parser_object, parsers_interface.FileObjectParser):
+        parser_object.Parse(self._parser_mediator, file_object)
+      else:
+        logging.warning(
+            u'{0:s} unsupported parser type.'.format(parser_object.NAME))
 
     # We catch the IOError so we can determine the parser that generated
     # the error.
@@ -375,11 +518,56 @@ class BaseEventExtractionWorker(queue.ItemQueueConsumer):
 
     return True
 
-  def _ProcessFileEntry(self, file_entry):
-    """Processes a file entry.
+  def _ProcessDataStream(self, file_entry, data_stream_name=u''):
+    """Processes a specific data stream of a file entry.
 
     Args:
       file_entry: A file entry object (instance of dfvfs.FileEntry).
+      data_stream_name: optional data stream name. The default is
+                        an empty string which represents the default
+                        data stream.
+    """
+    file_object = file_entry.GetFileObject(data_stream_name=data_stream_name)
+    if not file_object:
+      return
+
+    try:
+      parser_name_list = self._GetSignatureMatchParserNames(file_object)
+      if not parser_name_list:
+        parser_name_list = self._non_sigscan_parser_names
+
+      self._status = definitions.PROCESSING_STATUS_PARSING
+      for parser_name in parser_name_list:
+        parser_object = self._parser_objects.get(parser_name, None)
+        if not parser_object:
+          logging.warning(u'No such parser: {0:s}'.format(parser_name))
+          continue
+
+        if parser_object.FILTERS:
+          if not self._CanProcessFileEntryWithParser(file_entry, parser_object):
+            continue
+
+        logging.debug((
+            u'[ProcessDataStream] parsing file: {0:s} with parser: '
+            u'{1:s}').format(self._current_display_name, parser_name))
+
+        self._ParseFileEntryWithParser(
+            parser_object, file_entry, file_object=file_object)
+
+    finally:
+      file_object.close()
+
+      # Make sure frame.f_locals does not keep a reference to file_entry.
+      file_entry = None
+
+  def _ProcessFileEntry(self, file_entry, data_stream_name=u''):
+    """Processes a specific data stream of a file entry.
+
+    Args:
+      file_entry: A file entry object (instance of dfvfs.FileEntry).
+      data_stream_name: optional data stream name. The default is
+                        an empty string which represents the default
+                        data stream.
 
     Raises:
       RuntimeError: if the parser object is missing.
@@ -393,41 +581,59 @@ class BaseEventExtractionWorker(queue.ItemQueueConsumer):
 
     self._parser_mediator.SetFileEntry(file_entry)
 
+    logging.debug(u'[ProcessFileEntry] processing file entry: {0:s}'.format(
+        self._current_display_name))
+
     try:
-      self._HashFileEntry(file_entry)
+      if self._IsMetadataFile(file_entry):
+        parent_path_spec = getattr(file_entry.path_spec, u'parent', None)
+        if (self._usnjrnl_parser_object and parent_path_spec and
+            file_entry.name == u'$UsnJrnl' and data_stream_name == u'$J'):
 
-      logging.debug(u'[ProcessFileEntry] parsing file: {0:s}'.format(
-          self._current_display_name))
+          # To be able to ignore the sparse data ranges the UsnJrnl parser
+          # needs to read directly from the volume.
+          volume_file_object = path_spec_resolver.Resolver.OpenFileObject(
+              parent_path_spec, resolver_context=self._resolver_context)
 
-      # We always want to use the filestat parser.
-      if self._filestat_parser_object:
-        self._ParseFileEntryWithParser(self._filestat_parser_object, file_entry)
+          try:
+            self._ParseFileEntryWithParser(
+                self._usnjrnl_parser_object, file_entry,
+                file_object=volume_file_object)
+          finally:
+            volume_file_object.close()
 
-      is_archive = False
-      is_compressed_stream = False
-      is_file = file_entry.IsFile()
+      else:
+        # Not every file entry has a data stream. In such cases we want to
+        # extract the metadata only.
+        has_data_stream = file_entry.HasDataStream(data_stream_name)
 
-      if is_file:
-        is_compressed_stream = self._ProcessCompressedStreamFile(file_entry)
-        if not is_compressed_stream:
-          is_archive = self._ProcessArchiveFile(file_entry)
+        if has_data_stream:
+          self._HashDataStream(file_entry, data_stream_name=data_stream_name)
 
-      if is_file and not is_archive and not is_compressed_stream:
-        parser_name_list = self._GetSignatureMatchParserNames(file_entry)
-        if not parser_name_list:
-          parser_name_list = self._non_sigscan_parser_names
+        # We always want to use the filestat parser if set but we only want
+        # to invoke it once per file entry, so we only use it if we are
+        # processing the default (nameless) data stream.
+        if not data_stream_name and self._filestat_parser_object:
+          self._ParseFileEntryWithParser(
+              self._filestat_parser_object, file_entry)
 
-        for parser_name in parser_name_list:
-          parser_object = self._parser_objects.get(parser_name, None)
-          if not parser_object:
-            self._parser_mediator.ResetFileEntry()
-            raise RuntimeError(u'No such parser: {0:s}'.format(parser_name))
+        # Determine if the content of the file entry should not be extracted.
+        skip_content_extraction = self._CanSkipContentExtraction(file_entry)
+        if skip_content_extraction:
+          logging.info(u'Skipping content extraction of: {0:s}'.format(
+              self._current_display_name))
+        else:
+          is_archive = False
+          is_compressed_stream = False
 
-          logging.debug((
-              u'[ProcessFileEntry] parsing file: {0:s} with parser: '
-              u'{1:s}').format(self._current_display_name, parser_name))
+          if file_entry.IsFile():
+            is_compressed_stream = self._ProcessCompressedStreamFile(file_entry)
+            if not is_compressed_stream:
+              is_archive = self._ProcessArchiveFile(file_entry)
 
-          self._ParseFileEntryWithParser(parser_object, file_entry)
+          if has_data_stream and not is_archive and not is_compressed_stream:
+            self._ProcessDataStream(
+                file_entry, data_stream_name=data_stream_name)
 
     finally:
       if reference_count != self._resolver_context.GetFileObjectReferenceCount(
@@ -449,8 +655,9 @@ class BaseEventExtractionWorker(queue.ItemQueueConsumer):
     if self._enable_profiling:
       self._ProfilingSampleMemory()
 
-    logging.debug(u'[ParseFileEntry] done processing: {0:s}'.format(
-        self._current_display_name))
+    logging.debug(
+        u'[ProcessFileEntry] done processing file entry: {0:s}'.format(
+            self._current_display_name))
 
   def _ProcessPathSpec(self, path_spec):
     """Processes a path specification.
@@ -468,7 +675,13 @@ class BaseEventExtractionWorker(queue.ItemQueueConsumer):
                 path_spec.comparable))
         return
 
-      self._ProcessFileEntry(file_entry)
+      # Note that data stream can be set but contain None, we'll set it
+      # to an empty string here.
+      data_stream_name = getattr(path_spec, u'data_stream', None)
+      if not data_stream_name:
+        data_stream_name = u''
+
+      self._ProcessFileEntry(file_entry, data_stream_name=data_stream_name)
 
     except IOError as exception:
       logging.warning(
@@ -490,6 +703,9 @@ class BaseEventExtractionWorker(queue.ItemQueueConsumer):
               path_spec.comparable))
       logging.exception(exception)
 
+      if self._enable_debug_mode:
+        self._DebugProcessPathSpec()
+
     # Make sure frame.f_locals does not keep a reference to file_entry.
     file_entry = None
 
@@ -550,7 +766,7 @@ class BaseEventExtractionWorker(queue.ItemQueueConsumer):
     * A glob name for a single parser, eg: '*msie*' (case insensitive).
 
     Args:
-      parser_filter_string: Optional parser filter string. The default is None.
+      parser_filter_string: Optional parser filter string.
     """
     self._specification_store, non_sigscan_parser_names = (
         parsers_manager.ParsersManager.GetSpecificationStore(
@@ -558,7 +774,7 @@ class BaseEventExtractionWorker(queue.ItemQueueConsumer):
 
     self._non_sigscan_parser_names = []
     for parser_name in non_sigscan_parser_names:
-      if parser_name == u'filestat':
+      if parser_name in [u'filestat', u'usnjrnl']:
         continue
       self._non_sigscan_parser_names.append(parser_name)
 
@@ -572,6 +788,10 @@ class BaseEventExtractionWorker(queue.ItemQueueConsumer):
     if u'filestat' in self._parser_objects:
       del self._parser_objects[u'filestat']
 
+    self._usnjrnl_parser_object = self._parser_objects.get(u'usnjrnl', None)
+    if u'usnjrnl' in self._parser_objects:
+      del self._parser_objects[u'usnjrnl']
+
   def Run(self):
     """Extracts event objects from file entries."""
     self._parser_mediator.ResetCounters()
@@ -597,14 +817,14 @@ class BaseEventExtractionWorker(queue.ItemQueueConsumer):
     if self._enable_profiling:
       self._ProfilingStop()
 
-  def SetEnableDebugOutput(self, enable_debug_output):
-    """Enables or disables debug output.
+  def SetEnableDebugMode(self, enable_debug_mode):
+    """Enables or disables debug mode.
 
     Args:
-      enable_debug_output: boolean value to indicate if the debug output
-                           should be enabled.
+      enable_debug_mode: boolean value to indicate if the debug mode
+                         should be enabled.
     """
-    self._enable_debug_output = enable_debug_output
+    self._enable_debug_mode = enable_debug_mode
 
   def SetEnableProfiling(
       self, enable_profiling, profiling_sample_rate=1000,
@@ -617,7 +837,7 @@ class BaseEventExtractionWorker(queue.ItemQueueConsumer):
       profiling_sample_rate: optional integer indicating the profiling sample
                              rate. The value contains the number of files
                              processed. The default value is 1000.
-      profiling_type: optional profiling type. The default is 'all'.
+      profiling_type: optional profiling type.
     """
     self._enable_profiling = enable_profiling
     self._profiling_sample_rate = profiling_sample_rate
