@@ -3,7 +3,7 @@
 
 from plaso.formatters import mediator as formatters_mediator
 from plaso.frontend import frontend
-from plaso.lib import storage
+from plaso.storage import zip_file as storage_zip_file
 
 
 class AnalysisFrontend(frontend.Frontend):
@@ -13,7 +13,6 @@ class AnalysisFrontend(frontend.Frontend):
     """Initializes the front-end object."""
     super(AnalysisFrontend, self).__init__()
     self._data_location = None
-    self._storage_file = None
 
   def GetFormatterMediator(self):
     """Retrieves the formatter mediator.
@@ -30,12 +29,12 @@ class AnalysisFrontend(frontend.Frontend):
     Args:
       storage_file_path: the path of the storage file.
       read_only: optional boolean value to indicate the storage file should
-                 be opened in read-only mode. The default is True.
+                 be opened in read-only mode.
 
     Returns:
       The storage file object (instance of StorageFile).
     """
-    return storage.StorageFile(storage_file_path, read_only=read_only)
+    return storage_zip_file.StorageFile(storage_file_path, read_only=read_only)
 
   def SetDataLocation(self, data_location):
     """Set the data location.
@@ -45,11 +44,3 @@ class AnalysisFrontend(frontend.Frontend):
                      from.
     """
     self._data_location = data_location
-
-  def SetStorageFile(self, storage_file):
-    """Set the storage file.
-
-    Args:
-      storage_file: The path to the storage file being parsed.
-    """
-    self._storage_file = storage_file
