@@ -19,11 +19,12 @@ from plaso.frontend import presets
 from plaso.lib import definitions
 from plaso.lib import errors
 from plaso.lib import event
-from plaso.lib import storage
 from plaso.lib import timelib
 from plaso.multi_processing import multi_process
 from plaso.hashers import manager as hashers_manager
 from plaso.parsers import manager as parsers_manager
+from plaso.storage import writer as storage_writer
+from plaso.storage import zip_file as storage_zip_file
 
 import pytz
 
@@ -48,13 +49,14 @@ class ExtractionFrontend(frontend.Frontend):
     self._filter_expression = None
     self._filter_object = None
     self._mount_path = None
-    self._old_preprocess = False
     self._operating_system = None
     self._output_module = None
     self._parser_names = None
     self._process_archive_files = False
     self._profiling_sample_rate = self._DEFAULT_PROFILING_SAMPLE_RATE
     self._profiling_type = u'all'
+    self._use_old_preprocess = False
+    self._use_zeromq = False
     self._queue_size = self._DEFAULT_QUEUE_SIZE
     self._resolver_context = context.Context()
     self._single_process_mode = False
@@ -105,10 +107,9 @@ class ExtractionFrontend(frontend.Frontend):
 
     Args:
       os_guess: optional string containing the operating system guessed by
-                the preprocessing. The default is an empty string.
+                the preprocessing.
       os_version: optional string containing the operating system version
-                  determined by the preprocessing. The default is an empty
-                  string.
+                  determined by the preprocessing.
 
     Returns:
       The parser filter string or None.
@@ -149,8 +150,8 @@ class ExtractionFrontend(frontend.Frontend):
 
     return parser_filter_string
 
-  def _PreprocessSource(self, source_path_specs, source_type):
-    """Preprocesses the source.
+  def _PreprocessSources(self, source_path_specs, source_type):
+    """Preprocesses the sources.
 
     Args:
       source_path_specs: list of path specifications (instances of
@@ -162,16 +163,18 @@ class ExtractionFrontend(frontend.Frontend):
     """
     pre_obj = None
 
-    if self._old_preprocess and os.path.isfile(self._storage_file_path):
+    if self._use_old_preprocess and os.path.isfile(self._storage_file_path):
       # Check if the storage file contains a preprocessing object.
       try:
-        with storage.StorageFile(
-            self._storage_file_path, read_only=True) as store:
-          storage_information = store.GetStorageInformation()
+        with storage_zip_file.StorageFile(
+            self._storage_file_path, read_only=True) as storage_file:
+
+          storage_information = storage_file.GetStorageInformation()
           if storage_information:
             logging.info(u'Using preprocessing information from a prior run.')
             pre_obj = storage_information[-1]
             self._enable_preprocessing = False
+
       except IOError:
         logging.warning(u'Storage file does not exist, running preprocess.')
 
@@ -184,9 +187,8 @@ class ExtractionFrontend(frontend.Frontend):
         source_scanner.SourceScannerContext.SOURCE_TYPE_STORAGE_MEDIA_DEVICE,
         source_scanner.SourceScannerContext.SOURCE_TYPE_STORAGE_MEDIA_IMAGE]):
       try:
-        self._engine.PreprocessSource(
-            source_path_specs, self._operating_system,
-            resolver_context=self._resolver_context)
+        self._engine.PreprocessSources(
+            source_path_specs, resolver_context=self._resolver_context)
 
       except IOError as exception:
         logging.error(u'Unable to preprocess with error: {0:s}'.format(
@@ -204,7 +206,7 @@ class ExtractionFrontend(frontend.Frontend):
 
     return pre_obj
 
-  # TODO: have the frontend fill collecton information gradually
+  # TODO: have the frontend fill collection information gradually
   # and set it as the last step of preprocessing?
   # Split in:
   # * extraction preferences (user preferences)
@@ -216,18 +218,19 @@ class ExtractionFrontend(frontend.Frontend):
   #   * mount point
 
   def _PreprocessSetCollectionInformation(
-      self, pre_obj, source_type, unused_engine, filter_file=None,
-      parser_filter_string=None, preferred_encoding=u'utf-8'):
+      self, pre_obj, source_type, unused_engine, command_line_arguments=None,
+      filter_file=None, parser_filter_string=None, preferred_encoding=u'utf-8'):
     """Sets the collection information as part of the preprocessing.
 
     Args:
       pre_obj: the preprocess object (instance of PreprocessObject).
       source_type: the dfVFS source type definition.
       engine: the engine object (instance of BaseEngine).
-      filter_file: a path to a file that contains find specifications.
-                   The default is None.
-      parser_filter_string: optional parser filter string. The default is None.
-      preferred_encoding: optional preferred encoding. The default is UTF-8.
+      command_line_arguments: optional string of the command line arguments or
+                              None if not set.
+      filter_file: optional path to a file that contains find specifications.
+      parser_filter_string: optional parser filter string.
+      preferred_encoding: optional preferred encoding.
     """
     collection_information = {}
 
@@ -303,6 +306,8 @@ class ExtractionFrontend(frontend.Frontend):
     else:
       collection_information[u'method'] = u'OS collection'
 
+    collection_information[u'cmd_line'] = command_line_arguments
+
     pre_obj.collection_information = collection_information
 
   def _PreprocessSetTimezone(self, pre_obj, timezone=pytz.UTC):
@@ -311,7 +316,7 @@ class ExtractionFrontend(frontend.Frontend):
     Args:
       pre_obj: the previously created preprocessing object (instance of
                PreprocessObject) or None.
-      timezone: optional preferred timezone. The default is UTC.
+      timezone: optional preferred timezone.
     """
     if not timezone:
       timezone = pytz.UTC
@@ -348,13 +353,18 @@ class ExtractionFrontend(frontend.Frontend):
     """
     return hashers_manager.HashersManager.GetHashersInformation()
 
-  def GetParserPluginsInformation(self):
+  def GetParserPluginsInformation(self, parser_filter_string=None):
     """Retrieves the parser plugins information.
 
+    Args:
+      parser_filter_string: Optional parser filter string, where None
+                            represents all parsers and plugins.
+
     Returns:
       A list of tuples of parser plugin names and descriptions.
     """
-    return parsers_manager.ParsersManager.GetParserPluginsInformation()
+    return parsers_manager.ParsersManager.GetParserPluginsInformation(
+        parser_filter_string=parser_filter_string)
 
   def GetParserPresetsInformation(self):
     """Retrieves the parser presets information.
@@ -376,9 +386,18 @@ class ExtractionFrontend(frontend.Frontend):
     """
     return parsers_manager.ParsersManager.GetParsersInformation()
 
+  def GetNamesOfParsersWithPlugins(self):
+    """Retrieves the names of parser with plugins.
+
+    Returns:
+      A list of parser names.
+    """
+    return parsers_manager.ParsersManager.GetNamesOfParsersWithPlugins()
+
   def ProcessSources(
-      self, source_path_specs, source_type, enable_sigsegv_handler=False,
-      filter_file=None, hasher_names_string=None, parser_filter_string=None,
+      self, source_path_specs, source_type, command_line_arguments=None,
+      enable_sigsegv_handler=False, filter_file=None, hasher_names_string=None,
+      number_of_extraction_workers=0, parser_filter_string=None,
       preferred_encoding=u'utf-8', single_process_mode=False,
       status_update_callback=None,
       storage_serializer_format=definitions.SERIALIZER_FORMAT_PROTOBUF,
@@ -389,22 +408,23 @@ class ExtractionFrontend(frontend.Frontend):
       source_path_specs: list of path specifications (instances of
                          dfvfs.PathSpec) to process.
       source_type: the dfVFS source type definition.
+      command_line_arguments: optional string of the command line arguments or
+                              None if not set.
       enable_sigsegv_handler: optional boolean value to indicate the SIGSEGV
-                              handler should be enabled. The default is False.
+                              handler should be enabled.
       filter_file: optional path to a file that contains find specifications.
-                   The default is None.
       hasher_names_string: optional comma separated string of names of
-                           hashers to enable. The default is None.
-      parser_filter_string: optional parser filter string. The default is None.
-      preferred_encoding: optional preferred encoding. The default is UTF-8.
+                           hashers to enable.
+      number_of_extraction_workers: the number of extraction workers to run. If
+                                    0, the number will be selected
+                                    automatically.
+      parser_filter_string: optional parser filter string.
+      preferred_encoding: optional preferred encoding.
       single_process_mode: optional boolean value to indicate if the front-end
-                           should run in single process mode. The default is
-                           False.
+                           should run in single process mode.
       status_update_callback: optional callback function for status updates.
-                              The default is None.
       storage_serializer_format: optional storage serializer format.
-                                 The default is protobuf.
-      timezone: optional preferred timezone. The default is UTC.
+      timezone: optional preferred timezone.
 
     Returns:
       The processing status (instance of ProcessingStatus) or None.
@@ -439,7 +459,8 @@ class ExtractionFrontend(frontend.Frontend):
       self._engine = single_process.SingleProcessEngine(self._queue_size)
     else:
       self._engine = multi_process.MultiProcessEngine(
-          maximum_number_of_queued_items=self._queue_size)
+          maximum_number_of_queued_items=self._queue_size,
+          use_zeromq=self._use_zeromq)
 
     self._engine.SetEnableDebugOutput(self._debug_mode)
     self._engine.SetEnableProfiling(
@@ -447,7 +468,7 @@ class ExtractionFrontend(frontend.Frontend):
         profiling_sample_rate=self._profiling_sample_rate,
         profiling_type=self._profiling_type)
 
-    pre_obj = self._PreprocessSource(source_path_specs, source_type)
+    pre_obj = self._PreprocessSources(source_path_specs, source_type)
 
     self._operating_system = getattr(pre_obj, u'guessed_os', None)
 
@@ -486,21 +507,22 @@ class ExtractionFrontend(frontend.Frontend):
       filter_find_specs = None
 
     self._PreprocessSetCollectionInformation(
-        pre_obj, source_type, self._engine, filter_file=filter_file,
+        pre_obj, source_type, self._engine,
+        command_line_arguments=command_line_arguments, filter_file=filter_file,
         parser_filter_string=parser_filter_string,
         preferred_encoding=preferred_encoding)
 
     if self._output_module:
-      storage_writer = storage.BypassStorageWriter(
+      storage_writer_object = storage_writer.BypassStorageWriter(
           self._engine.event_object_queue, self._storage_file_path,
           output_module_string=self._output_module, pre_obj=pre_obj)
     else:
-      storage_writer = storage.FileStorageWriter(
+      storage_writer_object = storage_writer.FileStorageWriter(
           self._engine.event_object_queue, self._storage_file_path,
           buffer_size=self._buffer_size, pre_obj=pre_obj,
           serializer_format=storage_serializer_format)
 
-      storage_writer.SetEnableProfiling(
+      storage_writer_object.SetEnableProfiling(
           self._enable_profiling,
           profiling_type=self._profiling_type)
 
@@ -510,7 +532,7 @@ class ExtractionFrontend(frontend.Frontend):
         logging.debug(u'Starting extraction in single process mode.')
 
         processing_status = self._engine.ProcessSources(
-            source_path_specs, storage_writer,
+            source_path_specs, storage_writer_object,
             filter_find_specs=filter_find_specs,
             filter_object=self._filter_object,
             hasher_names_string=hasher_names_string,
@@ -527,13 +549,14 @@ class ExtractionFrontend(frontend.Frontend):
 
         # TODO: pass number_of_extraction_workers.
         processing_status = self._engine.ProcessSources(
-            source_path_specs, storage_writer,
+            source_path_specs, storage_writer_object,
             enable_sigsegv_handler=enable_sigsegv_handler,
             filter_find_specs=filter_find_specs,
             filter_object=self._filter_object,
             hasher_names_string=hasher_names_string,
             include_directory_stat=include_directory_stat,
             mount_path=self._mount_path,
+            number_of_extraction_workers=number_of_extraction_workers,
             parser_filter_string=parser_filter_string,
             process_archive_files=self._process_archive_files,
             status_update_callback=status_update_callback,
@@ -589,12 +612,22 @@ class ExtractionFrontend(frontend.Frontend):
       profiling_sample_rate: optional integer indicating the profiling sample
                              rate. The value contains the number of files
                              processed. The default value is 1000.
-      profiling_type: optional profiling type. The default is 'all'.
+      profiling_type: optional profiling type.
     """
     self._enable_profiling = enable_profiling
     self._profiling_sample_rate = profiling_sample_rate
     self._profiling_type = profiling_type
 
+  def SetShowMemoryInformation(self, show_memory=True):
+    """Sets a flag telling the worker monitor to show memory information.
+
+    Args:
+      show_memory: a boolean (defaults to True) that indicates whether or not
+                   the foreman should include memory information as part of
+                   the worker monitoring.
+    """
+    self._show_worker_memory_information = show_memory
+
   def SetStorageFile(self, storage_file_path):
     """Sets the storage file path.
 
@@ -603,27 +636,29 @@ class ExtractionFrontend(frontend.Frontend):
     """
     self._storage_file_path = storage_file_path
 
-  def SetStorageSerializer(self, storage_serializer_format):
-    """Sets the storage serializer.
+  def SetTextPrepend(self, text_prepend):
+    """Sets the text prepend.
 
     Args:
-      storage_serializer_format: String denoting the type of serializer
-                                 to be used in the storage. The values
-                                 can be either "proto" or "json".
+      text_prepend: free form text that is prepended to each path.
     """
-    if storage_serializer_format not in (
-        self._EVENT_SERIALIZER_FORMAT_JSON,
-        self._EVENT_SERIALIZER_FORMAT_PROTO):
-      return
+    self._text_prepend = text_prepend
 
-    self._storage_serializer_format = storage_serializer_format
-
-  def SetShowMemoryInformation(self, show_memory=True):
-    """Sets a flag telling the worker monitor to show memory information.
+  def SetUseOldPreprocess(self, use_old_preprocess):
+    """Set the use old preprocess flag.
 
     Args:
-      show_memory: A boolean (defaults to True) that indicates whether or not
-                   the foreman should include memory information as part of
-                   the worker monitoring.
+      use_old_preprocess: boolean value to indicate if the engine should
+                          use the old preprocessing information or run
+                          preprocessing again.
     """
-    self._show_worker_memory_information = show_memory
+    self._use_old_preprocess = use_old_preprocess
+
+  def SetUseZeroMQ(self, use_zeromq=False):
+    """Sets whether the frontend is using ZeroMQ for queueing or not.
+
+    Args:
+      use_zeromq: optional boolean value to indicate if ZeroMQ should be used
+                  for queuing.
+    """
+    self._use_zeromq = use_zeromq
