@@ -5,7 +5,6 @@ from __future__ import print_function
 import collections
 import multiprocessing
 import logging
-import sys
 
 from plaso import formatters   # pylint: disable=unused-import
 from plaso import output   # pylint: disable=unused-import
@@ -14,17 +13,18 @@ from plaso.analysis import manager as analysis_manager
 from plaso.analysis import mediator as analysis_mediator
 from plaso.engine import knowledge_base
 from plaso.engine import queue
+from plaso.engine import zeromq_queue
 from plaso.frontend import analysis_frontend
-from plaso.frontend import frontend
 from plaso.lib import bufferlib
-from plaso.lib import pfilter
+from plaso.lib import event
 from plaso.lib import timelib
 from plaso.multi_processing import multi_process
-from plaso.output import interface as output_interface
+from plaso.output import event_buffer as output_event_buffer
 from plaso.output import manager as output_manager
 from plaso.output import mediator as output_mediator
 from plaso.proto import plaso_storage_pb2
 from plaso.serializer import protobuf_serializer
+from plaso.storage import time_range as storage_time_range
 
 import pytz
 
@@ -44,12 +44,12 @@ class PsortFrontend(analysis_frontend.AnalysisFrontend):
     self._data_location = None
     self._filter_buffer = None
     self._filter_expression = None
+    # Instance of EventObjectFilter.
     self._filter_object = None
-    self._output_filename = None
-    self._output_file_object = None
     self._output_format = None
     self._preferred_language = u'en-US'
     self._quiet_mode = False
+    self._use_zeromq = False
 
   def _AppendEvent(self, event_object, output_buffer, event_queues):
     """Appends an event object to an output buffer and queues.
@@ -97,6 +97,15 @@ class PsortFrontend(analysis_frontend.AnalysisFrontend):
     """
     return analysis_manager.AnalysisPluginManager.GetAllPluginInformation()
 
+  def GetDisabledOutputClasses(self):
+    """Retrieves the disabled output classes.
+
+    Returns:
+      An output module generator which yields tuples of output class names
+      and type object.
+    """
+    return output_manager.OutputManager.GetDisabledOutputClasses()
+
   def GetOutputClasses(self):
     """Retrieves the available output classes.
 
@@ -106,39 +115,18 @@ class PsortFrontend(analysis_frontend.AnalysisFrontend):
     """
     return output_manager.OutputManager.GetOutputClasses()
 
-  def GetTimeSlice(self, event_time_string, duration=5, timezone=pytz.UTC):
-    """Retrieves a time slice.
-
-    Args:
-      event_time_string: event time string of the time slice or None.
-      duration: optional duration of the time slice in minutes.
-                The default is 5, which represent 2.5 minutes before
-                and 2.5 minutes after the event timestamp.
-      timezone: optional timezone. The default is UTC.
-
-    Returns:
-      A time slice object (instance of TimeSlice).
-    """
-    if event_time_string:
-      # TODO: move this to a place where bad config can be raised.
-      event_timestamp = timelib.Timestamp.FromTimeString(
-          event_time_string, timezone=timezone)
-    else:
-      event_timestamp = None
-
-    return frontend.TimeSlice(event_timestamp, duration=duration)
-
   def _ProcessAnalysisPlugins(
-      self, analysis_plugins, analysis_output_queue, storage_file, counter,
-      preferred_encoding=u'utf-8'):
+      self, analysis_plugins, analysis_report_incoming_queue, storage_file,
+      counter, preferred_encoding=u'utf-8'):
     """Runs the analysis plugins.
 
     Args:
       analysis_plugins: the analysis plugins.
-      analysis_output_queue: the analysis output queue (instance of Queue).
+      analysis_report_incoming_queue: the analysis output queue (instance of
+                                      Queue).
       storage_file: a storage file object (instance of StorageFile).
       counter: a counter object (instance of collections.Counter).
-      preferred_encoding: optional preferred encoding. The default is "utf-8".
+      preferred_encoding: optional preferred encoding.
     """
     if not analysis_plugins:
       return
@@ -170,7 +158,7 @@ class PsortFrontend(analysis_frontend.AnalysisFrontend):
 
     # Go over each output.
     analysis_queue_consumer = PsortAnalysisReportQueueConsumer(
-        analysis_output_queue, storage_file, self._filter_expression,
+        analysis_report_incoming_queue, storage_file, self._filter_expression,
         preferred_encoding=preferred_encoding)
 
     analysis_queue_consumer.ConsumeItems()
@@ -183,49 +171,118 @@ class PsortFrontend(analysis_frontend.AnalysisFrontend):
     for item, value in analysis_queue_consumer.counter.iteritems():
       counter[item] = value
 
-  def SetFilter(self, filter_object, filter_expression):
-    """Set the filter information.
+  def _StartAnalysisPlugins(
+      self, storage_file_path, analysis_plugins, pre_obj,
+      analysis_queue_port=None, analysis_report_incoming_queue=None,
+      command_line_arguments=None):
+    """Start all the analysis plugin.
 
     Args:
-      filter_object: a filter object (instance of FilterObject).
-      filter_expression: the filter expression string.
+      storage_file_path: string containing the path of the storage file.
+      analysis_plugins: list of analysis plugin objects (instance of
+                        AnalysisPlugin) that should be started.
+      pre_obj: The preprocessor object (instance of PreprocessObject).
+      analysis_queue_port: optional TCP port that the ZeroMQ analysis report
+                           queues should use.
+      analysis_report_incoming_queue: optional queue (instance of Queue) that
+                                      reports should to pushed to, when ZeroMQ
+                                      is not in use.
+      command_line_arguments: optional string of the command line arguments or
+                              None if not set.
     """
-    self._filter_object = filter_object
-    self._filter_expression = filter_expression
+    logging.info(u'Starting analysis plugins.')
+    self._SetAnalysisPluginProcessInformation(
+        storage_file_path, analysis_plugins, pre_obj,
+        command_line_arguments=command_line_arguments)
 
-  def SetPreferredLanguageIdentifier(self, language_identifier):
-    """Sets the preferred language identifier.
+    knowledge_base_object = knowledge_base.KnowledgeBase(pre_obj=pre_obj)
+    for analysis_plugin in analysis_plugins:
+      if self._use_zeromq:
+        analysis_plugin_output_queue = zeromq_queue.ZeroMQPushConnectQueue(
+            delay_open=True, port=analysis_queue_port)
+      else:
+        analysis_plugin_output_queue = analysis_report_incoming_queue
+
+      analysis_report_queue_producer = queue.ItemQueueProducer(
+          analysis_plugin_output_queue)
+
+      completion_event = multiprocessing.Event()
+      analysis_mediator_object = analysis_mediator.AnalysisMediator(
+          analysis_report_queue_producer, knowledge_base_object,
+          data_location=self._data_location,
+          completion_event=completion_event)
+      analysis_process = multiprocessing.Process(
+          name=u'Analysis {0:s}'.format(analysis_plugin.plugin_name),
+          target=analysis_plugin.RunPlugin,
+          args=(analysis_mediator_object,))
+
+      process_info = PsortAnalysisProcess(
+          completion_event, analysis_plugin, analysis_process)
+      self._analysis_process_info.append(process_info)
+
+      analysis_process.start()
+      logging.info(
+          u'Plugin: [{0:s}] started.'.format(analysis_plugin.plugin_name))
+
+    logging.info(u'Analysis plugins running')
+
+  def _SetAnalysisPluginProcessInformation(
+      self, storage_file_path, analysis_plugins, pre_obj,
+      command_line_arguments=None):
+    """Sets analysis plugin options in a preprocessor object.
 
     Args:
-      language_identifier: the language identifier string e.g. en-US for
-                           US English or is-IS for Icelandic.
+      storage_file_path: string containing the path of the storage file.
+      analysis_plugins: the list of analysis plugins to add.
+      pre_obj: the preprocessor object (instance of PreprocessObject).
+      command_line_arguments: optional string of the command line arguments or
+                              None if not set.
     """
-    self._preferred_language = language_identifier
+    analysis_plugin_names = [plugin.NAME for plugin in analysis_plugins]
+    time_of_run = timelib.Timestamp.GetNow()
 
-  def SetOutputFilename(self, output_filename):
-    """Sets the output format.
+    pre_obj.collection_information[u'cmd_line'] = command_line_arguments
+    pre_obj.collection_information[u'file_processed'] = storage_file_path
+    pre_obj.collection_information[u'method'] = u'Running Analysis Plugins'
+    pre_obj.collection_information[u'plugins'] = analysis_plugin_names
+    pre_obj.collection_information[u'time_of_run'] = time_of_run
+    pre_obj.counter = collections.Counter()
+
+  # TODO: fix docstring, function does not create the pre_obj the call to
+  # storage does. Likely refactor this functionality into the storage API.
+  def _GetLastGoodPreprocess(self, storage_file):
+    """Gets the last stored preprocessing object with time zone information.
+
+    From all preprocessing objects, try to get the last one that has
+    time zone information stored in it, the highest chance of it containing
+    the information we are seeking (defaulting to the last one). If there are
+    no preprocessing objects in the file, we'll make a new one
 
     Args:
-      output_filename: the output filename.
-    """
-    self._output_filename = output_filename
+      storage_file: a Plaso storage file object.
 
-  def SetOutputFormat(self, output_format):
-    """Sets the output format.
-
-    Args:
-      output_format: the output format.
+    Returns:
+      A preprocess object (instance of PreprocessObject), or None if there are
+      no preprocess objects in the storage file.
     """
-    self._output_format = output_format
+    pre_objs = storage_file.GetStorageInformation()
+    if not pre_objs:
+      return None
+    pre_obj = pre_objs[-1]
+    for obj in pre_objs:
+      if getattr(obj, u'time_zone_str', u''):
+        pre_obj = obj
+
+    return pre_obj
 
   def GetOutputModule(
       self, storage_file, preferred_encoding=u'utf-8', timezone=pytz.UTC):
     """Return an output module.
 
     Args:
-      storage_file: the storage file object (instance of StorageFile).
-      preferred_encoding: optional preferred encoding. The default is "utf-8".
-      timezone: optional timezone. The default is UTC.
+      storage_file: a storage file object (instance of StorageFile).
+      preferred_encoding: optional preferred encoding.
+      timezone: optional timezone.
 
     Returns:
       an output module object (instance of OutputModule) or None if not able to
@@ -243,8 +300,9 @@ class PsortFrontend(analysis_frontend.AnalysisFrontend):
       raise RuntimeError(exception)
 
     output_mediator_object = output_mediator.OutputMediator(
-        formatter_mediator, storage_file,
-        preferred_encoding=preferred_encoding, timezone=timezone)
+        formatter_mediator, preferred_encoding=preferred_encoding,
+        timezone=timezone)
+    output_mediator_object.SetStorageFile(storage_file)
 
     try:
       output_module = output_manager.OutputManager.NewOutputModule(
@@ -274,189 +332,52 @@ class PsortFrontend(analysis_frontend.AnalysisFrontend):
     if not analysis_plugins_string:
       return [], []
 
-    # Start queues and load up plugins.
-    event_queue_producers = []
-    event_queues = []
+    event_producers = []
+    # These are the queues analysis plugins will read from.
+    analysis_plugin_input_queues = []
     analysis_plugins_list = [
         name.strip() for name in analysis_plugins_string.split(u',')]
 
     for _ in range(0, len(analysis_plugins_list)):
-      # TODO: add upper queue limit.
-      analysis_plugin_queue = multi_process.MultiProcessingQueue(timeout=5)
-      event_queues.append(analysis_plugin_queue)
-      event_queue_producers.append(
-          queue.ItemQueueProducer(event_queues[-1]))
+      if self._use_zeromq:
+        output_queue = zeromq_queue.ZeroMQPushBindQueue()
+        # Open the queue so it can bind to a random port, and we can get the
+        # port number to use in the input queue.
+        output_queue.Open()
+        queue_port = output_queue.port
+        input_queue = zeromq_queue.ZeroMQPullConnectQueue(
+            port=queue_port, delay_open=True)
+        analysis_plugin_input_queues.append(input_queue)
+      else:
+        input_queue = multi_process.MultiProcessingQueue(timeout=5)
+        analysis_plugin_input_queues.append(input_queue)
+        output_queue = input_queue
+      event_producers.append(queue.ItemQueueProducer(output_queue))
 
     analysis_plugins = analysis_manager.AnalysisPluginManager.LoadPlugins(
-        analysis_plugins_list, event_queues)
+        analysis_plugins_list, analysis_plugin_input_queues)
 
     analysis_plugins = list(analysis_plugins)
 
-    return analysis_plugins, event_queue_producers
+    return analysis_plugins, event_producers
 
-  def SetQuietMode(self, quiet_mode=False):
-    """Sets whether tools is in quiet mode or not.
-
-    Args:
-      quiet_mode: boolean, when True the tool is in quiet mode.
-    """
-    self._quiet_mode = quiet_mode
-
-  def ProcessStorage(
-      self, output_module, storage_file, analysis_plugins,
-      event_queue_producers, deduplicate_events=True,
-      preferred_encoding=u'utf-8', time_slice=None, use_time_slicer=False):
-    """Processes a plaso storage file.
-
-    Args:
-      output_module: an output module (instance of OutputModule).
-      storage_file: the storage file object (instance of StorageFile).
-      analysis_plugins: list of analysis plugin objects (instance of
-                        AnalysisPlugin).
-      event_queue_producers: list of event queue producer objects (instance
-                             of ItemQueueProducer).
-      deduplicate_events: optional boolean value to indicate if the event
-                          objects should be deduplicated. The default is True.
-      preferred_encoding: optional preferred encoding. The default is "utf-8".
-      time_slice: optional time slice object (instance of TimeSlice).
-                  The default is None.
-      use_time_slicer: optional boolean value to indicate the 'time slicer'
-                       should be used. The default is False. The 'time slicer'
-                       will provide a context of events around an event of
-                       interest.
-
-    Returns:
-      A counter (an instance of counter.Counter) that contains the analysis
-      plugin results or None.
-
-    Raises:
-      RuntimeError: if a non-recoverable situation is encountered.
-    """
-    if time_slice:
-      if time_slice.event_timestamp:
-        pfilter.TimeRangeCache.SetLowerTimestamp(time_slice.start_timestamp)
-        pfilter.TimeRangeCache.SetUpperTimestamp(time_slice.end_timestamp)
-
-      elif use_time_slicer:
-        self._filter_buffer = bufferlib.CircularBuffer(time_slice.duration)
-
-    with storage_file:
-      storage_file.SetStoreLimit(self._filter_object)
-
-      # TODO: allow for single processing.
-      # TODO: add upper queue limit.
-      analysis_output_queue = multi_process.MultiProcessingQueue(timeout=5)
-
-      if analysis_plugins:
-        logging.info(u'Starting analysis plugins.')
-        # Within all preprocessing objects, try to get the last one that has
-        # time zone information stored in it, the highest chance of it
-        # containing the information we are seeking (defaulting to the last
-        # one).
-        pre_objs = storage_file.GetStorageInformation()
-        pre_obj = pre_objs[-1]
-        for obj in pre_objs:
-          if getattr(obj, u'time_zone_str', u''):
-            pre_obj = obj
-
-        # Fill in the collection information.
-        pre_obj.collection_information = {}
-        if preferred_encoding:
-          cmd_line = u' '.join(sys.argv)
-          try:
-            pre_obj.collection_information[u'cmd_line'] = cmd_line.decode(
-                preferred_encoding)
-          except UnicodeDecodeError:
-            pass
-        pre_obj.collection_information[u'file_processed'] = (
-            self._storage_file)
-        pre_obj.collection_information[u'method'] = u'Running Analysis Plugins'
-        analysis_plugin_names = [plugin.NAME for plugin in analysis_plugins]
-        pre_obj.collection_information[u'plugins'] = analysis_plugin_names
-        time_of_run = timelib.Timestamp.GetNow()
-        pre_obj.collection_information[u'time_of_run'] = time_of_run
-
-        pre_obj.counter = collections.Counter()
-
-        # Assign the preprocessing object to the storage.
-        # This is normally done in the construction of the storage object,
-        # however we cannot do that here since the preprocessing object is
-        # stored inside the storage file, so we need to open it first to
-        # be able to read it in, before we make changes to it. Thus we need
-        # to access this protected member of the class.
-        # pylint: disable=protected-access
-        storage_file._pre_obj = pre_obj
-
-        knowledge_base_object = knowledge_base.KnowledgeBase(pre_obj=pre_obj)
-
-        # Now we need to start all the plugins.
-        for analysis_plugin in analysis_plugins:
-          analysis_report_queue_producer = queue.ItemQueueProducer(
-              analysis_output_queue)
-
-          completion_event = multiprocessing.Event()
-          analysis_mediator_object = analysis_mediator.AnalysisMediator(
-              analysis_report_queue_producer, knowledge_base_object,
-              data_location=self._data_location,
-              completion_event=completion_event)
-          analysis_process = multiprocessing.Process(
-              name=u'Analysis {0:s}'.format(analysis_plugin.plugin_name),
-              target=analysis_plugin.RunPlugin,
-              args=(analysis_mediator_object,))
-          process_info = PsortAnalysisProcess(
-              completion_event, analysis_plugin, analysis_process)
-          self._analysis_process_info.append(process_info)
-
-          analysis_process.start()
-          logging.info(
-              u'Plugin: [{0:s}] started.'.format(analysis_plugin.plugin_name))
-      else:
-        event_queue_producers = []
-
-      output_buffer = output_interface.EventBuffer(
-          output_module, deduplicate_events)
-      with output_buffer:
-        counter = self.ProcessOutput(
-            storage_file, output_buffer, my_filter=self._filter_object,
-            filter_buffer=self._filter_buffer,
-            analysis_queues=event_queue_producers)
-
-      for information in storage_file.GetStorageInformation():
-        if hasattr(information, u'counter'):
-          counter[u'Stored Events'] += information.counter[u'total']
-
-      if not self._quiet_mode:
-        logging.info(u'Output processing is done.')
-
-      # Get all reports and tags from analysis plugins.
-      self._ProcessAnalysisPlugins(
-          analysis_plugins, analysis_output_queue, storage_file, counter,
-          preferred_encoding=preferred_encoding)
-
-    if self._output_file_object:
-      self._output_file_object.close()
-      self._output_file_object = None
-
-    if self._filter_object and not counter[u'Limited By']:
-      counter[u'Filter By Date'] = (
-          counter[u'Stored Events'] - counter[u'Events Included'] -
-          counter[u'Events Filtered Out'])
-
-    return counter
-
-  def ProcessOutput(
-      self, storage_file, output_buffer, my_filter=None, filter_buffer=None,
-      analysis_queues=None):
+  def ProcessEventsFromStorage(
+      self, storage_file, output_buffer, analysis_queues=None,
+      filter_buffer=None, my_filter=None, time_slice=None):
     """Reads event objects from the storage to process and filter them.
 
     Args:
-      storage_file: The storage file object (instance of StorageFile).
-      output_buffer: The output buffer object (instance of EventBuffer).
-      my_filter: Optional filter object (instance of PFilter).
-                 The default is None.
-      filter_buffer: Optional filter buffer used to store previously discarded
-                     events to store time slice history. The default is None.
-      analysis_queues: Optional list of analysis queues. The default is None.
+      storage_file: the storage file object (instance of StorageFile).
+      output_buffer: the output buffer object (instance of EventBuffer).
+      analysis_queues: optional list of analysis queues.
+      filter_buffer: optional filter buffer used to store previously discarded
+                     events to store time slice history.
+      my_filter: optional filter object (instance of PFilter).
+      time_slice: optional time slice object (instance of TimeRange).
+
+    Returns:
+      A Counter object (instance of collections.Counter), that tracks the
+      number of unique events extracted from storage.
     """
     counter = collections.Counter()
     my_limit = getattr(my_filter, u'limit', 0)
@@ -464,9 +385,13 @@ class PsortFrontend(analysis_frontend.AnalysisFrontend):
     if not analysis_queues:
       analysis_queues = []
 
-    event_object = storage_file.GetSortedEntry()
-    while event_object:
-      if my_filter:
+    # TODO: refactor to use StorageReader.
+    for event_object in storage_file.GetSortedEntries(time_range=time_slice):
+      # TODO: clean up this function.
+      if not my_filter:
+        counter[u'Events Included'] += 1
+        self._AppendEvent(event_object, output_buffer, analysis_queues)
+      else:
         event_match = event_object
         if isinstance(event_object, plaso_storage_pb2.EventObject):
           # TODO: move serialization to storage, if low-level filtering is
@@ -505,14 +430,10 @@ class PsortFrontend(analysis_frontend.AnalysisFrontend):
             counter[u'Events Filtered Out'] += 1
           else:
             counter[u'Events Filtered Out'] += 1
-      else:
-        counter[u'Events Included'] += 1
-        self._AppendEvent(event_object, output_buffer, analysis_queues)
-
-      event_object = storage_file.GetSortedEntry()
 
     for analysis_queue in analysis_queues:
       analysis_queue.Close()
+
     if output_buffer.duplicate_counter:
       counter[u'Duplicate Removals'] = output_buffer.duplicate_counter
 
@@ -520,6 +441,148 @@ class PsortFrontend(analysis_frontend.AnalysisFrontend):
       counter[u'Limited By'] = my_limit
     return counter
 
+  def ProcessStorage(
+      self, output_module, storage_file, storage_file_path, analysis_plugins,
+      event_queue_producers, command_line_arguments=None,
+      deduplicate_events=True, preferred_encoding=u'utf-8',
+      time_slice=None, use_time_slicer=False):
+    """Processes a plaso storage file.
+
+    Args:
+      output_module: an output module (instance of OutputModule).
+      storage_file: the storage file object (instance of StorageFile).
+      storage_file_path: string containing the path of the storage file.
+      analysis_plugins: list of analysis plugin objects (instance of
+                        AnalysisPlugin).
+      event_queue_producers: list of event queue producer objects (instance
+                             of ItemQueueProducer).
+      command_line_arguments: optional string of the command line arguments or
+                              None if not set.
+      deduplicate_events: optional boolean value to indicate if the event
+                          objects should be deduplicated.
+      preferred_encoding: optional preferred encoding.
+      time_slice: optional time slice object (instance of TimeSlice).
+      use_time_slicer: optional boolean value to indicate the 'time slicer'
+                       should be used. The 'time slicer' will provide a
+                       context of events around an event of interest.
+
+    Returns:
+      A counter (an instance of collections.Counter) that tracks the number of
+      events extracted from storage, and the analysis plugin results.
+
+    Raises:
+      RuntimeError: if a non-recoverable situation is encountered.
+    """
+    time_slice = None
+    if time_slice:
+      if time_slice.event_timestamp is not None:
+        time_slice = storage_time_range.TimeRange(
+            time_slice.start_timestamp, time_slice.end_timestamp)
+
+      elif use_time_slicer:
+        self._filter_buffer = bufferlib.CircularBuffer(time_slice.duration)
+
+    with storage_file:
+      # TODO: allow for single processing.
+      # TODO: add upper queue limit.
+      analysis_queue_port = None
+      if self._use_zeromq:
+        analysis_report_incoming_queue = zeromq_queue.ZeroMQPullBindQueue(
+            delay_open=False, port=None, linger_seconds=5)
+        analysis_queue_port = analysis_report_incoming_queue.port
+      else:
+        analysis_report_incoming_queue = multi_process.MultiProcessingQueue(
+            timeout=5)
+
+      pre_obj = self._GetLastGoodPreprocess(storage_file)
+      if pre_obj is None:
+        pre_obj = event.PreprocessObject()
+
+      if analysis_plugins:
+        self._StartAnalysisPlugins(
+            storage_file_path, analysis_plugins, pre_obj,
+            analysis_queue_port=analysis_queue_port,
+            analysis_report_incoming_queue=analysis_report_incoming_queue,
+            command_line_arguments=command_line_arguments)
+
+        # Assign the preprocessing object to the storage.
+        # This is normally done in the construction of the storage object,
+        # however we cannot do that here since the preprocessing object is
+        # stored inside the storage file, so we need to open it first to
+        # be able to read it in, before we make changes to it. Thus we need
+        # to access this protected member of the class.
+        # pylint: disable=protected-access
+        storage_file._pre_obj = pre_obj
+      else:
+        event_queue_producers = []
+
+      output_buffer = output_event_buffer.EventBuffer(
+          output_module, deduplicate_events)
+      with output_buffer:
+        counter = self.ProcessEventsFromStorage(
+            storage_file, output_buffer, analysis_queues=event_queue_producers,
+            filter_buffer=self._filter_buffer, my_filter=self._filter_object,
+            time_slice=time_slice)
+
+      for information in storage_file.GetStorageInformation():
+        if hasattr(information, u'counter'):
+          counter[u'Stored Events'] += information.counter[u'total']
+
+      # Get all reports and tags from analysis plugins.
+      self._ProcessAnalysisPlugins(
+          analysis_plugins, analysis_report_incoming_queue, storage_file,
+          counter, preferred_encoding=preferred_encoding)
+
+    if self._filter_object and not counter[u'Limited By']:
+      counter[u'Filter By Date'] = (
+          counter[u'Stored Events'] - counter[u'Events Included'] -
+          counter[u'Events Filtered Out'])
+
+    return counter
+
+  def SetFilter(self, filter_object, filter_expression):
+    """Set the filter information.
+
+    Args:
+      filter_object: a filter object (instance of FilterObject).
+      filter_expression: the filter expression string.
+    """
+    self._filter_object = filter_object
+    self._filter_expression = filter_expression
+
+  def SetPreferredLanguageIdentifier(self, language_identifier):
+    """Sets the preferred language identifier.
+
+    Args:
+      language_identifier: the language identifier string e.g. en-US for
+                           US English or is-IS for Icelandic.
+    """
+    self._preferred_language = language_identifier
+
+  def SetOutputFormat(self, output_format):
+    """Sets the output format.
+
+    Args:
+      output_format: the output format.
+    """
+    self._output_format = output_format
+
+  def SetQuietMode(self, quiet_mode=False):
+    """Sets whether tools is in quiet mode or not.
+
+    Args:
+      quiet_mode: boolean, when True the tool is in quiet mode.
+    """
+    self._quiet_mode = quiet_mode
+
+  def SetUseZeroMQ(self, use_zeromq=False):
+    """Sets whether the tool is using ZeroMQ for queueing or not.
+
+    Args:
+      use_zeromq: boolean, when True the tool will use ZeroMQ for queuing.
+    """
+    self._use_zeromq = use_zeromq
+
 
 class PsortAnalysisReportQueueConsumer(queue.ItemQueueConsumer):
   """Class that implements an analysis report queue consumer for psort."""
@@ -533,7 +596,7 @@ class PsortAnalysisReportQueueConsumer(queue.ItemQueueConsumer):
       queue_object: the queue object (instance of Queue).
       storage_file: the storage file (instance of StorageFile).
       filter_string: the filter string.
-      preferred_encoding: optional preferred encoding. The default is "utf-8".
+      preferred_encoding: optional preferred encoding.
     """
     super(PsortAnalysisReportQueueConsumer, self).__init__(queue_object)
     self._filter_string = filter_string
