@@ -12,11 +12,15 @@ PROCESS_TYPE_COLLECTOR = u'collector'
 PROCESS_TYPE_STORAGE_WRITER = u'storage_writer'
 PROCESS_TYPE_WORKER = u'worker'
 
+PROCESSING_STATUS_ABORTED = u'aborted'
 PROCESSING_STATUS_COMPLETED = u'completed'
 PROCESSING_STATUS_ERROR = u'error'
+PROCESSING_STATUS_HASHING = u'hashing'
 PROCESSING_STATUS_INITIALIZED = u'initialized'
 PROCESSING_STATUS_KILLED = u'killed'
+PROCESSING_STATUS_PARSING = u'parsing'
 PROCESSING_STATUS_RUNNING = u'running'
+PROCESSING_STATUS_WAITING = u'waiting'
 
 RESERVED_VARIABLE_NAMES = frozenset([
     u'body',
@@ -38,6 +42,7 @@ RESERVED_VARIABLE_NAMES = frozenset([
     u'store_index',
     u'store_number',
     u'tag',
+    u'text_prepend',
     u'timestamp',
     u'timestamp_desc',
     u'timezone',
