@@ -1,37 +1,38 @@
 # -*- coding: utf-8 -*-
-"""This file contains functions and variables used for time manipulations.
+"""Time manipulation functions and variables.
 
-This file should contain common methods that can be used in Plaso to convert
-timestamps in various formats into the standard micro seconds precision integer
-Epoch UTC time that is used internally to store timestamps in Plaso.
+This module contain common methods that can be used to convert timestamps
+from various formats into number of micro seconds since January 1, 1970,
+00:00:00 UTC that is used internally to store timestamps.
 
-The file can also contain common functions to change the default timestamp into
-a more human readable one.
+It also contains various functions to represent timestamps in a more
+human readable form.
 """
 
 import calendar
 import datetime
-import dateutil.parser
 import logging
 import time
 import pytz
 
+import dateutil.parser
+
 from plaso.lib import errors
 
 
 MONTH_DICT = {
-    'jan': 1,
-    'feb': 2,
-    'mar': 3,
-    'apr': 4,
-    'may': 5,
-    'jun': 6,
-    'jul': 7,
-    'aug': 8,
-    'sep': 9,
-    'oct': 10,
-    'nov': 11,
-    'dec': 12}
+    u'jan': 1,
+    u'feb': 2,
+    u'mar': 3,
+    u'apr': 4,
+    u'may': 5,
+    u'jun': 6,
+    u'jul': 7,
+    u'aug': 8,
+    u'sep': 9,
+    u'oct': 10,
+    u'nov': 11,
+    u'dec': 12}
 
 
 class Timestamp(object):
@@ -68,7 +69,7 @@ class Timestamp(object):
   # The number of micro seconds per second
   MICRO_SECONDS_PER_SECOND = 1000000
 
-  # The multiplication factor to change milli seconds to micro seconds.
+  # The multiplication factor to change milliseconds to micro seconds.
   MILLI_SECONDS_TO_MICRO_SECONDS = 1000
 
   # The difference between Jan 1, 1980 and Jan 1, 1970 in seconds.
@@ -77,9 +78,12 @@ class Timestamp(object):
   # The difference between Jan 1, 1601 and Jan 1, 1970 in micro seconds
   WEBKIT_TIME_TO_POSIX_BASE = 11644473600L * 1000000
 
-  # The difference between Jan 1, 1601 and Jan 1, 1970 in 100s of nanoseconds.
+  # The difference between Jan 1, 1601 and Jan 1, 1970 in 100 nanoseconds.
   FILETIME_TO_POSIX_BASE = 11644473600L * 10000000
 
+  # The difference between Nov 10, 1582 and Jan 1, 1970 in 100 nanoseconds.
+  UUID_TIME_TO_POSIX_BASE = 12219292800L * 10000000
+
   # The number of seconds between January 1, 1904 and Jan 1, 1970.
   # Value confirmed with sleuthkit:
   #  http://svn.sleuthkit.org/repos/sleuthkit/trunk/tsk3/fs/tsk_hfs.h
@@ -109,7 +113,8 @@ class Timestamp(object):
                    The default timezone is UTC.
 
     Returns:
-      An integer containing the timestamp.
+      The timestamp which is an integer containing the number of micro seconds
+      since January 1, 1970, 00:00:00 UTC.
 
     Raises:
       ValueError: if the time string is invalid or not supported.
@@ -249,19 +254,21 @@ class Timestamp(object):
     """Copies the timestamp to a datetime object.
 
     Args:
-      timestamp: An integer containing the timestamp.
+      timestamp: The timestamp which is an integer containing the number
+                 of micro seconds since January 1, 1970, 00:00:00 UTC.
       timezone: The timezone (pytz.timezone) object.
       raise_error: Boolean that if set to True will not absorb an OverflowError
                    if the timestamp is out of bounds. By default there will be
                    no error raised.
 
     Returns:
-      A datetime object.
+      A datetime object (instance of datetime.datetime). A datetime object of
+      January 1, 1970 00:00:00 UTC is returned on error if raises_error is
+      not set.
 
     Raises:
-      OverflowError: If raises_error is set to True and an OverflowError error
-                     occurs. Otherwise the error is absorbed and a datetime
-                     object from the beginning of UNIX Epoch is returned.
+      OverflowError: If raises_error is set to True and an overflow error
+                     occurs.
     """
     datetime_object = datetime.datetime(1970, 1, 1, 0, 0, 0, 0, tzinfo=pytz.UTC)
     try:
@@ -270,10 +277,10 @@ class Timestamp(object):
     except OverflowError as exception:
       if raise_error:
         raise
-      else:
-        logging.error((
-            u'Unable to copy {0:d} to a datetime object with error: '
-            u'{1:s}').format(timestamp, exception))
+
+      logging.error((
+          u'Unable to copy {0:d} to a datetime object with error: '
+          u'{1:s}').format(timestamp, exception))
 
     return datetime_object
 
@@ -282,9 +289,9 @@ class Timestamp(object):
     """Copies the timestamp to an ISO 8601 formatted string.
 
     Args:
-      timestamp: An integer containing the timestamp.
+      timestamp: The timestamp which is an integer containing the number
+                 of micro seconds since January 1, 1970, 00:00:00 UTC.
       timezone: Optional timezone (instance of pytz.timezone).
-                The default is UTC.
       raise_error: Boolean that if set to True will not absorb an OverflowError
                    if the timestamp is out of bounds. By default there will be
                    no error raised.
@@ -301,10 +308,12 @@ class Timestamp(object):
     """Converts microsecond timestamps to POSIX timestamps.
 
     Args:
-      timestamp: An integer containing the microsecond timestamp.
+      timestamp: The timestamp which is an integer containing the number
+                 of micro seconds since January 1, 1970, 00:00:00 UTC.
 
     Returns:
-      An integer value containing the timestamp.
+      The timestamp which is an integer containing the number of seconds
+      since January 1, 1970, 00:00:00 UTC.
     """
     return timestamp // cls.MICRO_SECONDS_PER_SECOND
 
@@ -378,7 +387,8 @@ class Timestamp(object):
       cocoa_time: The timestamp in Cocoa format.
 
     Returns:
-      An integer containing the timestamp or 0 on error.
+      The timestamp which is an integer containing the number of micro seconds
+      since January 1, 1970, 00:00:00 UTC or 0 on error.
     """
     return cls.FromPosixTime(cocoa_time + cls.COCOA_TIME_TO_POSIX_BASE)
 
@@ -396,7 +406,8 @@ class Timestamp(object):
       delphi_time: The timestamp in Delphi format.
 
     Returns:
-      An integer containing the timestamp or 0 on error.
+      The timestamp which is an integer containing the number of micro seconds
+      since January 1, 1970, 00:00:00 UTC or 0 on error.
     """
     posix_time = (delphi_time - cls.DELPHI_TIME_TO_POSIX_BASE) * 86400.0
     if (posix_time < cls.TIMESTAMP_MIN_SECONDS or
@@ -425,7 +436,8 @@ class Timestamp(object):
       fat_date_time: The 32-bit FAT date time.
 
     Returns:
-      An integer containing the timestamp or 0 on error.
+      The timestamp which is an integer containing the number of micro seconds
+      since January 1, 1970, 00:00:00 UTC or 0 on error.
     """
     number_of_seconds = cls.FAT_DATE_TO_POSIX_BASE
 
@@ -471,7 +483,8 @@ class Timestamp(object):
       filetime: The 64-bit FILETIME timestamp.
 
     Returns:
-      An integer containing the timestamp or 0 on error.
+      The timestamp which is an integer containing the number of micro seconds
+      since January 1, 1970, 00:00:00 UTC or 0 on error.
     """
     # TODO: Add a handling for if the timestamp equals to zero.
     if filetime < 0:
@@ -494,10 +507,10 @@ class Timestamp(object):
       timezone: The timezone object of the system's local time.
       is_dst: A boolean to indicate the timestamp is corrected for daylight
               savings time (DST) only used for the DST transition period.
-              The default is false.
 
     Returns:
-      An integer containing the timestamp or 0 on error.
+      The timestamp which is an integer containing the number of micro seconds
+      since January 1, 1970, 00:00:00 UTC or 0 on error.
     """
     timestamp_local = cls.FromHfsPlusTime(hfs_time)
     return cls.LocaltimeToUTC(timestamp_local, timezone, is_dst)
@@ -514,7 +527,8 @@ class Timestamp(object):
       hfs_time: The timestamp in HFS+ format.
 
     Returns:
-      An integer containing the timestamp or 0 on error.
+      The timestamp which is an integer containing the number of micro seconds
+      since January 1, 1970, 00:00:00 UTC or 0 on error.
     """
     return cls.FromPosixTime(hfs_time - cls.HFSTIME_TO_POSIX_BASE)
 
@@ -532,7 +546,8 @@ class Timestamp(object):
       java_time: The Java Timestamp.
 
     Returns:
-      An integer containing the timestamp or 0 on error.
+      The timestamp which is an integer containing the number of micro seconds
+      since January 1, 1970, 00:00:00 UTC or 0 on error.
     """
     return java_time * cls.MILLI_SECONDS_TO_MICRO_SECONDS
 
@@ -547,7 +562,8 @@ class Timestamp(object):
       posix_time: The POSIX timestamp.
 
     Returns:
-      An integer containing the timestamp or 0 on error.
+      The timestamp which is an integer containing the number of micro seconds
+      since January 1, 1970, 00:00:00 UTC or 0 on error.
     """
     if (posix_time < cls.TIMESTAMP_MIN_SECONDS or
         posix_time > cls.TIMESTAMP_MAX_SECONDS):
@@ -566,7 +582,8 @@ class Timestamp(object):
       microsecond: The microseconds to add to the timestamp.
 
     Returns:
-      An integer containing the timestamp or 0 on error.
+      The timestamp which is an integer containing the number of micro seconds
+      since January 1, 1970, 00:00:00 UTC or 0 on error.
     """
     timestamp = cls.FromPosixTime(posix_time)
     if not timestamp:
@@ -575,13 +592,57 @@ class Timestamp(object):
 
   @classmethod
   def FromPythonDatetime(cls, datetime_object):
-    """Converts a Python datetime object into a timestamp."""
+    """Converts a Python datetime object into a timestamp.
+
+    Args:
+      datetime_object: The datetime object (instance of datetime.datetime).
+
+    Returns:
+      The timestamp which is an integer containing the number of micro seconds
+      since January 1, 1970, 00:00:00 UTC or 0 on error.
+    """
     if not isinstance(datetime_object, datetime.datetime):
       return 0
 
-    posix_epoch = int(calendar.timegm(datetime_object.utctimetuple()))
-    epoch = cls.FromPosixTime(posix_epoch)
-    return epoch + datetime_object.microsecond
+    posix_time = int(calendar.timegm(datetime_object.utctimetuple()))
+    return cls.FromPosixTime(posix_time) + datetime_object.microsecond
+
+  @classmethod
+  def FromRFC2579Datetime(
+      cls, year, month, day, hour, minutes, seconds, deciseconds,
+      direction_from_utc, hours_from_utc, minutes_from_utc):
+    """Converts values from an RFC2579 time to a timestamp.
+
+    See https://tools.ietf.org/html/rfc2579.
+
+    Args:
+      year: An integer representing the year.
+      month: An integer between 1 and 12.
+      day: An integer representing the number of day in the month.
+      hour: An integer representing the hour, 0 <= hour < 24.
+      minutes: An integer, 0 <= minute < 60.
+      seconds: An integer, 0 <= second < 60.
+      deciseconds: An integer, 0 <= deciseconds < 10
+      direction_from_utc: An ascii character, either '+' or '-'.
+      hours_from_utc: An integer representing the number of hours the time is
+                      offset from UTC.
+      minutes_from_utc: An integer representing the number of seconds the time
+                        is offset from UTC.
+
+    Returns:
+      The timestamp which is an integer containing the number of micro seconds
+      since January 1, 1970, 00:00:00 UTC or 0 on error.
+
+    Raises:
+      TimestampError: if the timestamp cannot be created from the time parts.
+    """
+    microseconds = deciseconds * 100000
+    utc_offset_minutes = (hours_from_utc * 60) + minutes_from_utc
+    if direction_from_utc == u'-':
+      utc_offset_minutes = -utc_offset_minutes
+    timezone = pytz.FixedOffset(utc_offset_minutes)
+    return cls.FromTimeParts(
+        year, month, day, hour, minutes, seconds, microseconds, timezone)
 
   @classmethod
   def FromTimeParts(
@@ -597,31 +658,33 @@ class Timestamp(object):
       minutes: An integer, 0 <= minute < 60.
       seconds: An integer, 0 <= second < 60.
       microseconds: Optional number of microseconds ranging from:
-                    0 <= microsecond < 1000000. The default is 0.
+                    0 <= microsecond < 1000000.
       timezone: Optional timezone (instance of pytz.timezone).
-                The default is UTC.
 
     Returns:
-      An integer containing the timestamp or 0 on error.
+      The timestamp which is an integer containing the number of micro seconds
+      since January 1, 1970, 00:00:00 UTC or 0 on error.
+
+    Raises:
+      TimestampError: if the timestamp cannot be created from the time parts.
     """
     try:
       date = datetime.datetime(
           year, month, day, hour, minutes, seconds, microseconds)
     except ValueError as exception:
-      logging.warning((
+      raise errors.TimestampError((
           u'Unable to create timestamp from {0:04d}-{1:02d}-{2:02d} '
           u'{3:02d}:{4:02d}:{5:02d}.{6:06d} with error: {7:s}').format(
               year, month, day, hour, minutes, seconds, microseconds,
               exception))
-      return 0
 
     if isinstance(timezone, basestring):
       timezone = pytz.timezone(timezone)
 
     date_use = timezone.localize(date)
-    epoch = int(calendar.timegm(date_use.utctimetuple()))
+    posix_time = int(calendar.timegm(date_use.utctimetuple()))
 
-    return cls.FromPosixTime(epoch) + microseconds
+    return cls.FromPosixTime(posix_time) + microseconds
 
   @classmethod
   def FromTimeString(
@@ -645,7 +708,8 @@ class Timestamp(object):
                 is used when the timezone cannot be determined from the string.
 
     Returns:
-      An integer containing the timestamp.
+      The timestamp which is an integer containing the number of micro seconds
+      since January 1, 1970, 00:00:00 UTC or 0 on error.
 
     Raises:
       TimestampError: if the time string could not be parsed.
@@ -669,6 +733,29 @@ class Timestamp(object):
 
     return cls.FromPythonDatetime(datetime_object)
 
+  @classmethod
+  def FromUUIDTime(cls, uuid_time):
+    """Converts a UUID verion 1 time into a timestamp.
+
+    The UUID version 1 time is a 60-bit value containing:
+      100th nano seconds since 1582-10-15 00:00:00
+
+    Args:
+      uuid_time: The 60-bit UUID version 1 timestamp.
+
+    Returns:
+      The timestamp which is an integer containing the number of micro seconds
+      since January 1, 1970, 00:00:00 UTC or 0 on error.
+    """
+    # TODO: Add a handling for if the timestamp equals to zero.
+    if uuid_time < 0:
+      return 0
+    timestamp = (uuid_time - cls.UUID_TIME_TO_POSIX_BASE) / 10
+
+    if timestamp > cls.TIMESTAMP_MAX_MICRO_SECONDS:
+      return 0
+    return timestamp
+
   @classmethod
   def FromWebKitTime(cls, webkit_time):
     """Converts a WebKit time into a timestamp.
@@ -680,7 +767,8 @@ class Timestamp(object):
       webkit_time: The 64-bit WebKit time timestamp.
 
     Returns:
-      An integer containing the timestamp or 0 on error.
+      The timestamp which is an integer containing the number of micro seconds
+      since January 1, 1970, 00:00:00 UTC or 0 on error.
     """
     if webkit_time < (cls.TIMESTAMP_MIN_MICRO_SECONDS +
                       cls.WEBKIT_TIME_TO_POSIX_BASE):
@@ -689,7 +777,12 @@ class Timestamp(object):
 
   @classmethod
   def GetNow(cls):
-    """Retrieves the current time (now) as a timestamp in UTC."""
+    """Retrieves the current time (now) as a timestamp in UTC.
+
+    Returns:
+      The timestamp which is an integer containing the number of micro seconds
+      since January 1, 1970, 00:00:00 UTC.
+    """
     time_elements = time.gmtime()
     return calendar.timegm(time_elements) * 1000000
 
@@ -697,8 +790,7 @@ class Timestamp(object):
   def IsLeapYear(cls, year):
     """Determines if a year is a leap year.
 
-       A leap year is dividable by 4 and not by 100 or by 400
-       without a remainder.
+       A leap year is divisible by 4 and not by 100 or by 400.
 
     Args:
       year: The year as in 1970.
@@ -713,14 +805,15 @@ class Timestamp(object):
     """Converts the timestamp in localtime of the timezone to UTC.
 
     Args:
-      timestamp: An integer containing the timestamp.
+      timestamp: The timestamp which is an integer containing the number
+                 of micro seconds since January 1, 1970, 00:00:00 UTC.
       timezone: The timezone (pytz.timezone) object.
       is_dst: A boolean to indicate the timestamp is corrected for daylight
               savings time (DST) only used for the DST transition period.
-              The default is false.
 
     Returns:
-      An integer containing the timestamp or 0 on error.
+      The timestamp which is an integer containing the number of micro seconds
+      since January 1, 1970, 00:00:00 UTC or 0 on error.
     """
     if timezone and timezone != pytz.UTC:
       datetime_object = (
