@@ -18,6 +18,7 @@ from plaso.engine import collector
 from plaso.engine import engine
 from plaso.engine import queue
 from plaso.engine import worker
+from plaso.engine import zeromq_queue
 from plaso.lib import definitions
 from plaso.lib import errors
 from plaso.multi_processing import process_info
@@ -41,7 +42,7 @@ class MultiProcessBaseProcess(multiprocessing.Process):
     Args:
       process_type: the process type.
       enable_sigsegv_handler: optional boolean value to indicate the SIGSEGV
-                              handler should be enabled. The default is False.
+                              handler should be enabled.
       kwargs: keyword arguments to pass to multiprocessing.Process.
     """
     super(MultiProcessBaseProcess, self).__init__(**kwargs)
@@ -201,7 +202,7 @@ class MultiProcessBaseProcess(multiprocessing.Process):
 
     self._pid = os.getpid()
 
-    # We need to set the is running statue explictily to True in case
+    # We need to set the is running status explicitly to True in case
     # the process completes before the engine is able to determine
     # the status of the process, e.g. in the unit tests.
     self._status_is_running = True
@@ -242,10 +243,9 @@ class MultiProcessCollectorProcess(MultiProcessBaseProcess):
       path_spec_queue: the path specification queue object (instance of
                        MultiProcessingQueue).
       filter_find_specs: Optional list of filter find specifications (instances
-                         of dfvfs.FindSpec). The default is None.
+                         of dfvfs.FindSpec).
       include_directory_stat: Optional boolean value to indicate whether
                               directory stat information should be collected.
-                              The default is True.
       kwargs: keyword arguments to pass to multiprocessing.Process.
     """
     super(MultiProcessCollectorProcess, self).__init__(
@@ -255,9 +255,9 @@ class MultiProcessCollectorProcess(MultiProcessBaseProcess):
     self._collector = collector.Collector(
         path_spec_queue, resolver_context=resolver_context)
     self._path_spec_queue = path_spec_queue
+    self._path_spec_queue_port = None
     self._source_path_specs = source_path_specs
     self._stop_collector_event = stop_collector_event
-
     self._collector.SetCollectDirectoryMetadata(include_directory_stat)
 
     if filter_find_specs:
@@ -266,6 +266,7 @@ class MultiProcessCollectorProcess(MultiProcessBaseProcess):
   def _GetStatus(self):
     """Returns a status dictionary."""
     status = self._collector.GetStatus()
+    status[u'path_spec_queue_port'] = self._path_spec_queue_port
     self._status_is_running = status.get(u'is_running', False)
     return status
 
@@ -273,8 +274,15 @@ class MultiProcessCollectorProcess(MultiProcessBaseProcess):
     """The main loop."""
     logging.debug(u'Collector (PID: {0:d}) started'.format(self._pid))
 
+    # The ZeroMQ backed queue must be started first, so we can save its port.
+    if isinstance(self._path_spec_queue, zeromq_queue.ZeroMQQueue):
+      self._path_spec_queue.name = u'Collector pathspec'
+      self._path_spec_queue.Open()
+      self._path_spec_queue_port = self._path_spec_queue.port
+
     abort = False
     try:
+      logging.debug(u'Collector starting collection.')
       self._collector.Collect(self._source_path_specs)
 
     except Exception as exception:
@@ -283,6 +291,8 @@ class MultiProcessCollectorProcess(MultiProcessBaseProcess):
       logging.exception(exception)
       abort = True
 
+    self._path_spec_queue.Close()
+
     logging.debug(u'Collector (PID: {0:d}) stopped'.format(self._pid))
 
     # The collector will typically collect the path specifications and
@@ -291,9 +301,10 @@ class MultiProcessCollectorProcess(MultiProcessBaseProcess):
     # is necessary since the queue uses a pipe under the hood. However
     # this behavior causes unwanted side effect for the abort code paths
     # hence we have the process wait for the stop process event and then
-    # close the queue with out the separate blocking thread.
+    # close the queue without the separate blocking thread.
 
     if not abort:
+      logging.debug(u'Collector waiting for stop event.')
       self._stop_collector_event.wait()
 
     self._path_spec_queue.Close(abort=True)
@@ -317,34 +328,56 @@ class MultiProcessEngine(engine.BaseEngine):
   _PROCESS_JOIN_TIMEOUT = 5.0
   _PROCESS_TERMINATION_SLEEP = 0.5
 
+  _QUEUE_START_WAIT = 0.3
+  _QUEUE_START_WAIT_ATTEMPTS_MAXIMUM = 3
+
   # Note that on average Windows seems to require a bit longer wait
   # than 5 seconds.
   _RPC_SERVER_TIMEOUT = 8.0
+  _MAXIMUM_RPC_ERRORS = 10
 
   _STATUS_CHECK_SLEEP = 1.5
 
   _WORKER_PROCESSES_MINIMUM = 2
   _WORKER_PROCESSES_MAXIMUM = 15
 
-  def __init__(self, maximum_number_of_queued_items=0):
+  def __init__(self, maximum_number_of_queued_items=0, use_zeromq=False):
     """Initialize the multi-process engine object.
 
     Args:
-      maximum_number_of_queued_items: The maximum number of queued items.
+      maximum_number_of_queued_items: Optional maximum number of queued items.
                                       The default is 0, which represents
                                       no limit.
+      use_zeromq: Optional boolean to indicate whether to use ZeroMQ for
+                  queuing. If False, the multiprocessing queue will be used.
     """
-    path_spec_queue = MultiProcessingQueue(
-        maximum_number_of_queued_items=maximum_number_of_queued_items)
-    event_object_queue = MultiProcessingQueue(
-        maximum_number_of_queued_items=maximum_number_of_queued_items)
-    parse_error_queue = MultiProcessingQueue(
-        maximum_number_of_queued_items=maximum_number_of_queued_items)
+    self._use_zeromq = use_zeromq
+    if self._use_zeromq:
+      path_spec_outbound_queue = zeromq_queue.ZeroMQBufferedReplyBindQueue(
+          delay_open=True, name=u'Pathspec inbound', buffer_timeout_seconds=300)
+      path_spec_queue = path_spec_outbound_queue
+      event_object_inbound_queue = zeromq_queue.ZeroMQBufferedPullBindQueue(
+          delay_open=True, name=u'Event object inbound', timeout_seconds=60,
+          buffer_max_size=10000, linger_seconds=0)
+      event_object_queue = event_object_inbound_queue
+      parse_error_inbound_queue = zeromq_queue.ZeroMQBufferedPullBindQueue(
+          delay_open=True, name=u'Parse error inbound', timeout_seconds=60,
+          buffer_max_size=10000, linger_seconds=0)
+      parse_error_queue = parse_error_inbound_queue
+    else:
+      path_spec_queue = MultiProcessingQueue(
+          maximum_number_of_queued_items=maximum_number_of_queued_items)
+      event_object_queue = MultiProcessingQueue(
+          maximum_number_of_queued_items=maximum_number_of_queued_items)
+      parse_error_queue = MultiProcessingQueue(
+          maximum_number_of_queued_items=maximum_number_of_queued_items)
 
     super(MultiProcessEngine, self).__init__(
         path_spec_queue, event_object_queue, parse_error_queue)
 
     self._enable_sigsegv_handler = False
+    self._event_object_queue_port = None
+    self._extraction_complete_event = None
     self._filter_find_specs = None
     self._filter_object = None
     self._hasher_names_string = None
@@ -352,14 +385,16 @@ class MultiProcessEngine(engine.BaseEngine):
     self._last_worker_number = 0
     self._mount_path = None
     self._number_of_extraction_workers = 0
+    self._parse_error_queue_port = None
     self._parser_filter_string = None
+    self._path_spec_queue_port = None
     self._process_archive_files = False
     self._process_information_per_pid = {}
     self._processes_per_pid = {}
     self._rpc_clients_per_pid = {}
     self._rpc_errors_per_pid = {}
-    self._storage_writer_completed = False
     self._show_memory_usage = False
+    self._storage_writer_complete_event = None
     self._stop_collector_event = None
     self._text_prepend = None
 
@@ -409,6 +444,10 @@ class MultiProcessEngine(engine.BaseEngine):
     for pid in iter(self._process_information_per_pid.keys()):
       self._CheckProcessStatus(pid)
 
+    extraction_completed = self._processing_status.GetExtractionCompleted()
+    if extraction_completed:
+      logging.debug(u'Extraction completed.')
+
     processing_completed = self._processing_status.GetProcessingCompleted()
     if processing_completed and not self._processing_status.error_detected:
       logging.debug(u'Processing completed.')
@@ -419,40 +458,44 @@ class MultiProcessEngine(engine.BaseEngine):
       self._processing_status.error_detected = True
       raise errors.EngineAbort(u'Workers idle for too long')
 
-    return processing_completed
+    return extraction_completed, processing_completed
 
   def _CheckProcessStatus(self, pid):
-    """Check status of a specific monitored process.
+    """Check status of a specific monitored process and take action on it.
+
+    This method examines the status of a process, and then terminates it,
+    signals other processes to terminate or starts a replacement process as
+    appropriate.
 
     Args:
       pid: The process ID (PID).
 
     Raises:
       EngineAbort: when the collector or storage worker process
-                   unexpectedly terminated.
+                   unexpectedly terminates.
       KeyError: if the process is not registered with the engine.
     """
+    # TODO: Refactor this method, simplify and separate concerns (monitoring
+    # vs management).
     self._RaiseIfNotRegistered(pid)
 
     process = self._processes_per_pid[pid]
 
-    process_is_alive = process.is_alive()
-    if process_is_alive:
-      rpc_client = self._rpc_clients_per_pid.get(pid, None)
-      process_status = rpc_client.CallFunction()
+    process_status = self._GetProcessStatus(process)
+    if process_status is None:
+      process_is_alive = False
     else:
-      process_status = None
+      process_is_alive = True
 
     if isinstance(process_status, dict):
       self._rpc_errors_per_pid[pid] = 0
-
       status_indicator = process_status.get(u'processing_status', None)
 
     else:
       rpc_errors = self._rpc_errors_per_pid.get(pid, 0) + 1
       self._rpc_errors_per_pid[pid] = rpc_errors
 
-      if rpc_errors > 10:
+      if rpc_errors > self._MAXIMUM_RPC_ERRORS:
         process_is_alive = False
 
       if process_is_alive:
@@ -480,14 +523,17 @@ class MultiProcessEngine(engine.BaseEngine):
 
     self._UpdateProcessingStatus(pid, process_status)
 
-    if status_indicator not in [
+    if status_indicator not in (
         definitions.PROCESSING_STATUS_COMPLETED,
         definitions.PROCESSING_STATUS_INITIALIZED,
-        definitions.PROCESSING_STATUS_RUNNING]:
+        definitions.PROCESSING_STATUS_RUNNING,
+        definitions.PROCESSING_STATUS_PARSING,
+        definitions.PROCESSING_STATUS_HASHING):
 
       logging.error(
-          u'Process {0:s} (PID: {1:d}) is not functioning correctly.'.format(
-              process.name, pid))
+          (u'Process {0:s} (PID: {1:d}) is not functioning correctly. '
+           u'Status code {2!s}.').format(
+               process.name, pid, status_indicator))
 
       self._processing_status.error_detected = True
 
@@ -505,16 +551,63 @@ class MultiProcessEngine(engine.BaseEngine):
       self._StartMonitoringProcess(worker_process.pid)
 
     elif status_indicator == definitions.PROCESSING_STATUS_COMPLETED:
-      logging.debug((
-          u'Process {0:s} (PID: {1:d}) has completed its processing. '
-          u'Total of {2:d} events extracted').format(
-              process.name, pid, process_status.get(u'number_of_events', 0)))
+      if process.type == definitions.PROCESS_TYPE_WORKER:
+        number_of_events = process_status.get(u'number_of_events', 0)
+        number_of_pathspecs = process_status.get(
+            u'consumed_number_of_path_specs', 0)
+        logging.debug((
+            u'Process {0:s} (PID: {1:d}) has completed its processing. '
+            u'Total of {2:d} events extracted from {3:d} pathspecs').format(
+                process.name, pid, number_of_events, number_of_pathspecs))
+      elif process.type == definitions.PROCESS_TYPE_COLLECTOR:
+        number_of_pathspecs = process_status.get(
+            u'produced_number_of_path_specs', 0)
+        logging.debug((
+            u'Process {0:s} (PID: {1:d}) has completed its processing. '
+            u'Total of {2:d} pathspecs extracted').format(
+                process.name, pid, number_of_pathspecs))
+      elif process.type == definitions.PROCESS_TYPE_STORAGE_WRITER:
+        logging.debug((
+            u'Process {0:s} (PID: {1:d}) has completed its processing. '
+            u'Total of {2:d} events written').format(
+                process.name, pid, process_status.get(u'number_of_events', 0)))
 
       self._StopMonitoringProcess(pid)
 
     elif self._show_memory_usage:
       self._LogMemoryUsage(pid)
 
+  def _CloseStorageZeroMQQueues(self):
+    """Closes all ZeroMQ queues being used by the storage writer."""
+    logging.debug(u'Closing ZeroMQ storage queues.')
+    for port in (self._event_object_queue_port, self._parse_error_queue_port):
+      terminate_queue = zeromq_queue.ZeroMQPushConnectQueue(
+          linger_seconds=0, port=port, timeout_seconds=0, name=u'Termination')
+      terminate_queue.PushItem(queue.QueueAbort(), block=False)
+      terminate_queue.Close()
+
+  def _CollectorQueueHasBound(self):
+    """Checks if the collector has bound to a queue."""
+    return self._path_spec_queue_port is not None
+
+  def _GetProcessStatus(self, process):
+    """Queries a process to determine its status.
+
+    Args:
+      process: The handle to the process to query (instance of
+               MultiProcessBaseProcess).
+
+    Returns:
+      A dictionary containing the process status.
+    """
+    process_is_alive = process.is_alive()
+    if process_is_alive:
+      rpc_client = self._rpc_clients_per_pid.get(process.pid, None)
+      process_status = rpc_client.CallFunction()
+    else:
+      process_status = None
+    return process_status
+
   def _KillProcess(self, pid):
     """Issues a SIGKILL or equivalent to the process.
 
@@ -565,9 +658,32 @@ class MultiProcessEngine(engine.BaseEngine):
       MultiProcessEventExtractionWorkerProcess).
     """
     process_name = u'Worker_{0:02d}'.format(self._last_worker_number)
+
+    if self._use_zeromq:
+      path_spec_queue = zeromq_queue.ZeroMQRequestConnectQueue(
+          delay_open=True, port=self._path_spec_queue_port,
+          timeout_seconds=2,
+          name=u'{0:s} pathspec'.format(process_name),
+          linger_seconds=0)
+      # The high water mark is set to 2 for these queues so that data
+      # doesn't build up in the workers, to prevent data loss in the case
+      # that they crash.
+      parse_error_queue = zeromq_queue.ZeroMQPushConnectQueue(
+          delay_open=True, port=self._parse_error_queue_port,
+          name=u'{0:s} parse error'.format(process_name),
+          timeout_seconds=20, maximum_items=2)
+      event_object_queue = zeromq_queue.ZeroMQPushConnectQueue(
+          delay_open=True, port=self._event_object_queue_port,
+          name=u'{0:s} event object'.format(process_name),
+          timeout_seconds=20, maximum_items=2)
+    else:
+      parse_error_queue = self._parse_error_queue
+      path_spec_queue = self._path_spec_queue
+      event_object_queue = self.event_object_queue
+
     worker_process = MultiProcessEventExtractionWorkerProcess(
-        self._path_spec_queue, self.event_object_queue,
-        self._parse_error_queue, self.knowledge_base, self._last_worker_number,
+        path_spec_queue, event_object_queue,
+        parse_error_queue, self.knowledge_base, self._last_worker_number,
         enable_debug_output=self._enable_debug_output,
         enable_profiling=self._enable_profiling,
         enable_sigsegv_handler=self._enable_sigsegv_handler,
@@ -592,7 +708,11 @@ class MultiProcessEngine(engine.BaseEngine):
     Args:
       abort: optional boolean to indicate the stop is issued on abort.
     """
+    logging.debug(u'Stopping extraction processes.')
     self._StopProcessMonitoring()
+    # We can stop the collector and storage writer now.
+    self._stop_collector_event.set()
+    self._extraction_complete_event.set()
 
     # Note that multiprocessing.Queue is very sensitive regarding
     # blocking on either a get or a put. So we try to prevent using
@@ -602,36 +722,49 @@ class MultiProcessEngine(engine.BaseEngine):
       # Signal all the processes to abort.
       self._AbortTerminate()
 
-      # Wake the processes to make sure that they are not blocking
-      # waiting for the queue not to be full.
+    # Wake the processes to make sure that they are not blocking
+    # waiting for the queue not to be full.
+    if self._use_zeromq:
+      self._CloseStorageZeroMQQueues()
+    else:
+      logging.debug(u'Emptying queues.')
       self._path_spec_queue.Empty()
       self.event_object_queue.Empty()
       self._parse_error_queue.Empty()
 
-    # On abort we need to have the collector process get a SIGTERM first.
-    self._stop_collector_event.set()
+      # Wake the processes to make sure that they are not blocking
+      # waiting for new items.
+      for _ in range(self._number_of_extraction_workers):
+        self._path_spec_queue.PushItem(queue.QueueAbort(), block=False)
+      self.event_object_queue.PushItem(queue.QueueAbort(), block=False)
 
-    # Wake the processes to make sure that they are not blocking
-    # waiting for new items.
-    for _ in range(self._number_of_extraction_workers):
-      self._path_spec_queue.PushItem(queue.QueueAbort(), block=False)
+      # TODO: The following line is commented out as a work around for
+      # infinite blocking wait in storage writer process. Fix this by
+      # using a phased processing approach. For now we remove the parse
+      # error queue from the storage writer.
+      # self._parse_error_queue.PushItem(queue.QueueAbort(), block=False)
 
-    self.event_object_queue.PushItem(queue.QueueAbort(), block=False)
-    # TODO: enable this when the parse error queue consumer is operational.
-    # self._parse_error_queue.PushItem(queue.QueueAbort(), block=False)
-
-    # Try terminating the processes in the normal way.
+    # Try waiting for the processes to exit normally.
     self._AbortJoin(timeout=self._PROCESS_JOIN_TIMEOUT)
 
+    # The storage writer process sometimes needs some additional time to
+    # close the storage file.
+    while not abort and not self._storage_writer_complete_event.is_set():
+      logging.info(u'Waiting for storage writer to close storage file.')
+      time.sleep(self._STATUS_CHECK_SLEEP)
+
     if not abort:
       # Check if the processes are still alive and terminate them if necessary.
       self._AbortTerminate()
       self._AbortJoin(timeout=self._PROCESS_JOIN_TIMEOUT)
 
-    # Set abort to True to stop queue.join_thread() from blocking.
-    self._path_spec_queue.Close(abort=True)
-    self.event_object_queue.Close(abort=True)
-    self._parse_error_queue.Close(abort=True)
+    # For Multiprocessing queues, set abort to True to stop queue.join_thread()
+    # from blocking.
+    extraction_queues = [
+        self._path_spec_queue, self.event_object_queue, self._parse_error_queue]
+    for extraction_queue in extraction_queues:
+      if isinstance(extraction_queue, MultiProcessingQueue):
+        extraction_queue.Close(abort=True)
 
     if abort:
       # Kill any remaining processes, which can be necessary if
@@ -686,6 +819,16 @@ class MultiProcessEngine(engine.BaseEngine):
     self._rpc_clients_per_pid[pid] = rpc_client
     self._process_information_per_pid[pid] = process_info.ProcessInfo(pid)
 
+  def _StorageQueuesBound(self):
+    """Checks if the storage queues are bound to ports.
+
+    Returns:
+      True if both the event_object queue and the parse_error queues are bound
+      to ports. False otherwise.
+    """
+    return (self._event_object_queue_port is not None and
+            self._parse_error_queue_port is not None)
+
   def _RaiseIfNotMonitored(self, pid):
     """Raises if the process is not monitored by the engine.
 
@@ -732,10 +875,6 @@ class MultiProcessEngine(engine.BaseEngine):
 
     self._processes_per_pid[process.pid] = process
 
-  def _StartProcessMonitoring(self):
-    """Starts monitoring the registered processes."""
-    for pid in iter(self._processes_per_pid.keys()):
-      self._StartMonitoringProcess(pid)
 
   def _StopMonitoringProcess(self, pid):
     """Stops monitoring a process.
@@ -857,32 +996,28 @@ class MultiProcessEngine(engine.BaseEngine):
                          dfvfs.PathSpec) to process.
       storage_writer: A storage writer object (instance of BaseStorageWriter).
       enable_sigsegv_handler: optional boolean value to indicate the SIGSEGV
-                              handler should be enabled. The default is False.
+                              handler should be enabled.
       filter_find_specs: Optional list of filter find specifications (instances
-                         of dfvfs.FindSpec). The default is None.
+                         of dfvfs.FindSpec).
       filter_object: Optional filter object (instance of objectfilter.Filter).
-                     The default is None.
       hasher_names_string: Optional comma separated string of names of
-                           hashers to enable enable. The default is None.
+                           hashers to enable enable.
       include_directory_stat: Optional boolean value to indicate whether
                               directory stat information should be collected.
-                              The default is True.
       mount_path: Optional string containing the mount path. The default
                   is None.
       number_of_extraction_workers: Optional number of extraction worker
                                     processes. The default is 0 which means
                                     the function will determine the suitable
                                     number.
-      parser_filter_string: Optional parser filter string. The default is None.
+      parser_filter_string: Optional parser filter string.
       process_archive_files: Optional boolean value to indicate if the worker
                              should scan for file entries inside files.
-                             The default is False.
       status_update_callback: Optional callback function for status updates.
-                              The default is None.
       show_memory_usage: Optional boolean value to indicate memory information
-                         should be included in logging. The default is False.
+                         should be included in logging.
       text_prepend: Optional string that contains the text to prepend to every
-                    event object. The default is None.
+                    event object.
 
     Returns:
       The processing status (instance of ProcessingStatus).
@@ -922,16 +1057,33 @@ class MultiProcessEngine(engine.BaseEngine):
 
     logging.debug(u'Starting processes.')
 
+    # Start the storage writer first, as we need it to start up and bind its
+    # queues to ports before we can start the workers.
+    self._extraction_complete_event = multiprocessing.Event()
+    self._storage_writer_complete_event = multiprocessing.Event()
+
+    if self._use_zeromq:
+      parse_error_queue = self._parse_error_queue
+    else:
+      # TODO: This is a work around for infinite blocking wait in storage
+      # writer process. Fix this by using a phased processing approach.
+      # For now we remove the parse error queue from the storage writer.
+      parse_error_queue = None
+
     storage_writer_process = MultiProcessStorageWriterProcess(
-        self.event_object_queue, self._parse_error_queue, storage_writer,
-        enable_sigsegv_handler=self._enable_sigsegv_handler,
+        self.event_object_queue, self._extraction_complete_event,
+        self._storage_writer_complete_event, parse_error_queue,
+        storage_writer, enable_sigsegv_handler=self._enable_sigsegv_handler,
         name=u'StorageWriter')
     storage_writer_process.start()
     self._RegisterProcess(storage_writer_process)
+    self._StartMonitoringProcess(storage_writer_process.pid)
 
-    for _ in range(number_of_extraction_workers):
-      _ = self._StartExtractionWorkerProcess()
+    if self._use_zeromq:
+      self._GetStorageQueuePorts(storage_writer_process)
 
+   # Next start the collector, as we needs its port number for the workers as
+   # well.
     self._stop_collector_event = multiprocessing.Event()
     collector_process = MultiProcessCollectorProcess(
         self._stop_collector_event, source_path_specs, self._path_spec_queue,
@@ -940,20 +1092,38 @@ class MultiProcessEngine(engine.BaseEngine):
         include_directory_stat=self._include_directory_stat, name=u'Collector')
     collector_process.start()
     self._RegisterProcess(collector_process)
+    self._StartMonitoringProcess(collector_process.pid)
 
-    self._StartProcessMonitoring()
+    if self._use_zeromq:
+      self._GetCollectorStoragePorts(collector_process)
+
+    # Finally, start the workers.
+    for _ in range(number_of_extraction_workers):
+      extraction_process = self._StartExtractionWorkerProcess()
+      self._StartMonitoringProcess(extraction_process.pid)
 
     try:
       logging.debug(u'Processing started.')
 
       time.sleep(self._STATUS_CHECK_SLEEP)
-      while not self._CheckStatus():
+      extraction_completed, processing_completed = self._CheckStatus()
+      while not processing_completed:
         if status_update_callback:
           status_update_callback(self._processing_status)
 
+        if extraction_completed:
+          self._extraction_complete_event.set()
+
         time.sleep(self._STATUS_CHECK_SLEEP)
+        extraction_completed, processing_completed = self._CheckStatus()
+
+      # If the storage writer finishes writing very soon after the workers,
+      # we may not set the event before the storage writer finishes, so set it
+      # explicitly here.
+      self._extraction_complete_event.set()
 
       logging.debug(u'Processing stopped.')
+
     except errors.EngineAbort as exception:
       logging.warning(
           u'Processing aborted with engine error: {0:s}.'.format(exception))
@@ -967,6 +1137,49 @@ class MultiProcessEngine(engine.BaseEngine):
 
     return self._processing_status
 
+  def _GetCollectorStoragePorts(self, collector_process):
+    """Ensures that port number for the collector pathspec queue is captured.
+
+    Args:
+      collector_process: the collector process object (instance of
+                         MultiProcessCollectorProcess).
+
+    Raises:
+      RuntimeError: if the collector is not able to start its queue before the
+                    the timeout is reached.
+    """
+    queue_start_wait_attempts = 0
+    while (not self._CollectorQueueHasBound() and
+           queue_start_wait_attempts < self._QUEUE_START_WAIT_ATTEMPTS_MAXIMUM):
+      status = self._GetProcessStatus(collector_process)
+      self._path_spec_queue_port = status[u'path_spec_queue_port']
+      queue_start_wait_attempts += 1
+      time.sleep(self._QUEUE_START_WAIT)
+    if not self._CollectorQueueHasBound():
+      raise RuntimeError(u'Collector queue did not bind in time.')
+
+  def _GetStorageQueuePorts(self, storage_writer_process):
+    """Ensures that port numbers for the storage writer queues are captured.
+
+    Args:
+      storage_writer_process: the storage writer process object (instance of
+                              MultiProcessStorageWriterProcess).
+
+    Raises:
+      RuntimeError: if the storage writer process is not able to starts its
+                    queues before the timeout is reached.
+    """
+    queue_start_wait_attempts = 0
+    while (not self._StorageQueuesBound() and
+           queue_start_wait_attempts < self._QUEUE_START_WAIT_ATTEMPTS_MAXIMUM):
+      status = self._GetProcessStatus(storage_writer_process)
+      self._event_object_queue_port = status[u'event_object_queue_port']
+      self._parse_error_queue_port = status[u'parse_error_queue_port']
+      queue_start_wait_attempts += 1
+      time.sleep(self._QUEUE_START_WAIT)
+    if not self._StorageQueuesBound():
+      raise RuntimeError(u'Storage queues did not bind in time.')
+
   def SignalAbort(self):
     """Signals the engine to abort."""
     try:
@@ -1004,25 +1217,23 @@ class MultiProcessEventExtractionWorkerProcess(MultiProcessBaseProcess):
                       for parsing.
       worker_number: A number that identifies the worker.
       enable_debug_output: Optional boolean value to indicate if the debug
-                           output should be enabled. The default is False.
+                           output should be enabled.
       enable_profiling: Optional boolean value to indicate if profiling should
-                        be enabled. The default is False.
+                        be enabled.
       filter_object: Optional filter object (instance of objectfilter.Filter).
-                     The default is None.
       hasher_names_string: Optional comma separated string of names of
-                           hashers to enable enable. The default is None.
+                           hashers to enable enable.
       mount_path: Optional string containing the mount path. The default
                   is None.
-      parser_filter_string: Optional parser filter string. The default is None.
+      parser_filter_string: Optional parser filter string.
       process_archive_files: Optional boolean value to indicate if the worker
                              should scan for file entries inside files.
-                             The default is False.
       profiling_sample_rate: optional integer indicating the profiling sample
                              rate. The value contains the number of files
                              processed. The default value is 1000.
-      profiling_type: optional profiling type. The default is 'all'.
+      profiling_type: optional profiling type.
       text_prepend: Optional string that contains the text to prepend to every
-                    event object. The default is None.
+                    event object.
       kwargs: keyword arguments to pass to multiprocessing.Process.
     """
     super(MultiProcessEventExtractionWorkerProcess, self).__init__(
@@ -1088,8 +1299,6 @@ class MultiProcessEventExtractionWorkerProcess(MultiProcessBaseProcess):
         self._parse_error_queue_producer, parser_mediator,
         resolver_context=resolver_context)
 
-    self._extraction_worker.SetEnableDebugOutput(self._enable_debug_output)
-
     self._extraction_worker.SetEnableProfiling(
         self._enable_profiling,
         profiling_sample_rate=self._profiling_sample_rate,
@@ -1130,9 +1339,14 @@ class MultiProcessEventExtractionWorkerProcess(MultiProcessBaseProcess):
     logging.debug(u'Extraction worker: {0!s} (PID: {1:d}) stopped'.format(
         self._name, self._pid))
 
-    self._path_spec_queue.Close(abort=True)
-    self._event_object_queue.Close(abort=True)
-    self._parse_error_queue.Close(abort=True)
+    worker_queues = (
+        self._path_spec_queue, self._event_object_queue,
+        self._parse_error_queue)
+    for worker_queue in worker_queues:
+      if not isinstance(worker_queue, MultiProcessingQueue):
+        worker_queue.Close()
+      else:
+        worker_queue.Close(abort=True)
 
   def _OnCriticalError(self):
     """The process on critical error handler."""
@@ -1155,24 +1369,39 @@ class MultiProcessStorageWriterProcess(MultiProcessBaseProcess):
   """Class that defines a multi-processing storage writer process."""
 
   def __init__(
-      self, event_object_queue, parse_error_queue, storage_writer, **kwargs):
+      self, event_object_queue, extraction_complete_event,
+      storage_writer_complete_event, parse_error_queue, storage_writer,
+      **kwargs):
     """Initializes the process object.
 
     Args:
       event_object_queue: the event object queue object (instance of
                           MultiProcessingQueue).
+      extraction_complete_event: a multi processing event (instance of
+                                 multiprocessing.Event) that, when set,
+                                 indicates that extraction is complete.
+      storage_writer_complete_event: a multi processing event (instance of
+                                     multiprocessing.Event) that, when set,
+                                     indicates that the storage writer is
+                                     complete.
       parse_error_queue: the parser error queue object (instance of Queue).
-      storage_writer: A storage writer object (instance of BaseStorageWriter).
+      storage_writer: a storage writer object (instance of BaseStorageWriter).
     """
     super(MultiProcessStorageWriterProcess, self).__init__(
         definitions.PROCESS_TYPE_STORAGE_WRITER, **kwargs)
+    self._extraction_complete_event = extraction_complete_event
     self._event_object_queue = event_object_queue
+    self._event_object_queue_port = None
     self._parse_error_queue = parse_error_queue
+    self._parse_error_queue_port = None
     self._storage_writer = storage_writer
+    self._storage_writer_complete_event = storage_writer_complete_event
 
   def _GetStatus(self):
     """Returns a status dictionary."""
     status = self._storage_writer.GetStatus()
+    status[u'event_object_queue_port'] = self._event_object_queue_port
+    status[u'parse_error_queue_port'] = self._parse_error_queue_port
     self._status_is_running = status.get(u'is_running', False)
     return status
 
@@ -1180,15 +1409,47 @@ class MultiProcessStorageWriterProcess(MultiProcessBaseProcess):
     """The main loop."""
     logging.debug(u'Storage writer (PID: {0:d}) started.'.format(self._pid))
 
+    event_queue_is_zeromq = isinstance(
+        self._event_object_queue, zeromq_queue.ZeroMQQueue)
+    error_queue_is_zeromq = isinstance(
+        self._parse_error_queue, zeromq_queue.ZeroMQQueue)
+
+    if event_queue_is_zeromq:
+      self._event_object_queue.name = u'Storage writer event'
+      self._event_object_queue.Open()
+      self._event_object_queue_port = self._event_object_queue.port
+
+    if error_queue_is_zeromq:
+      self._parse_error_queue.name = u'Storage writer error'
+      self._parse_error_queue.Open()
+      self._parse_error_queue_port = self._parse_error_queue.port
+
     try:
       self._storage_writer.WriteEventObjects()
-
+      while (event_queue_is_zeromq and
+             not self._extraction_complete_event.is_set()):
+        logging.debug(
+            u'Storage writer event queue stalled - restarting and waiting for '
+            u'extraction to complete.')
+        self._storage_writer.WriteEventObjects()
     except Exception as exception:
       logging.warning(
           u'Unhandled exception in storage writer (PID: {0:d}).'.format(
               self._pid))
       logging.exception(exception)
 
+    # Set the storage writer completion event to indicate that it is complete
+    # and the process can be terminated.
+    self._storage_writer_complete_event.set()
+
+    self._event_object_queue.Close()
+
+    # TODO: This is a work around for infinite blocking wait in storage
+    # writer process. Fix this by using a phased processing approach.
+    # For now we remove the parse error queue from the storage writer.
+    if self._parse_error_queue:
+      self._parse_error_queue.Close()
+
     logging.debug(u'Storage writer (PID: {0:d}) stopped.'.format(self._pid))
 
   def SignalAbort(self):
@@ -1232,6 +1493,10 @@ class MultiProcessingQueue(queue.Queue):
     # This queue appears not to be FIFO.
     self._queue = multiprocessing.Queue(maxsize=maximum_number_of_queued_items)
 
+  def Open(self):
+    """Opens the queue."""
+    pass
+
   # pylint: disable=arguments-differ
   def Close(self, abort=False):
     """Closes the queue.
@@ -1278,10 +1543,12 @@ class MultiProcessingQueue(queue.Queue):
       pass
 
   def PopItem(self):
-    """Pops an item off the queue or None on timeout.
+    """Pops an item off the queue.
 
     Raises:
-      QueueEmpty: when the queue is empty.
+      QueueClose: if the queue has already been closed.
+      QueueEmpty: if no item could be retrieved from the queue within the
+                  specified timeout.
     """
     try:
       # If no timeout is specified the queue will block if empty otherwise
