@@ -1,6 +1,8 @@
 # -*- coding: utf-8 -*-
 from plaso.output import dynamic
-from plaso.output import elastic
+# TODO: renable after refactor as indicated in:
+# https://github.com/log2timeline/plaso/issues/197
+# from plaso.output import elastic
 from plaso.output import json_line
 from plaso.output import json_out
 from plaso.output import l2t_csv
@@ -11,3 +13,4 @@ from plaso.output import rawpy
 from plaso.output import sqlite_4n6time
 from plaso.output import timesketch_out
 from plaso.output import tln
+from plaso.output import xlsx
