@@ -18,8 +18,8 @@ import construct
 import logging
 import os
 
+from plaso.events import time_events
 from plaso.lib import errors
-from plaso.lib import event
 from plaso.lib import eventdata
 from plaso.lib import timelib
 from plaso.parsers import interface
@@ -29,19 +29,20 @@ from plaso.parsers import manager
 __author__ = 'Joaquin Moreno Garijo (Joaquin.MorenoGarijo.2013@live.rhul.ac.uk)'
 
 
-class KeychainInternetRecordEvent(event.EventObject):
+class KeychainInternetRecordEvent(time_events.TimestampEvent):
   """Convenience class for an keychain internet record event."""
 
   DATA_TYPE = u'mac:keychain:internet'
 
   def __init__(
-      self, timestamp, timestamp_desc, entry_name, account_name,
+      self, timestamp, timestamp_description, entry_name, account_name,
       text_description, comments, where, protocol, type_protocol, ssgp_hash):
     """Initializes the event object.
 
     Args:
-      timestamp: Description of the timestamp value.
-      timestamp_desc: Timelib type of the timestamp.
+      timestamp: The timestamp which is an integer containing the number
+                 of micro seconds since January 1, 1970, 00:00:00 UTC.
+      timestamp_description: The usage string for the timestamp value.
       entry_name: Name of the entry.
       account_name: Name of the account.
       text_description: Short description about the entry.
@@ -51,48 +52,47 @@ class KeychainInternetRecordEvent(event.EventObject):
       type_protocol: The sub-protocol used (eg. form).
       ssgp_hash: String with hexadecimal values from the password / cert hash.
     """
-    super(KeychainInternetRecordEvent, self).__init__()
-    self.timestamp = timestamp
-    self.timestamp_desc = timestamp_desc
-    self.entry_name = entry_name
+    super(KeychainInternetRecordEvent, self).__init__(
+        timestamp, timestamp_description)
     self.account_name = account_name
-    self.text_description = text_description
-    self.where = where
-    self.protocol = protocol
-    self.type_protocol = type_protocol
     self.comments = comments
+    self.entry_name = entry_name
+    self.protocol = protocol
     self.ssgp_hash = ssgp_hash
+    self.text_description = text_description
+    self.type_protocol = type_protocol
+    self.where = where
 
 
-class KeychainApplicationRecordEvent(event.EventObject):
+class KeychainApplicationRecordEvent(time_events.TimestampEvent):
   """Convenience class for an keychain application password record event."""
   DATA_TYPE = u'mac:keychain:application'
 
   def __init__(
-      self, timestamp, timestamp_desc, entry_name,
-      account_name, text_description, comments, ssgp_hash):
+      self, timestamp, timestamp_description, entry_name, account_name,
+      text_description, comments, ssgp_hash):
     """Initializes the event object.
 
     Args:
-      timestamp: Description of the timestamp value.
-      timestamp_desc: Timelib type of the timestamp.
+      timestamp: The timestamp which is an integer containing the number
+                 of micro seconds since January 1, 1970, 00:00:00 UTC.
+      timestamp_description: The usage string for the timestamp value.
       entry_name: Name of the entry.
       account_name: Name of the account.
       text_description: Short description about the entry.
       comments: String that contains the comments added by the user.
       ssgp_hash: String with hexadecimal values from the password / cert hash.
     """
-    super(KeychainApplicationRecordEvent, self).__init__()
-    self.timestamp = timestamp
-    self.timestamp_desc = timestamp_desc
-    self.entry_name = entry_name
+    super(KeychainApplicationRecordEvent, self).__init__(
+        timestamp, timestamp_description)
     self.account_name = account_name
-    self.text_description = text_description
     self.comments = comments
+    self.entry_name = entry_name
     self.ssgp_hash = ssgp_hash
+    self.text_description = text_description
 
 
-class KeychainParser(interface.SingleFileBaseParser):
+class KeychainParser(interface.FileObjectParser):
   """Parser for Keychain files."""
 
   _INITIAL_FILE_OFFSET = None
@@ -122,6 +122,7 @@ class KeychainParser(interface.SingleFileBaseParser):
       u'db_schema',
       construct.UBInt32(u'size'),
       construct.UBInt32(u'number_of_tables'))
+
   # For each number_of_tables, the schema has a TABLE_OFFSET with the
   # offset starting in the DB_SCHEMA.
   TABLE_OFFSET = construct.UBInt32(u'table_offset')
@@ -143,7 +144,7 @@ class KeychainParser(interface.SingleFileBaseParser):
       construct.UBInt32(u'ssgp_length'),
       construct.Padding(4),
       construct.UBInt32(u'creation_time'),
-      construct.UBInt32(u'last_mod_time'),
+      construct.UBInt32(u'last_modification_time'),
       construct.UBInt32(u'text_description'),
       construct.Padding(4),
       construct.UBInt32(u'comments'),
@@ -152,10 +153,12 @@ class KeychainParser(interface.SingleFileBaseParser):
       construct.Padding(20),
       construct.UBInt32(u'account_name'),
       construct.Padding(4))
+
   RECORD_HEADER_APP = construct.Struct(
       u'record_entry_app',
       RECORD_HEADER,
       construct.Padding(4))
+
   RECORD_HEADER_INET = construct.Struct(
       u'record_entry_inet',
       RECORD_HEADER,
@@ -167,6 +170,7 @@ class KeychainParser(interface.SingleFileBaseParser):
 
   TEXT = construct.PascalString(
       u'text', length_field=construct.UBInt32(u'length'))
+
   TIME = construct.Struct(
       u'timestamp',
       construct.String(u'year', 4),
@@ -176,6 +180,7 @@ class KeychainParser(interface.SingleFileBaseParser):
       construct.String(u'minute', 2),
       construct.String(u'second', 2),
       construct.Padding(2))
+
   TYPE_TEXT = construct.String(u'type', 4)
 
   # TODO: add more protocols.
@@ -185,12 +190,10 @@ class KeychainParser(interface.SingleFileBaseParser):
       u'imap': u'imap',
       u'http': u'http'}
 
-  def _GetTimestampFromEntry(self, parser_mediator, structure):
-    """Parse a time entry structure into a microseconds since Epoch in UTC.
+  def _GetTimestampFromEntry(self, structure):
+    """Parses a timestamp from a TIME entry structure.
 
     Args:
-      parser_mediator: A parser mediator object (instance of ParserMediator).
-      file_entry: A file entry object (instance of dfvfs.FileEntry).
       structure: TIME entry structure:
                  year: String with the number of the year.
                  month: String with the number of the month.
@@ -200,18 +203,26 @@ class KeychainParser(interface.SingleFileBaseParser):
                  second: String with the number of the second.
 
     Returns:
-      Microseconds since Epoch in UTC.
+      The timestamp which is an integer containing the number of micro seconds
+      since January 1, 1970, 00:00:00 UTC.
+
+    Raises:
+      TimestampError: if the timestamp cannot be created from the date and
+                      time values.
     """
     try:
-      return timelib.Timestamp.FromTimeParts(
-          int(structure.year, 10), int(structure.month, 10),
-          int(structure.day, 10), int(structure.hour, 10),
-          int(structure.minute, 10), int(structure.second, 10))
+      year = int(structure.year, 10)
+      month = int(structure.month, 10)
+      day = int(structure.day, 10)
+      hours = int(structure.hour, 10)
+      minutes = int(structure.minute, 10)
+      seconds = int(structure.second, 10)
     except ValueError:
-      logging.warning(
-          u'[{0:s}] Invalid keychain time {1!s} in file: {2:s}'.format(
-              self.NAME, parser_mediator.GetDisplayName(), structure))
-      return 0
+      raise errors.TimestampError(
+          u'Invalid keychain time {0!s}'.format(structure))
+
+    return timelib.Timestamp.FromTimeParts(
+        year, month, day, hours, minutes, seconds)
 
   def _ReadEntryApplication(self, parser_mediator, file_object):
     """Extracts the information from an application password entry.
@@ -230,27 +241,27 @@ class KeychainParser(interface.SingleFileBaseParser):
               self.NAME, offset, parser_mediator.GetDisplayName()))
       return
 
-    (ssgp_hash, creation_time, last_mod_time, text_description,
+    (ssgp_hash, creation_time, last_modification_time, text_description,
      comments, entry_name, account_name) = self._ReadEntryHeader(
          parser_mediator, file_object, record.record_header, offset)
 
     # Move to the end of the record, and then, prepared for the next record.
-    file_object.seek(
-        record.record_header.entry_length + offset - file_object.tell(),
-        os.SEEK_CUR)
+    next_record_offset = (
+        record.record_header.entry_length + offset - file_object.tell())
+    file_object.seek(next_record_offset, os.SEEK_CUR)
+
     event_object = KeychainApplicationRecordEvent(
         creation_time, eventdata.EventTimestamp.CREATION_TIME,
         entry_name, account_name, text_description, comments, ssgp_hash)
     parser_mediator.ProduceEvent(event_object)
 
-    if creation_time != last_mod_time:
+    if creation_time != last_modification_time:
       event_object = KeychainApplicationRecordEvent(
-          last_mod_time, eventdata.EventTimestamp.MODIFICATION_TIME,
+          last_modification_time, eventdata.EventTimestamp.MODIFICATION_TIME,
           entry_name, account_name, text_description, comments, ssgp_hash)
       parser_mediator.ProduceEvent(event_object)
 
-  def _ReadEntryHeader(
-      self, parser_mediator, file_object, record, offset):
+  def _ReadEntryHeader(self, parser_mediator, file_object, record, offset):
     """Read the common record attributes.
 
     Args:
@@ -261,73 +272,95 @@ class KeychainParser(interface.SingleFileBaseParser):
       offset: First byte of the record.
 
     Returns:
-      A list of:
+      A tuple containing:
         ssgp_hash: Hash of the encrypted data (passwd, cert, note).
         creation_time: When the entry was created.
-        last_mod_time: Last time the entry was updated.
+        last_modification_time: Last time the entry was updated.
         text_description: A brief description of the entry.
         entry_name: Name of the entry
         account_name: Name of the account.
     """
+    # TODO: reduce number of seeks and/or offset calculations needed
+    # for parsing.
+
     # Info: The hash header always start with the string ssgp follow by
     #       the hash. Furthermore The fields are always a multiple of four.
     #       Then if it is not multiple the value is padded by 0x00.
     ssgp_hash = binascii.hexlify(file_object.read(record.ssgp_length)[4:])
 
-    file_object.seek(
-        record.creation_time - file_object.tell() + offset - 1, os.SEEK_CUR)
-    creation_time = self._GetTimestampFromEntry(
-        parser_mediator, self.TIME.parse_stream(file_object))
+    structure_offset = record.creation_time - file_object.tell() + offset - 1
+    file_object.seek(structure_offset, os.SEEK_CUR)
+    try:
+      time_structure = self.TIME.parse_stream(file_object)
+      creation_time = self._GetTimestampFromEntry(time_structure)
+    except (construct.FieldError, errors.TimestampError) as exception:
+      creation_time = timelib.Timestamp.NONE_TIMESTAMP
+      parser_mediator.ProduceParseError(
+          u'unable to determine creation time with error: {0:s}'.format(
+              exception))
 
-    file_object.seek(
-        record.last_mod_time - file_object.tell() + offset - 1, os.SEEK_CUR)
-    last_mod_time = self._GetTimestampFromEntry(
-        parser_mediator, self.TIME.parse_stream(file_object))
+    structure_offset = (
+        record.last_modification_time - file_object.tell() + offset - 1)
+    file_object.seek(structure_offset, os.SEEK_CUR)
 
-    # The comment field does not always contain data.
+    try:
+      time_structure = self.TIME.parse_stream(file_object)
+      last_modification_time = self._GetTimestampFromEntry(time_structure)
+    except (construct.FieldError, errors.TimestampError) as exception:
+      last_modification_time = timelib.Timestamp.NONE_TIMESTAMP
+      parser_mediator.ProduceParseError((
+          u'unable to determine last modification time with error: '
+          u'{0:s}').format(exception))
+
+    text_description = u'N/A'
     if record.text_description:
-      file_object.seek(
-          record.text_description - file_object.tell() + offset -1,
-          os.SEEK_CUR)
+      structure_offset = (
+          record.text_description - file_object.tell() + offset - 1)
+      file_object.seek(structure_offset, os.SEEK_CUR)
+
       try:
         text_description = self.TEXT.parse_stream(file_object)
-      except construct.FieldError:
-        text_description = u'N/A (error)'
-    else:
-      text_description = u'N/A'
+      except construct.FieldError as exception:
+        parser_mediator.ProduceParseError(
+            u'unable to parse text description with error: {0:s}'.format(
+                exception))
 
-    # The comment field does not always contain data.
+    comments = u'N/A'
     if record.comments:
-      file_object.seek(
-          record.text_description - file_object.tell() + offset -1,
-          os.SEEK_CUR)
+      structure_offset = record.comments - file_object.tell() + offset - 1
+      file_object.seek(structure_offset, os.SEEK_CUR)
+
       try:
         comments = self.TEXT.parse_stream(file_object)
-      except construct.FieldError:
-        comments = u'N/A (error)'
-    else:
-      comments = u'N/A'
+      except construct.FieldError as exception:
+        parser_mediator.ProduceParseError(
+            u'unable to parse comments with error: {0:s}'.format(exception))
+
+    structure_offset = record.entry_name - file_object.tell() + offset - 1
+    file_object.seek(structure_offset, os.SEEK_CUR)
 
-    file_object.seek(
-        record.entry_name - file_object.tell() + offset - 1, os.SEEK_CUR)
     try:
       entry_name = self.TEXT.parse_stream(file_object)
-    except construct.FieldError:
-      entry_name = u'N/A (error)'
+    except construct.FieldError as exception:
+      entry_name = u'N/A'
+      parser_mediator.ProduceParseError(
+          u'unable to parse entry name with error: {0:s}'.format(exception))
+
+    structure_offset = record.account_name - file_object.tell() + offset - 1
+    file_object.seek(structure_offset, os.SEEK_CUR)
 
-    file_object.seek(
-        record.account_name - file_object.tell() + offset - 1, os.SEEK_CUR)
     try:
       account_name = self.TEXT.parse_stream(file_object)
-    except construct.FieldError:
-      account_name = u'N/A (error)'
+    except construct.FieldError as exception:
+      account_name = u'N/A'
+      parser_mediator.ProduceParseError(
+          u'unable to parse account name with error: {0:s}'.format(exception))
 
     return (
-        ssgp_hash, creation_time, last_mod_time,
+        ssgp_hash, creation_time, last_modification_time,
         text_description, comments, entry_name, account_name)
 
-  def _ReadEntryInternet(
-      self, parser_mediator, file_object):
+  def _ReadEntryInternet(self, parser_mediator, file_object):
     """Extracts the information from an Internet password entry.
 
     Args:
@@ -344,7 +377,7 @@ class KeychainParser(interface.SingleFileBaseParser):
               self.NAME, offset, parser_mediator.GetDisplayName()))
       return
 
-    (ssgp_hash, creation_time, last_mod_time, text_description,
+    (ssgp_hash, creation_time, last_modification_time, text_description,
      comments, entry_name, account_name) = self._ReadEntryHeader(
          parser_mediator, file_object, record.record_header, offset)
     if not record.where:
@@ -380,9 +413,9 @@ class KeychainParser(interface.SingleFileBaseParser):
         comments, where, protocol, type_protocol, ssgp_hash)
     parser_mediator.ProduceEvent(event_object)
 
-    if creation_time != last_mod_time:
+    if creation_time != last_modification_time:
       event_object = KeychainInternetRecordEvent(
-          last_mod_time, eventdata.EventTimestamp.MODIFICATION_TIME,
+          last_modification_time, eventdata.EventTimestamp.MODIFICATION_TIME,
           entry_name, account_name, text_description,
           comments, where, protocol, type_protocol, ssgp_hash)
       parser_mediator.ProduceEvent(event_object)
@@ -428,7 +461,7 @@ class KeychainParser(interface.SingleFileBaseParser):
     """Parses a Mac OS X keychain file-like object.
 
     Args:
-      parser_mediator: A parser context object (instance of ParserContext).
+      parser_mediator: A parser mediator object (instance of ParserMediator).
       file_object: A file-like object.
 
     Raises:
