@@ -30,7 +30,7 @@ import pytz
 
 
 class SlowLexicalTextParser(
-    interface.SingleFileBaseParser, lexer.SelfFeederMixIn):
+    interface.FileObjectParser, lexer.SelfFeederMixIn):
   """Generic text based parser that uses lexer to assist with parsing.
 
   This text parser is based on a rather slow lexer, which makes the
@@ -65,7 +65,8 @@ class SlowLexicalTextParser(
     """
     # TODO: remove the multiple inheritance.
     lexer.SelfFeederMixIn.__init__(self)
-    interface.SingleFileBaseParser.__init__(self)
+    interface.FileObjectParser.__init__(self)
+    self._file_verified = False
     self.line_ready = False
     self.attributes = {
         u'body': u'',
@@ -77,7 +78,6 @@ class SlowLexicalTextParser(
         u'username': u'',
     }
     self.local_zone = local_zone
-    self.file_entry = None
 
   def ClearValues(self):
     """Clears all the values inside the attributes dict.
@@ -115,7 +115,6 @@ class SlowLexicalTextParser(
 
     Args:
       match: optional regular expression match object (instance of SRE_Match).
-             The default is None.
     """
     if not match:
       return
@@ -148,9 +147,9 @@ class SlowLexicalTextParser(
     path_spec_printable = u'{0:s}:{1:s}'.format(
         file_entry.path_spec.type_indicator, file_entry.name)
 
-    self.file_entry = file_entry
     # TODO: this is necessary since we inherit from lexer.SelfFeederMixIn.
     self.file_object = file_object
+    self._file_verified = False
 
     # Start by checking, is this a text file or not? Before we proceed
     # any further.
@@ -161,7 +160,6 @@ class SlowLexicalTextParser(
     file_object.seek(0, os.SEEK_SET)
 
     error_count = 0
-    file_verified = False
     # We need to clear out few values in the Lexer before continuing.
     # There might be some leftovers from previous run.
     self.error = 0
@@ -174,7 +172,7 @@ class SlowLexicalTextParser(
         self.entry_offset = getattr(self, u'next_entry_offset', 0)
         self.next_entry_offset = file_object.tell() - len(self.buffer)
 
-      if not file_verified and self.error >= self.MAX_LINES * 2:
+      if not self._file_verified and self.error >= self.MAX_LINES * 2:
         logging.debug(
             u'Lexer error count: {0:d} and current state {1:s}'.format(
                 self.error, self.state))
@@ -185,11 +183,11 @@ class SlowLexicalTextParser(
       if self.line_ready:
         try:
           self.ParseLine(parser_mediator)
-          file_verified = True
+          self._file_verified = True
 
         except errors.TimestampError as exception:
           error_count += 1
-          if file_verified:
+          if self._file_verified:
             logging.debug(
                 u'[{0:s} VERIFIED] Error count: {1:d} and ERROR: {2:d}'.format(
                     path_spec_printable, error_count, self.error))
@@ -217,9 +215,9 @@ class SlowLexicalTextParser(
       if self.Empty():
         break
 
-    if not file_verified:
+    if not self._file_verified:
       raise errors.UnableToParseFile(
-          u'[{0:s}] unable to parser file: {1:s}.'.format(
+          u'[{0:s}] unable to parse file: {1:s}.'.format(
               self.NAME, path_spec_printable))
 
     file_offset = file_object.get_offset()
@@ -234,7 +232,6 @@ class SlowLexicalTextParser(
 
     Args:
       match: optional regular expression match object (instance of SRE_Match).
-             The default is None.
 
     Returns:
       A string that combines the values that are so far
@@ -282,57 +279,80 @@ class SlowLexicalTextParser(
     return line
 
   def ParseLine(self, parser_mediator):
-    """Return an event object extracted from the current line.
+    """Parses the current log line for events.
 
     Args:
       parser_mediator: a parser mediator object (instance of ParserMediator).
-
-    Raises:
-      TimestampError: if time is not defined or invalid.
     """
-    if not self.attributes[u'time']:
-      raise errors.TimestampError(
-          u'Unable to parse log line: {0:s} - missing time.'.format(
-              self.PrintLine()))
+    year_string = self.attributes.get(u'iyear')
+    if not year_string:
+      if not self._file_verified:
+        raise errors.UnableToParseFile()
+      parser_mediator.ProduceParseError(
+          u'year missing in log line: {0:s}'.format(self.PrintLine()))
+      return
 
-    if not self.attributes[u'iyear']:
-      raise errors.TimestampError(
-          u'Unable to parse log line: {0:s} - missing year.'.format(
+    time_string = self.attributes.get(u'time')
+    if not time_string:
+      if not self._file_verified:
+        raise errors.UnableToParseFile()
+      parser_mediator.ProduceParseError(
+          u'time values missing in log line: {0:s}'.format(self.PrintLine()))
+      return
+
+    time_values = time_string.split(u':')
+    if len(time_values) < 3:
+      if not self._file_verified:
+        raise errors.UnableToParseFile()
+      parser_mediator.ProduceParseError(
+          u'unsupported time format in log line: {0:s}'.format(
               self.PrintLine()))
+      return
+
+    seconds_values = time_values[2].split(u'.')
+    if len(seconds_values) == 2:
+      seconds_string, microseconds_string = seconds_values
+    else:
+      seconds_string = time_values[2]
+      microseconds_string = 0
+
+    try:
+      # TODO: fix the need to convert non string values into integers and
+      # string to integer conversion without an explicit base.
+      year = int(year_string)
+      hours = int(time_values[0])
+      minutes = int(time_values[1])
+      seconds = int(seconds_string)
+      microseconds = int(microseconds_string)
+
+    except ValueError as exception:
+      if not self._file_verified:
+        raise errors.UnableToParseFile()
+      parser_mediator.ProduceParseError(
+          u'unable to parse log line: {0:s} with error: {1:s}'.format(
+              self.PrintLine(), exception))
+      return
 
-    times = self.attributes[u'time'].split(u':')
     if self.local_zone:
       timezone = parser_mediator.timezone
     else:
       timezone = pytz.UTC
 
-    if len(times) < 3:
-      raise errors.TimestampError(
-          u'Unable to parse log line - unsupported format: {0:s}'.format(
-              self.PrintLine()))
     try:
-      secs = times[2].split('.')
-      if len(secs) == 2:
-        sec, us = secs
-      else:
-        sec = times[2]
-        us = 0
-
       timestamp = timelib.Timestamp.FromTimeParts(
-          int(self.attributes[u'iyear']), self.attributes[u'imonth'],
-          self.attributes[u'iday'], int(times[0]), int(times[1]),
-          int(sec), microseconds=int(us), timezone=timezone)
-
-    except ValueError as exception:
-      raise errors.TimestampError(
-          u'Unable to parse log line: {0:s} with error: {1:s}'.format(
-              self.PrintLine(), exception))
+          year, self.attributes[u'imonth'], self.attributes[u'iday'],
+          hours, minutes, seconds, microseconds=microseconds,
+          timezone=timezone)
+    except errors.TimestampError as exception:
+      timestamp = timelib.Timestamp.NONE_TIMESTAMP
+      parser_mediator.ProduceParseError(
+          u'unable to determine timestamp with error: {0:s}'.format(
+              exception))
 
     event_object = self.CreateEvent(
         timestamp, getattr(self, u'entry_offset', 0), self.attributes)
     parser_mediator.ProduceEvent(event_object)
 
-
   def SetDay(self, match=None, **unused_kwargs):
     """Parses the day of the month.
 
@@ -341,7 +361,6 @@ class SlowLexicalTextParser(
 
     Args:
       match: optional regular expression match object (instance of SRE_Match).
-             The default is None.
     """
     self.attributes[u'iday'] = int(match.group(1))
   def SetMonth(self, match=None, **unused_kwargs):
@@ -352,7 +371,6 @@ class SlowLexicalTextParser(
 
     Args:
       match: optional regular expression match object (instance of SRE_Match).
-             The default is None.
     """
     self.attributes[u'imonth'] = int(
         timelib.MONTH_DICT.get(match.group(1).lower(), 1))
@@ -362,7 +380,6 @@ class SlowLexicalTextParser(
 
     Args:
       match: optional regular expression match object (instance of SRE_Match).
-             The default is None.
     """
     self.attributes[u'time'] = match.group(1)
 
@@ -374,12 +391,11 @@ class SlowLexicalTextParser(
 
     Args:
       match: optional regular expression match object (instance of SRE_Match).
-             The default is None.
     """
     self.attributes[u'iyear'] = int(match.group(1))
 
 
-class TextCSVParser(interface.SingleFileBaseParser):
+class TextCSVParser(interface.FileObjectParser):
   """An implementation of a simple CSV line-per-entry log files.
 
   Attributes:
@@ -679,7 +695,7 @@ class PyparsingConstants(object):
       pyparsing.nums, min=1, max=5).setParseAction(PyParseIntCast)
 
 
-class PyparsingSingleLineTextParser(interface.SingleFileBaseParser):
+class PyparsingSingleLineTextParser(interface.FileObjectParser):
   """Single line text parser based on the pyparsing library."""
 
   # The actual structure, this needs to be defined by each parser.
@@ -708,31 +724,29 @@ class PyparsingSingleLineTextParser(interface.SingleFileBaseParser):
   # If this value needs to be calculated on the fly (not a fixed constant for
   # this particular file type) it can be done by modifying the self.encoding
   # attribute.
-  ENCODING = u''
+  _ENCODING = u'ascii'
 
   def __init__(self):
     """Initializes the pyparsing single-line text parser object."""
     super(PyparsingSingleLineTextParser, self).__init__()
-    self.encoding = self.ENCODING
+    self.encoding = self._ENCODING
     self._current_offset = 0
     # TODO: self._line_structures is a work-around and this needs
     # a structural fix.
     self._line_structures = self.LINE_STRUCTURES
 
   def _ReadLine(
-      self, parser_mediator, file_entry, text_file_object, max_len=0,
-      quiet=False, depth=0):
+      self, parser_mediator, text_file_object, max_len=0, quiet=False, depth=0):
     """Read a single line from a text file and return it back.
 
     Args:
       parser_mediator: a parser mediator object (instance of ParserMediator).
-      file_entry: a file entry object (instance of dfvfs.FileEntry).
       text_file_object: a text file object (instance of dfvfs.TextFile).
-      max_len: if defined determines the maximum number of bytes a single line
-               can take.
-      quiet: if True then a decode warning is not displayed.
-      depth: a threshold of how many newlines we can encounter before bailing
-             out.
+      max_len: optional maximum number of bytes a single line can take.
+      quiet: optional boolean value to indicate parse warning should not be
+             displayed.
+      depth: optional number of new lines the parser can encounter before
+             bailing out.
 
     Returns:
       A single line read from the file-like object, or the maximum number of
@@ -753,8 +767,7 @@ class PyparsingSingleLineTextParser(interface.SingleFileBaseParser):
         return u''
 
       return self._ReadLine(
-          parser_mediator, file_entry, text_file_object, max_len=max_len,
-          depth=depth + 1)
+          parser_mediator, text_file_object, max_len=max_len, depth=depth + 1)
 
     if not self.encoding:
       return line.strip()
@@ -764,11 +777,11 @@ class PyparsingSingleLineTextParser(interface.SingleFileBaseParser):
       return decoded_line.strip()
     except UnicodeDecodeError:
       if not quiet:
+        display_name = parser_mediator.GetDisplayName()
         logging.warning((
             u'Unable to decode line [{0:s}...] with encoding: {1:s} in '
             u'file: {2:s}').format(
-                repr(line[1:30]), self.encoding,
-                parser_mediator.GetDisplayName(file_entry)))
+                repr(line[1:30]), self.encoding, display_name))
       return line.strip()
 
   def ParseFileObject(self, parser_mediator, file_object, **kwargs):
@@ -781,12 +794,6 @@ class PyparsingSingleLineTextParser(interface.SingleFileBaseParser):
     Raises:
       UnableToParseFile: when the file cannot be parsed.
     """
-    file_entry = parser_mediator.GetFileEntry()
-
-    # TODO: find a more elegant way for this; currently the mac_wifi and
-    # syslog parser seem to rely on this member.
-    self.file_entry = file_entry
-
     # TODO: self._line_structures is a work-around and this needs
     # a structural fix.
     if not self._line_structures:
@@ -796,8 +803,8 @@ class PyparsingSingleLineTextParser(interface.SingleFileBaseParser):
     text_file_object = text_file.TextFile(file_object)
 
     line = self._ReadLine(
-        parser_mediator, file_entry, text_file_object,
-        max_len=self.MAX_LINE_LENGTH, quiet=True)
+        parser_mediator, text_file_object, max_len=self.MAX_LINE_LENGTH,
+        quiet=True)
     if not line:
       raise errors.UnableToParseFile(u'Not a text file.')
 
@@ -845,11 +852,11 @@ class PyparsingSingleLineTextParser(interface.SingleFileBaseParser):
                 repr(line), self._current_offset))
 
       self._current_offset = text_file_object.get_offset()
-      line = self._ReadLine(parser_mediator, file_entry, text_file_object)
+      line = self._ReadLine(parser_mediator, text_file_object)
 
   @abc.abstractmethod
   def ParseRecord(self, parser_mediator, key, structure):
-    """Parse a single extracted pyparsing structure.
+    """Parses a log record structure and produces events.
 
     This function takes as an input a parsed pyparsing structure
     and produces an EventObject if possible from that structure.
@@ -888,8 +895,8 @@ class EncodedTextReader(object):
     """Initializes the encoded test reader object.
 
     Args:
-      buffer_size: optional buffer size. The default is 2048.
-      encoding: optional encoding. The default is None.
+      buffer_size: optional buffer size.
+      encoding: optional encoding.
     """
     super(EncodedTextReader, self).__init__()
     self._buffer = b''
@@ -1019,7 +1026,7 @@ class PyparsingMultiLineTextParser(PyparsingSingleLineTextParser):
     super(PyparsingMultiLineTextParser, self).__init__()
     self._buffer_size = self.BUFFER_SIZE
     self._text_reader = EncodedTextReader(
-        buffer_size=self.BUFFER_SIZE, encoding=self.ENCODING)
+        buffer_size=self.BUFFER_SIZE, encoding=self._ENCODING)
 
   def ParseFileObject(self, parser_mediator, file_object, **kwargs):
     """Parses a text file-like object using a pyparsing definition.
@@ -1048,6 +1055,11 @@ class PyparsingMultiLineTextParser(PyparsingSingleLineTextParser):
     if not self.VerifyStructure(parser_mediator, self._text_reader.lines):
       raise errors.UnableToParseFile(u'Wrong file structure.')
 
+    # Using parseWithTabs() overrides Pyparsing's default replacement of tabs
+    # with spaces to SkipAhead() the correct number of bytes after a match.
+    for key, structure in self.LINE_STRUCTURES:
+      structure.parseWithTabs()
+
     # Read every line in the text file.
     while self._text_reader.lines:
       # Initialize pyparsing objects.
@@ -1076,11 +1088,7 @@ class PyparsingMultiLineTextParser(PyparsingSingleLineTextParser):
           break
 
       if tokens and start == 0:
-        parsed_event = self.ParseRecord(parser_mediator, key, tokens)
-        if parsed_event:
-          # TODO: need a reliable way to handle this.
-          # parsed_event.offset = self._text_reader.line_offset
-          parser_mediator.ProduceEvent(parsed_event)
+        self.ParseRecord(parser_mediator, key, tokens)
 
         self._text_reader.SkipAhead(file_object, end)
 
