@@ -42,7 +42,7 @@ class WinPrefetchExecutionEvent(time_events.FiletimeEvent):
     self.offset = 0
 
     self.version = file_header.get(u'version', None)
-    self.executable = binary.Ut16StreamCopyToString(
+    self.executable = binary.UTF16StreamCopyToString(
         file_header.get(u'executable', u''))
     self.prefetch_hash = file_header.get(u'prefetch_hash', None)
 
@@ -55,7 +55,7 @@ class WinPrefetchExecutionEvent(time_events.FiletimeEvent):
     self.volume_device_paths = volume_device_paths
 
 
-class WinPrefetchParser(interface.SingleFileBaseParser):
+class WinPrefetchParser(interface.FileObjectParser):
   """A parser for Windows Prefetch files."""
 
   _INITIAL_FILE_OFFSET = None
@@ -184,6 +184,9 @@ class WinPrefetchParser(interface.SingleFileBaseParser):
 
     Returns:
       The file header construct object.
+
+    Raises:
+      UnableToParseFile: when the header cannot be parsed.
     """
     try:
       file_header = self._FILE_HEADER_STRUCT.parse_stream(file_object)
@@ -208,6 +211,9 @@ class WinPrefetchParser(interface.SingleFileBaseParser):
 
     Returns:
       The file information construct object.
+
+    Raises:
+      UnableToParseFile: when the file information cannot be parsed.
     """
     try:
       if format_version == 17:
@@ -238,6 +244,9 @@ class WinPrefetchParser(interface.SingleFileBaseParser):
 
     Returns:
       A list of metrics array entry construct objects.
+
+    Raises:
+      UnableToParseFile: when the metrics array cannot be parsed.
     """
     metrics_array = []
 
@@ -289,7 +298,7 @@ class WinPrefetchParser(interface.SingleFileBaseParser):
     if filename_strings_offset > 0 and filename_strings_size > 0:
       file_object.seek(filename_strings_offset, os.SEEK_SET)
       filename_strings_data = file_object.read(filename_strings_size)
-      filename_strings = binary.ArrayOfUt16StreamCopyToStringTable(
+      filename_strings = binary.ArrayOfUTF16StreamCopyToStringTable(
           filename_strings_data)
 
     else:
@@ -308,6 +317,9 @@ class WinPrefetchParser(interface.SingleFileBaseParser):
 
     Yields:
       A volume information construct object.
+
+    Raises:
+      UnableToParseFile: when the volume information cannot be parsed.
     """
     volumes_information_offset = file_information.get(
         u'volumes_information_offset', 0)
@@ -358,7 +370,7 @@ class WinPrefetchParser(interface.SingleFileBaseParser):
         current_offset = file_object.tell()
 
         file_object.seek(device_path_offset, os.SEEK_SET)
-        device_path = binary.ReadUtf16Stream(
+        device_path = binary.ReadUTF16Stream(
             file_object, byte_size=device_path_size)
 
         file_object.seek(current_offset, os.SEEK_SET)
@@ -409,7 +421,7 @@ class WinPrefetchParser(interface.SingleFileBaseParser):
       logging.debug(
           u'Mismatch in number of metrics and filename strings array entries.')
 
-    executable = binary.Ut16StreamCopyToString(
+    executable = binary.UTF16StreamCopyToString(
         file_header.get(u'executable', u''))
 
     volume_serial_numbers = []
@@ -429,7 +441,7 @@ class WinPrefetchParser(interface.SingleFileBaseParser):
       if timestamp:
         event_object = windows_events.WindowsVolumeCreationEvent(
             timestamp, volume_device_path, volume_serial_number,
-            parser_mediator.GetFileEntry().name)
+            parser_mediator.GetFilename())
         parser_mediator.ProduceEvent(event_object)
 
       for filename in filename_strings.itervalues():
