@@ -17,22 +17,21 @@ class AppCompatCacheEvent(time_events.FiletimeEvent):
   DATA_TYPE = u'windows:registry:appcompatcache'
 
   def __init__(
-      self, filetime, usage, key, entry_index, path, offset):
+      self, filetime, usage, key_name, entry_index, path, offset):
     """Initializes a Windows Registry event.
 
     Args:
       filetime: The FILETIME timestamp value.
       usage: The description of the usage of the time value.
-      key: Name of the Registry key being parsed.
+      key_name: The name of the corresponding Windows Registry key.
       entry_index: The cache entry index number for the record.
       path: The full path to the executable.
-      offset: The (data) offset of the Registry key or value.
+      offset: The (data) offset of the Windows Registry key or value.
     """
     super(AppCompatCacheEvent, self).__init__(filetime, usage)
-
-    self.keyname = key
-    self.offset = offset
     self.entry_index = entry_index
+    self.keyname = key_name
+    self.offset = offset
     self.path = path
 
 
@@ -62,19 +61,23 @@ class AppCompatCacheCachedEntry(object):
     self.path = None
 
 
-class AppCompatCachePlugin(interface.KeyPlugin):
+class AppCompatCachePlugin(interface.WindowsRegistryPlugin):
   """Class that parses the Application Compatibility Cache Registry data."""
 
   NAME = u'appcompatcache'
   DESCRIPTION = u'Parser for Application Compatibility Cache Registry data.'
 
-  REG_KEYS = [
-      u'\\{current_control_set}\\Control\\Session Manager\\AppCompatibility',
-      u'\\{current_control_set}\\Control\\Session Manager\\AppCompatCache']
-  REG_TYPE = u'SYSTEM'
-  URL = [
-      (u'https://code.google.com/p/winreg-kb/wiki/'
-       u'ApplicationCompatibilityCacheKey')]
+  FILTERS = frozenset([
+      interface.WindowsRegistryKeyPathFilter(
+          u'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\'
+          u'Session Manager\\AppCompatibility'),
+      interface.WindowsRegistryKeyPathFilter(
+          u'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\'
+          u'Session Manager\\AppCompatCache')])
+
+  URLS = [
+      (u'https://github.com/libyal/winreg-kb/blob/master/documentation/'
+       u'Application%20Compatibility%20Cache%20key.asciidoc')]
 
   _FORMAT_TYPE_2000 = 1
   _FORMAT_TYPE_XP = 2
@@ -473,7 +476,7 @@ class AppCompatCachePlugin(interface.KeyPlugin):
           break
         string_size += 2
 
-      cached_entry_object.path = binary.Ut16StreamCopyToString(
+      cached_entry_object.path = binary.UTF16StreamCopyToString(
           cached_entry_data[0:string_size])
 
     elif format_type in [
@@ -485,7 +488,7 @@ class AppCompatCachePlugin(interface.KeyPlugin):
       path_size = cached_entry_struct.get(u'path_size')
 
       cached_entry_data_offset = 14 + path_size
-      cached_entry_object.path = binary.Ut16StreamCopyToString(
+      cached_entry_object.path = binary.UTF16StreamCopyToString(
           cached_entry_data[14:cached_entry_data_offset])
 
       if format_type == self._FORMAT_TYPE_8:
@@ -537,7 +540,7 @@ class AppCompatCachePlugin(interface.KeyPlugin):
     if path_offset > 0 and path_size > 0:
       path_size += path_offset
 
-      cached_entry_object.path = binary.Ut16StreamCopyToString(
+      cached_entry_object.path = binary.UTF16StreamCopyToString(
           value_data[path_offset:path_size])
 
     if data_size > 0:
@@ -547,20 +550,15 @@ class AppCompatCachePlugin(interface.KeyPlugin):
 
     return cached_entry_object
 
-  def GetEntries(
-      self, parser_mediator, key=None, registry_type=None, codepage=u'cp1252',
-      **unused_kwargs):
+  def GetEntries(self, parser_mediator, registry_key, **kwargs):
     """Extracts event objects from a Application Compatibility Cache key.
 
     Args:
       parser_mediator: A parser mediator object (instance of ParserMediator).
-      key: Optional Registry key (instance of winreg.WinRegKey).
-           The default is None.
-      file_entry: Optional file entry object (instance of dfvfs.FileEntry).
-                  The default is None.
-      codepage: Optional extended ASCII string codepage. The default is cp1252.
+      registry_key: A Windows Registry key (instance of
+                    dfwinreg.WinRegistryKey).
     """
-    value = key.GetValue(u'AppCompatCache')
+    value = registry_key.GetValueByName(u'AppCompatCache')
     if not value:
       return
 
@@ -571,7 +569,7 @@ class AppCompatCachePlugin(interface.KeyPlugin):
     if not format_type:
       parser_mediator.ProduceParseError(
           u'Unsupported signature in AppCompatCache key: {0:s}'.format(
-              key.path))
+              registry_key.path))
       return
 
     header_object = self._ParseHeader(format_type, value_data)
@@ -586,9 +584,9 @@ class AppCompatCachePlugin(interface.KeyPlugin):
         format_type, value_data, cached_entry_offset)
 
     if not cached_entry_size:
-      parser_mediator.ProduceParseError(
+      parser_mediator.ProduceParseError((
           u'Unsupported cached entry size at offset {0:d} in AppCompatCache '
-          u'key: {1:s}'.format(cached_entry_offset, key.path))
+          u'key: {1:s}').format(cached_entry_offset, registry_key.path))
       return
 
     cached_entry_index = 0
@@ -600,7 +598,7 @@ class AppCompatCachePlugin(interface.KeyPlugin):
         # TODO: refactor to file modification event.
         event_object = AppCompatCacheEvent(
             cached_entry_object.last_modification_time,
-            u'File Last Modification Time', key.path,
+            u'File Last Modification Time', registry_key.path,
             cached_entry_index + 1, cached_entry_object.path,
             cached_entry_offset)
         parser_mediator.ProduceEvent(event_object)
@@ -609,7 +607,7 @@ class AppCompatCachePlugin(interface.KeyPlugin):
         # TODO: refactor to process run event.
         event_object = AppCompatCacheEvent(
             cached_entry_object.last_update_time,
-            eventdata.EventTimestamp.LAST_RUNTIME, key.path,
+            eventdata.EventTimestamp.LAST_RUNTIME, registry_key.path,
             cached_entry_index + 1, cached_entry_object.path,
             cached_entry_offset)
         parser_mediator.ProduceEvent(event_object)
