@@ -13,18 +13,56 @@ from plaso.parsers.shared import shell_items
 from plaso.parsers.winreg_plugins import interface
 
 
-# A mixin class is used here to not to have the duplicate functionality
-# to parse the MRUListEx Registry values. However multiple inheritance
-# and thus mixins are to be used sparsely in this codebase, hence we need
-# to find a better solution in not needing to distinguish between key and
-# value plugins.
-# TODO: refactor Registry key and value plugin to rid ourselves of the mixin.
-class MRUListExPluginMixin(object):
+class MRUListExStringRegistryKeyFilter(
+    interface.WindowsRegistryKeyWithValuesFilter):
+  """Windows Registry key with values filter."""
+
+  _IGNORE_KEY_PATH_SEGMENTS = frozenset([
+      u'\\BagMRU\\'.upper(),
+      u'\\Explorer\\ComDlg32\\OpenSavePidlMRU\\'.upper()])
+
+  _IGNORE_KEY_PATH_SUFFIXES = frozenset([
+      u'\\BagMRU'.upper(),
+      u'\\Explorer\\StreamMRU'.upper(),
+      u'\\Explorer\\ComDlg32\\OpenSavePidlMRU'.upper()])
+
+  _VALUE_NAMES = [u'0', u'MRUListEx']
+
+  def __init__(self):
+    """Initializes Windows Registry key filter object."""
+    super(MRUListExStringRegistryKeyFilter, self).__init__(self._VALUE_NAMES)
+
+  def Match(self, registry_key):
+    """Determines if a Windows Registry key matches the filter.
+
+    Args:
+      registry_key: a Windows Registry key (instance of
+                    dfwinreg.WinRegistryKey).
+
+    Returns:
+      A boolean value that indicates a match.
+    """
+    key_path_upper = registry_key.path.upper()
+    # Prevent this filter matching non-string MRUListEx values.
+    for ignore_key_path_suffix in self._IGNORE_KEY_PATH_SUFFIXES:
+      if key_path_upper.endswith(ignore_key_path_suffix):
+        return False
+
+    for ignore_key_path_segment in self._IGNORE_KEY_PATH_SEGMENTS:
+      if ignore_key_path_segment in key_path_upper:
+        return False
+
+    return super(MRUListExStringRegistryKeyFilter, self).Match(registry_key)
+
+
+class BaseMRUListExPlugin(interface.WindowsRegistryPlugin):
   """Class for common MRUListEx Windows Registry plugin functionality."""
 
   _MRULISTEX_STRUCT = construct.Range(
       1, 500, construct.ULInt32(u'entry_number'))
 
+  _SOURCE_APPEND = u': MRUListEx'
+
   @abc.abstractmethod
   def _ParseMRUListExEntryValue(
       self, parser_mediator, key, entry_index, entry_number, **kwargs):
@@ -32,7 +70,7 @@ class MRUListExPluginMixin(object):
 
     Args:
       parser_mediator: A parser mediator object (instance of ParserMediator).
-      key: the Registry key (instance of winreg.WinRegKey) that contains
+      key: the Registry key (instance of dfwinreg.WinRegistryKey) that contains
            the MRUListEx value.
       entry_index: integer value representing the MRUListEx entry index.
       entry_number: integer value representing the entry number.
@@ -45,14 +83,14 @@ class MRUListExPluginMixin(object):
     """Parses the MRUListEx value in a given Registry key.
 
     Args:
-      key: the Registry key (instance of winreg.WinRegKey) that contains
+      key: the Registry key (instance of dfwinreg.WinRegistryKey) that contains
            the MRUListEx value.
 
     Returns:
       A MRUListEx value generator, which returns the MRU index number
       and entry value.
     """
-    mru_list_value = key.GetValue(u'MRUListEx')
+    mru_list_value = key.GetValueByName(u'MRUListEx')
 
     # The key exists but does not contain a value named "MRUListEx".
     if not mru_list_value:
@@ -67,17 +105,15 @@ class MRUListExPluginMixin(object):
 
     return enumerate(mru_list)
 
-  def _ParseMRUListExKey(
-      self, parser_mediator, key, registry_type=None, codepage=u'cp1252'):
+  def _ParseMRUListExKey(self, parser_mediator, key, codepage=u'cp1252'):
     """Extract event objects from a MRUListEx Registry key.
 
     Args:
       parser_mediator: A parser mediator object (instance of ParserMediator).
-      key: the Registry key (instance of winreg.WinRegKey).
-      registry_type: Optional Registry type string. The default is None.
-      codepage: Optional extended ASCII string codepage. The default is cp1252.
+      key: the Registry key (instance of dfwinreg.WinRegistryKey).
+      codepage: Optional extended ASCII string codepage.
     """
-    text_dict = {}
+    values_dict = {}
     for entry_index, entry_number in self._ParseMRUListExValue(key):
       # TODO: detect if list ends prematurely.
       # MRU lists are terminated with 0xffffffff (-1).
@@ -90,23 +126,21 @@ class MRUListExPluginMixin(object):
       value_text = u'Index: {0:d} [MRU Value {1:d}]'.format(
           entry_index + 1, entry_number)
 
-      text_dict[value_text] = value_string
+      values_dict[value_text] = value_string
 
     event_object = windows_events.WindowsRegistryEvent(
-        key.last_written_timestamp, key.path, text_dict,
-        offset=key.offset, registry_type=registry_type,
-        source_append=': MRUListEx')
+        key.last_written_time, key.path, values_dict,
+        offset=key.offset, source_append=self._SOURCE_APPEND)
     parser_mediator.ProduceEvent(event_object)
 
 
-class MRUListExStringPlugin(interface.ValuePlugin, MRUListExPluginMixin):
+class MRUListExStringPlugin(BaseMRUListExPlugin):
   """Windows Registry plugin to parse a string MRUListEx."""
 
   NAME = u'mrulistex_string'
   DESCRIPTION = u'Parser for Most Recently Used (MRU) Registry data.'
 
-  REG_TYPE = u'any'
-  REG_VALUES = frozenset([u'MRUListEx', u'0'])
+  FILTERS = frozenset([MRUListExStringRegistryKeyFilter()])
 
   URLS = [
       u'http://forensicartifacts.com/2011/02/recentdocs/',
@@ -123,7 +157,7 @@ class MRUListExStringPlugin(interface.ValuePlugin, MRUListExPluginMixin):
 
     Args:
       parser_mediator: A parser mediator object (instance of ParserMediator).
-      key: the Registry key (instance of winreg.WinRegKey) that contains
+      key: the Registry key (instance of dfwinreg.WinRegistryKey) that contains
            the MRUListEx value.
       entry_index: integer value representing the MRUListEx entry index.
       entry_number: integer value representing the entry number.
@@ -133,20 +167,20 @@ class MRUListExStringPlugin(interface.ValuePlugin, MRUListExPluginMixin):
     """
     value_string = u''
 
-    value = key.GetValue(u'{0:d}'.format(entry_number))
+    value = key.GetValueByName(u'{0:d}'.format(entry_number))
     if value is None:
       logging.debug(
           u'[{0:s}] Missing MRUListEx entry value: {1:d} in key: {2:s}.'.format(
               self.NAME, entry_number, key.path))
 
     elif value.DataIsString():
-      value_string = value.data
+      value_string = value.GetDataAsObject()
 
     elif value.DataIsBinaryData():
       logging.debug((
           u'[{0:s}] Non-string MRUListEx entry value: {1:d} parsed as string '
           u'in key: {2:s}.').format(self.NAME, entry_number, key.path))
-      utf16_stream = binary.ByteStreamCopyToUtf16Stream(value.data)
+      utf16_stream = binary.ByteStreamCopyToUTF16Stream(value.data)
 
       try:
         value_string = utf16_stream.decode(u'utf-16-le')
@@ -160,51 +194,31 @@ class MRUListExStringPlugin(interface.ValuePlugin, MRUListExPluginMixin):
     return value_string
 
   def GetEntries(
-      self, parser_mediator, key=None, registry_type=None, codepage=u'cp1252',
-      **kwargs):
+      self, parser_mediator, registry_key, codepage=u'cp1252', **kwargs):
     """Extract event objects from a Registry key containing a MRUListEx value.
 
     Args:
       parser_mediator: A parser mediator object (instance of ParserMediator).
-      key: Optional Registry key (instance of winreg.WinRegKey).
-           The default is None.
-      registry_type: Optional Registry type string. The default is None.
-      codepage: Optional extended ASCII string codepage. The default is cp1252.
+      registry_key: A Windows Registry key (instance of
+                    dfwinreg.WinRegistryKey).
+      codepage: Optional extended ASCII string codepage.
     """
-    self._ParseMRUListExKey(
-        parser_mediator, key, registry_type=registry_type, codepage=codepage)
-
-  def Process(self, parser_mediator, key=None, codepage=u'cp1252', **kwargs):
-    """Determine if we can process this Registry key or not.
-
-    Args:
-      parser_mediator: A parser mediator object (instance of ParserMediator).
-      key: Optional Windows Registry key (instance of WinRegKey).
-           The default is None.
-      codepage: Optional extended ASCII string codepage. The default is cp1252.
-    """
-    # Prevent this plugin triggering on sub paths of non-string MRUListEx
-    # values.
-    if (u'BagMRU' in key.path or u'Explorer\\StreamMRU' in key.path or
-        u'\\Explorer\\ComDlg32\\OpenSavePidlMRU' in key.path):
-      return
-
-    super(MRUListExStringPlugin, self).Process(
-        parser_mediator, key=key, codepage=codepage)
+    self._ParseMRUListExKey(parser_mediator, registry_key, codepage=codepage)
 
 
-class MRUListExShellItemListPlugin(interface.KeyPlugin, MRUListExPluginMixin):
+class MRUListExShellItemListPlugin(BaseMRUListExPlugin):
   """Windows Registry plugin to parse a shell item list MRUListEx."""
 
   NAME = u'mrulistex_shell_item_list'
   DESCRIPTION = u'Parser for Most Recently Used (MRU) Registry data.'
 
-  REG_TYPE = u'any'
-  REG_KEYS = frozenset([
-      # The regular expression indicated a file extension (.jpg) or '*'.
-      (u'\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\ComDlg32\\'
-       u'OpenSavePidlMRU'),
-      u'\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\StreamMRU'])
+  FILTERS = frozenset([
+      interface.WindowsRegistryKeyPathFilter(
+          u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
+          u'Explorer\\ComDlg32\\OpenSavePidlMRU'),
+      interface.WindowsRegistryKeyPathFilter(
+          u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
+          u'Explorer\\StreamMRU')])
 
   def _ParseMRUListExEntryValue(
       self, parser_mediator, key, entry_index, entry_number, codepage=u'cp1252',
@@ -213,18 +227,18 @@ class MRUListExShellItemListPlugin(interface.KeyPlugin, MRUListExPluginMixin):
 
     Args:
       parser_mediator: A parser mediator object (instance of ParserMediator).
-      key: the Registry key (instance of winreg.WinRegKey) that contains
+      key: the Registry key (instance of dfwinreg.WinRegistryKey) that contains
            the MRUListEx value.
       entry_index: integer value representing the MRUListEx entry index.
       entry_number: integer value representing the entry number.
-      codepage: Optional extended ASCII string codepage. The default is cp1252.
+      codepage: Optional extended ASCII string codepage.
 
     Returns:
       A string containing the value.
     """
     value_string = u''
 
-    value = key.GetValue(u'{0:d}'.format(entry_number))
+    value = key.GetValueByName(u'{0:d}'.format(entry_number))
     if value is None:
       logging.debug(
           u'[{0:s}] Missing MRUListEx entry value: {1:d} in key: {2:s}.'.format(
@@ -237,8 +251,8 @@ class MRUListExShellItemListPlugin(interface.KeyPlugin, MRUListExPluginMixin):
 
     elif value.data:
       shell_items_parser = shell_items.ShellItemsParser(key.path)
-      shell_items_parser.UpdateChainAndParse(
-          parser_mediator, value.data, None, codepage=codepage)
+      shell_items_parser.ParseByteStream(
+          parser_mediator, value.data, codepage=codepage)
 
       value_string = u'Shell item path: {0:s}'.format(
           shell_items_parser.CopyToPath())
@@ -246,40 +260,35 @@ class MRUListExShellItemListPlugin(interface.KeyPlugin, MRUListExPluginMixin):
     return value_string
 
   def GetEntries(
-      self, parser_mediator, key=None, registry_type=None, codepage=u'cp1252',
-      **kwargs):
+      self, parser_mediator, registry_key, codepage=u'cp1252', **kwargs):
     """Extract event objects from a Registry key containing a MRUListEx value.
 
     Args:
       parser_mediator: A parser mediator object (instance of ParserMediator).
-      key: Optional Registry key (instance of winreg.WinRegKey).
-           The default is None.
-      registry_type: Optional Registry type string. The default is None.
-      codepage: Optional extended ASCII string codepage. The default is cp1252.
+      registry_key: A Windows Registry key (instance of
+                    dfwinreg.WinRegistryKey).
+      codepage: Optional extended ASCII string codepage.
     """
-    if key.name != u'OpenSavePidlMRU':
-      self._ParseMRUListExKey(
-          parser_mediator, key, registry_type=registry_type, codepage=codepage)
+    if registry_key.name != u'OpenSavePidlMRU':
+      self._ParseMRUListExKey(parser_mediator, registry_key, codepage=codepage)
 
-    if key.name == u'OpenSavePidlMRU':
+    if registry_key.name == u'OpenSavePidlMRU':
       # For the OpenSavePidlMRU MRUListEx we also need to parse its subkeys
       # since the Registry key path does not support wildcards yet.
-      for subkey in key.GetSubkeys():
-        self._ParseMRUListExKey(
-            parser_mediator, subkey, registry_type=registry_type,
-            codepage=codepage)
+      for subkey in registry_key.GetSubkeys():
+        self._ParseMRUListExKey(parser_mediator, subkey, codepage=codepage)
 
 
-class MRUListExStringAndShellItemPlugin(
-    interface.KeyPlugin, MRUListExPluginMixin):
+class MRUListExStringAndShellItemPlugin(BaseMRUListExPlugin):
   """Windows Registry plugin to parse a string and shell item MRUListEx."""
 
   NAME = u'mrulistex_string_and_shell_item'
   DESCRIPTION = u'Parser for Most Recently Used (MRU) Registry data.'
 
-  REG_TYPE = u'any'
-  REG_KEYS = frozenset([
-      u'\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\RecentDocs'])
+  FILTERS = frozenset([
+      interface.WindowsRegistryKeyPathFilter(
+          u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
+          u'Explorer\\RecentDocs')])
 
   _STRING_AND_SHELL_ITEM_STRUCT = construct.Struct(
       u'string_and_shell_item',
@@ -294,18 +303,18 @@ class MRUListExStringAndShellItemPlugin(
 
     Args:
       parser_mediator: A parser mediator object (instance of ParserMediator).
-      key: the Registry key (instance of winreg.WinRegKey) that contains
+      key: the Registry key (instance of dfwinreg.WinRegistryKey) that contains
            the MRUListEx value.
       entry_index: integer value representing the MRUListEx entry index.
       entry_number: integer value representing the entry number.
-      codepage: Optional extended ASCII string codepage. The default is cp1252.
+      codepage: Optional extended ASCII string codepage.
 
     Returns:
       A string containing the value.
     """
     value_string = u''
 
-    value = key.GetValue(u'{0:d}'.format(entry_number))
+    value = key.GetValueByName(u'{0:d}'.format(entry_number))
     if value is None:
       logging.debug(
           u'[{0:s}] Missing MRUListEx entry value: {1:d} in key: {2:s}.'.format(
@@ -340,8 +349,8 @@ class MRUListExStringAndShellItemPlugin(
 
         else:
           shell_items_parser = shell_items.ShellItemsParser(key.path)
-          shell_items_parser.UpdateChainAndParse(
-              parser_mediator, shell_item_list_data, None, codepage=codepage)
+          shell_items_parser.ParseByteStream(
+              parser_mediator, shell_item_list_data, codepage=codepage)
 
           value_string = u'Path: {0:s}, Shell item: [{1:s}]'.format(
               path, shell_items_parser.CopyToPath())
@@ -349,40 +358,34 @@ class MRUListExStringAndShellItemPlugin(
     return value_string
 
   def GetEntries(
-      self, parser_mediator, key=None, registry_type=None, codepage=u'cp1252',
-      **kwargs):
+      self, parser_mediator, registry_key, codepage=u'cp1252', **kwargs):
     """Extract event objects from a Registry key containing a MRUListEx value.
 
     Args:
       parser_mediator: A parser mediator object (instance of ParserMediator).
-      key: Optional Registry key (instance of winreg.WinRegKey).
-           The default is None.
-      registry_type: Optional Registry type string. The default is None.
-      codepage: Optional extended ASCII string codepage. The default is cp1252.
+      registry_key: A Windows Registry key (instance of
+                    dfwinreg.WinRegistryKey).
+      codepage: Optional extended ASCII string codepage.
     """
-    self._ParseMRUListExKey(
-        parser_mediator, key, registry_type=registry_type, codepage=codepage)
+    self._ParseMRUListExKey(parser_mediator, registry_key, codepage=codepage)
 
-    if key.name == u'RecentDocs':
+    if registry_key.name == u'RecentDocs':
       # For the RecentDocs MRUListEx we also need to parse its subkeys
       # since the Registry key path does not support wildcards yet.
-      for subkey in key.GetSubkeys():
-        self._ParseMRUListExKey(
-            parser_mediator, subkey, registry_type=registry_type,
-            codepage=codepage)
+      for subkey in registry_key.GetSubkeys():
+        self._ParseMRUListExKey(parser_mediator, subkey, codepage=codepage)
 
 
-class MRUListExStringAndShellItemListPlugin(
-    interface.KeyPlugin, MRUListExPluginMixin):
+class MRUListExStringAndShellItemListPlugin(BaseMRUListExPlugin):
   """Windows Registry plugin to parse a string and shell item list MRUListEx."""
 
   NAME = u'mrulistex_string_and_shell_item_list'
   DESCRIPTION = u'Parser for Most Recently Used (MRU) Registry data.'
 
-  REG_TYPE = u'any'
-  REG_KEYS = frozenset([
-      (u'\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\ComDlg32\\'
-       u'LastVisitedPidlMRU')])
+  FILTERS = frozenset([
+      interface.WindowsRegistryKeyPathFilter(
+          u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
+          u'Explorer\\ComDlg32\\LastVisitedPidlMRU')])
 
   _STRING_AND_SHELL_ITEM_LIST_STRUCT = construct.Struct(
       u'string_and_shell_item',
@@ -397,18 +400,18 @@ class MRUListExStringAndShellItemListPlugin(
 
     Args:
       parser_mediator: A parser mediator object (instance of ParserMediator).
-      key: the Registry key (instance of winreg.WinRegKey) that contains
+      key: the Registry key (instance of dfwinreg.WinRegistryKey) that contains
            the MRUListEx value.
       entry_index: integer value representing the MRUListEx entry index.
       entry_number: integer value representing the entry number.
-      codepage: Optional extended ASCII string codepage. The default is cp1252.
+      codepage: Optional extended ASCII string codepage.
 
     Returns:
       A string containing the value.
     """
     value_string = u''
 
-    value = key.GetValue(u'{0:d}'.format(entry_number))
+    value = key.GetValueByName(u'{0:d}'.format(entry_number))
     if value is None:
       logging.debug(
           u'[{0:s}] Missing MRUListEx entry value: {1:d} in key: {2:s}.'.format(
@@ -443,8 +446,8 @@ class MRUListExStringAndShellItemListPlugin(
 
         else:
           shell_items_parser = shell_items.ShellItemsParser(key.path)
-          shell_items_parser.UpdateChainAndParse(
-              parser_mediator, shell_item_list_data, None, codepage=codepage)
+          shell_items_parser.ParseByteStream(
+              parser_mediator, shell_item_list_data, codepage=codepage)
 
           value_string = u'Path: {0:s}, Shell item path: {1:s}'.format(
               path, shell_items_parser.CopyToPath())
@@ -452,19 +455,16 @@ class MRUListExStringAndShellItemListPlugin(
     return value_string
 
   def GetEntries(
-      self, parser_mediator, key=None, registry_type=None, codepage=u'cp1252',
-      **kwargs):
+      self, parser_mediator, registry_key, codepage=u'cp1252', **kwargs):
     """Extract event objects from a Registry key containing a MRUListEx value.
 
     Args:
       parser_mediator: A parser mediator object (instance of ParserMediator).
-      key: Optional Registry key (instance of winreg.WinRegKey).
-           The default is None.
-      registry_type: Optional Registry type string. The default is None.
-      codepage: Optional extended ASCII string codepage. The default is cp1252.
+      registry_key: A Windows Registry key (instance of
+                    dfwinreg.WinRegistryKey).
+      codepage: Optional extended ASCII string codepage.
     """
-    self._ParseMRUListExKey(
-        parser_mediator, key, registry_type=registry_type, codepage=codepage)
+    self._ParseMRUListExKey(parser_mediator, registry_key, codepage=codepage)
 
 
 winreg.WinRegistryParser.RegisterPlugins([
