@@ -7,6 +7,7 @@ from dfvfs.serializer import protobuf_serializer as dfvfs_protobuf_serializer
 from google.protobuf import message
 
 from plaso.lib import event
+from plaso.lib import py2to3
 from plaso.lib import utils
 from plaso.proto import plaso_storage_pb2
 from plaso.serializer import interface
@@ -139,7 +140,7 @@ class ProtobufEventAttributeSerializer(object):
     elif isinstance(attribute_value, bool):
       proto_attribute.boolean = attribute_value
 
-    elif isinstance(attribute_value, (int, long)):
+    elif isinstance(attribute_value, py2to3.INTEGER_TYPES):
       # TODO: add some bounds checking.
       proto_attribute.integer = attribute_value
 
@@ -167,6 +168,9 @@ class ProtobufEventAttributeSerializer(object):
       proto_attribute: a protobuf attribute object.
       attribute_name: the name of the attribute.
       ditctobject: a dictionary object that is the value of the attribute.
+
+    Raises:
+      AttributeError: if the attribute cannot be merged with the dictionary.
     """
     dict_proto = plaso_storage_pb2.Dict()
 
@@ -175,7 +179,12 @@ class ProtobufEventAttributeSerializer(object):
       cls.WriteSerializedObject(dict_proto_add, dict_key, dict_value)
 
     dict_attribute = getattr(proto_attribute, attribute_name)
-    dict_attribute.MergeFrom(dict_proto)
+    try:
+      dict_attribute.MergeFrom(dict_proto)
+    except AttributeError as exception:
+      raise AttributeError(
+          u'Unable to merge attribute: {0:s} with error: {1:s}'.format(
+              attribute_name, exception))
 
   @classmethod
   def WriteSerializedListObject(
@@ -186,6 +195,9 @@ class ProtobufEventAttributeSerializer(object):
       proto_attribute: a protobuf attribute object.
       attribute_name: the name of the attribute.
       list_object: a list object that is the value of the attribute.
+
+    Raises:
+      AttributeError: if the attribute cannot be merged with the list.
     """
     list_proto = plaso_storage_pb2.Array()
 
@@ -194,7 +206,12 @@ class ProtobufEventAttributeSerializer(object):
       cls.WriteSerializedObject(list_proto_add, u'', list_value)
 
     list_attribute = getattr(proto_attribute, attribute_name)
-    list_attribute.MergeFrom(list_proto)
+    try:
+      list_attribute.MergeFrom(list_proto)
+    except AttributeError as exception:
+      raise AttributeError(
+          u'Unable to merge attribute: {0:s} with error: {1:s}'.format(
+              attribute_name, exception))
 
 
 class ProtobufAnalysisReportSerializer(interface.AnalysisReportSerializer):
@@ -467,7 +484,7 @@ class ProtobufEventObjectSerializer(interface.EventObjectSerializer):
                 u'coming from file: {3:s}').format(
                     attribute_name, type(attribute_value), exception, path))
             # Catch potential out of range errors.
-            if isinstance(attribute_value, (int, long)):
+            if isinstance(attribute_value, py2to3.INTEGER_TYPES):
               setattr(proto, attribute_name, -1)
 
       else:
@@ -523,9 +540,9 @@ class ProtobufEventTagSerializer(interface.EventTagSerializer):
 
     for proto_attribute, attribute_value in proto.ListFields():
       if proto_attribute.name == u'tags':
-        event_tag.tags = []
+        event_tag._tags = []
         for proto_tag in proto.tags:
-          event_tag.tags.append(proto_tag.value)
+          event_tag._tags.append(proto_tag.value)
       else:
         setattr(event_tag, proto_attribute.name, attribute_value)
 
@@ -564,7 +581,7 @@ class ProtobufEventTagSerializer(interface.EventTagSerializer):
     for attribute_name in event_tag.__dict__:
       attribute_value = getattr(event_tag, attribute_name, None)
 
-      if (attribute_name == u'tags' and
+      if (attribute_name == u'_tags' and
           isinstance(attribute_value, (tuple, list))):
         for tag_string in attribute_value:
           proto_tag_add = proto.tags.add()
