@@ -16,7 +16,7 @@ class WindowsVolumeCreationEventFormatterTest(
   def testInitialization(self):
     """Tests the initialization."""
     event_formatter = windows.WindowsVolumeCreationEventFormatter()
-    self.assertNotEqual(event_formatter, None)
+    self.assertIsNotNone(event_formatter)
 
   def testGetFormatStringAttributeNames(self):
     """Tests the GetFormatStringAttributeNames function."""
