@@ -6,12 +6,13 @@ import unittest
 
 from dfvfs.path import fake_path_spec
 
+from plaso.dfwinreg import definitions as dfwinreg_definitions
+from plaso.dfwinreg import fake as dfwinreg_fake
 from plaso.formatters import winreg as _  # pylint: disable=unused-import
 from plaso.lib import timelib
 from plaso.parsers.winreg_plugins import appcompatcache
 
 from tests.parsers.winreg_plugins import test_lib
-from tests.winreg import test_lib as winreg_test_lib
 
 
 class TestFileEntry(object):
@@ -33,7 +34,7 @@ class TestFileEntry(object):
     self.path_spec = fake_path_spec.FakePathSpec(location=name)
 
   def GetStat(self):
-    """Retrieves the stat object (instance of vfs.VFSStat)."""
+    """Retrieves the stat object (instance of dfvfs.VFSStat)."""
     return
 
 
@@ -41,42 +42,47 @@ class AppCompatCacheRegistryPluginTest(test_lib.RegistryPluginTestCase):
   """Tests for the AppCompatCache Windows Registry plugin."""
 
   def setUp(self):
-    """Sets up the needed objects used throughout the test."""
+    """Makes preparations before running an individual test."""
     self._plugin = appcompatcache.AppCompatCachePlugin()
 
-  def _CreateTestAppCompatCache(self, time_string, binary_data):
-    """Creates a AppCompatCache Registry key and value for testing.
+  def _CreateTestKey(self, time_string, binary_data):
+    """Creates Registry keys and values for testing.
 
     Args:
       time_string: string containing the key last written date and time.
       binary_data: the binary data of the AppCompatCache Registry value.
 
     Returns:
-      A Windows Registry key object (instance of TestRegKey).
+      A Windows Registry key (instance of dfwinreg.WinRegistryKey).
     """
     key_path = u'\\ControlSet001\\Control\\Session Manager\\AppCompatCache'
-    expected_timestamp = timelib.Timestamp.CopyFromString(time_string)
-    values = [winreg_test_lib.TestRegValue(
-        u'AppCompatCache', binary_data,
-        winreg_test_lib.TestRegValue.REG_BINARY)]
+    filetime = dfwinreg_fake.Filetime()
+    filetime.CopyFromString(time_string)
+    registry_key = dfwinreg_fake.FakeWinRegistryKey(
+        u'AppCompatCache', key_path=key_path,
+        last_written_time=filetime.timestamp, offset=1456)
 
-    return winreg_test_lib.TestRegKey(
-        key_path, expected_timestamp, values, offset=1456)
+    registry_value = dfwinreg_fake.FakeWinRegistryValue(
+        u'AppCompatCache', data=binary_data,
+        data_type=dfwinreg_definitions.REG_BINARY)
+    registry_key.AddValue(registry_value)
 
-  def _ParseAppCompatCacheKey(self, file_entry, winreg_key):
+    return registry_key
+
+  def _ParseAppCompatCacheKey(self, file_entry, registry_key):
     """Parses the AppCompatCacheKey.
 
     Args:
       file_entry: the file entry object (instance of dfvfs.FileEntry).
-      winreg_key: the Windows Registry key object (instance of WinRegKey).
+      registry_key: the Windows Registry key object (instance of
+                    dfwinreg.WinRegistryKey).
 
     Returns:
       A list of event objects (instances of EventObjects).
     """
-    knowledge_base_values = {u'current_control_set': u'ControlSet001'}
     event_queue_consumer = self._ParseKeyWithPlugin(
-        self._plugin, winreg_key, knowledge_base_values=knowledge_base_values,
-        file_entry=file_entry, parser_chain=self._plugin.plugin_name)
+        self._plugin, registry_key, file_entry=file_entry,
+        parser_chain=self._plugin.plugin_name)
     return self._GetEventObjectsFromQueue(event_queue_consumer)
 
   def testProcessWindowsXP(self):
@@ -164,10 +170,10 @@ class AppCompatCacheRegistryPluginTest(test_lib.RegistryPluginTestCase):
         0xe9, 0x39, 0xca, 0x01
     ]))
 
-    file_entry = TestFileEntry(u'SYSTEM-XP')
-    winreg_key = self._CreateTestAppCompatCache(
+    test_file_entry = TestFileEntry(u'SYSTEM-XP')
+    registry_key = self._CreateTestKey(
         u'2015-06-15 11:53:37.043061', binary_data)
-    event_objects = self._ParseAppCompatCacheKey(file_entry, winreg_key)
+    event_objects = self._ParseAppCompatCacheKey(test_file_entry, registry_key)
 
     self.assertEqual(len(event_objects), 2)
 
@@ -175,11 +181,12 @@ class AppCompatCacheRegistryPluginTest(test_lib.RegistryPluginTestCase):
     event_object = event_objects[event_object_index]
 
     expected_path = u'\\??\\C:\\WINDOWS\\system32\\hticons.dll'
-    expected_msg = u'[{0:s}] Cached entry: {1:d} Path: {2:s}'.format(
+    expected_message = u'[{0:s}] Cached entry: {1:d} Path: {2:s}'.format(
         event_object.keyname, event_object_index + 1, expected_path)
-    expected_msg_short = u'Path: {0:s}'.format(expected_path)
+    expected_short_message = u'Path: {0:s}'.format(expected_path)
 
-    self._TestGetMessageStrings(event_object, expected_msg, expected_msg_short)
+    self._TestGetMessageStrings(
+        event_object, expected_message, expected_short_message)
 
   def testProcessWindows2003(self):
     """Tests the Process function for Windows 2003 AppCompatCache data."""
@@ -199,10 +206,10 @@ class AppCompatCacheRegistryPluginTest(test_lib.RegistryPluginTestCase):
         0x65, 0x00, 0x00, 0x00
     ]))
 
-    file_entry = TestFileEntry(u'SYSTEM-Windows2003')
-    winreg_key = self._CreateTestAppCompatCache(
+    test_file_entry = TestFileEntry(u'SYSTEM-Windows2003')
+    registry_key = self._CreateTestKey(
         u'2015-06-15 11:53:37.043061', binary_data)
-    event_objects = self._ParseAppCompatCacheKey(file_entry, winreg_key)
+    event_objects = self._ParseAppCompatCacheKey(test_file_entry, registry_key)
 
     self.assertEqual(len(event_objects), 1)
 
@@ -211,11 +218,12 @@ class AppCompatCacheRegistryPluginTest(test_lib.RegistryPluginTestCase):
 
     expected_path = (
         u'\\??\\C:\\WINDOWS\\Microsoft.NET\\Framework\\v1.1.4322\\ngen.exe')
-    expected_msg = u'[{0:s}] Cached entry: {1:d} Path: {2:s}'.format(
+    expected_message = u'[{0:s}] Cached entry: {1:d} Path: {2:s}'.format(
         event_object.keyname, event_object_index + 1, expected_path)
-    expected_msg_short = u'Path: {0:s}'.format(expected_path)
+    expected_short_message = u'Path: {0:s}'.format(expected_path)
 
-    self._TestGetMessageStrings(event_object, expected_msg, expected_msg_short)
+    self._TestGetMessageStrings(
+        event_object, expected_message, expected_short_message)
 
     # TODO: implement 64 bit
 
@@ -233,10 +241,10 @@ class AppCompatCacheRegistryPluginTest(test_lib.RegistryPluginTestCase):
         0x45, 0x00, 0x58, 0x00, 0x45, 0x00, 0x00, 0x00
     ]))
 
-    file_entry = TestFileEntry(u'SYSTEM-Vista')
-    winreg_key = self._CreateTestAppCompatCache(
+    test_file_entry = TestFileEntry(u'SYSTEM-Vista')
+    registry_key = self._CreateTestKey(
         u'2015-06-15 11:53:37.043061', binary_data)
-    event_objects = self._ParseAppCompatCacheKey(file_entry, winreg_key)
+    event_objects = self._ParseAppCompatCacheKey(test_file_entry, registry_key)
 
     self.assertEqual(len(event_objects), 1)
 
@@ -244,20 +252,25 @@ class AppCompatCacheRegistryPluginTest(test_lib.RegistryPluginTestCase):
     event_object = event_objects[event_object_index]
 
     expected_path = u'\\??\\C:\\Windows\\SYSTEM32\\WISPTIS.EXE'
-    expected_msg = u'[{0:s}] Cached entry: {1:d} Path: {2:s}'.format(
+    expected_message = u'[{0:s}] Cached entry: {1:d} Path: {2:s}'.format(
         event_object.keyname, event_object_index + 1, expected_path)
-    expected_msg_short = u'Path: {0:s}'.format(expected_path)
+    expected_short_message = u'Path: {0:s}'.format(expected_path)
 
-    self._TestGetMessageStrings(event_object, expected_msg, expected_msg_short)
+    self._TestGetMessageStrings(
+        event_object, expected_message, expected_short_message)
 
     # TODO: implement 64 bit
 
   def testProcessWindows7(self):
     """Tests the Process function for Windows 7 AppCompatCache data."""
-    file_entry = self._GetTestFileEntryFromPath([u'SYSTEM'])
-    key_path = u'\\ControlSet001\\Control\\Session Manager\\AppCompatCache'
-    winreg_key = self._GetKeyFromFileEntry(file_entry, key_path)
-    event_objects = self._ParseAppCompatCacheKey(file_entry, winreg_key)
+    test_file_entry = self._GetTestFileEntryFromPath([u'SYSTEM'])
+    key_path = (
+        u'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Control\\'
+        u'Session Manager\\AppCompatCache')
+
+    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
+    registry_key = win_registry.GetKeyByPath(key_path)
+    event_objects = self._ParseAppCompatCacheKey(test_file_entry, registry_key)
 
     self.assertEqual(len(event_objects), 330)
 
@@ -268,17 +281,18 @@ class AppCompatCacheRegistryPluginTest(test_lib.RegistryPluginTestCase):
         u'2012-04-04 01:46:37.932964')
     self.assertEqual(event_object.timestamp, expected_timestamp)
 
-    self.assertEqual(event_object.pathspec, file_entry.path_spec)
+    self.assertEqual(event_object.pathspec, test_file_entry.path_spec)
     # This should just be the plugin name, as we're invoking it directly,
     # and not through the parser.
     self.assertEqual(event_object.parser, self._plugin.plugin_name)
 
     expected_path = u'\\??\\C:\\Windows\\PSEXESVC.EXE'
-    expected_msg = u'[{0:s}] Cached entry: {1:d} Path: {2:s}'.format(
+    expected_message = u'[{0:s}] Cached entry: {1:d} Path: {2:s}'.format(
         event_object.keyname, event_object_index + 1, expected_path)
-    expected_msg_short = u'Path: {0:s}'.format(expected_path)
+    expected_short_message = u'Path: {0:s}'.format(expected_path)
 
-    self._TestGetMessageStrings(event_object, expected_msg, expected_msg_short)
+    self._TestGetMessageStrings(
+        event_object, expected_message, expected_short_message)
 
     # TODO: implement 64 bit
 
@@ -308,10 +322,10 @@ class AppCompatCacheRegistryPluginTest(test_lib.RegistryPluginTestCase):
         0x00, 0x00, 0x00, 0x00
     ]))
 
-    file_entry = TestFileEntry(u'SYSTEM-Windows8.0')
-    winreg_key = self._CreateTestAppCompatCache(
+    test_file_entry = TestFileEntry(u'SYSTEM-Windows8.0')
+    registry_key = self._CreateTestKey(
         u'2015-06-15 11:53:37.043061', binary_data)
-    event_objects = self._ParseAppCompatCacheKey(file_entry, winreg_key)
+    event_objects = self._ParseAppCompatCacheKey(test_file_entry, registry_key)
 
     self.assertEqual(len(event_objects), 1)
 
@@ -319,11 +333,12 @@ class AppCompatCacheRegistryPluginTest(test_lib.RegistryPluginTestCase):
     event_object = event_objects[event_object_index]
 
     expected_path = u'SYSVOL\\Windows\\System32\\wbem\\WmiPrvSE.exe'
-    expected_msg = u'[{0:s}] Cached entry: {1:d} Path: {2:s}'.format(
+    expected_message = u'[{0:s}] Cached entry: {1:d} Path: {2:s}'.format(
         event_object.keyname, event_object_index + 1, expected_path)
-    expected_msg_short = u'Path: {0:s}'.format(expected_path)
+    expected_short_message = u'Path: {0:s}'.format(expected_path)
 
-    self._TestGetMessageStrings(event_object, expected_msg, expected_msg_short)
+    self._TestGetMessageStrings(
+        event_object, expected_message, expected_short_message)
 
   def testProcessWindows8_1(self):
     """Tests the Process function for Windows 8.1 AppCompatCache data."""
@@ -350,10 +365,10 @@ class AppCompatCacheRegistryPluginTest(test_lib.RegistryPluginTestCase):
         0xce, 0x01, 0x00, 0x00, 0x00, 0x00
     ]))
 
-    file_entry = TestFileEntry(u'SYSTEM-Windows8.1')
-    winreg_key = self._CreateTestAppCompatCache(
+    test_file_entry = TestFileEntry(u'SYSTEM-Windows8.1')
+    registry_key = self._CreateTestKey(
         u'2015-06-15 11:53:37.043061', binary_data)
-    event_objects = self._ParseAppCompatCacheKey(file_entry, winreg_key)
+    event_objects = self._ParseAppCompatCacheKey(test_file_entry, registry_key)
 
     self.assertEqual(len(event_objects), 1)
 
@@ -361,11 +376,12 @@ class AppCompatCacheRegistryPluginTest(test_lib.RegistryPluginTestCase):
     event_object = event_objects[event_object_index]
 
     expected_path = u'SYSVOL\\Windows\\System32\\dllhost.exe'
-    expected_msg = u'[{0:s}] Cached entry: {1:d} Path: {2:s}'.format(
+    expected_message = u'[{0:s}] Cached entry: {1:d} Path: {2:s}'.format(
         event_object.keyname, event_object_index + 1, expected_path)
-    expected_msg_short = u'Path: {0:s}'.format(expected_path)
+    expected_short_message = u'Path: {0:s}'.format(expected_path)
 
-    self._TestGetMessageStrings(event_object, expected_msg, expected_msg_short)
+    self._TestGetMessageStrings(
+        event_object, expected_message, expected_short_message)
 
   def testProcessWindows10(self):
     """Tests the Process function for Windows 10 AppCompatCache data."""
@@ -394,10 +410,10 @@ class AppCompatCacheRegistryPluginTest(test_lib.RegistryPluginTestCase):
         0x00, 0x01, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00
     ]))
 
-    file_entry = TestFileEntry(u'SYSTEM-Windows10')
-    winreg_key = self._CreateTestAppCompatCache(
+    test_file_entry = TestFileEntry(u'SYSTEM-Windows10')
+    registry_key = self._CreateTestKey(
         u'2015-06-15 11:53:37.043061', binary_data)
-    event_objects = self._ParseAppCompatCacheKey(file_entry, winreg_key)
+    event_objects = self._ParseAppCompatCacheKey(test_file_entry, registry_key)
 
     self.assertEqual(len(event_objects), 1)
 
@@ -405,11 +421,12 @@ class AppCompatCacheRegistryPluginTest(test_lib.RegistryPluginTestCase):
     event_object = event_objects[event_object_index]
 
     expected_path = u'C:\\Windows\\system32\\MpSigStub.exe'
-    expected_msg = u'[{0:s}] Cached entry: {1:d} Path: {2:s}'.format(
+    expected_message = u'[{0:s}] Cached entry: {1:d} Path: {2:s}'.format(
         event_object.keyname, event_object_index + 1, expected_path)
-    expected_msg_short = u'Path: {0:s}'.format(expected_path)
+    expected_short_message = u'Path: {0:s}'.format(expected_path)
 
-    self._TestGetMessageStrings(event_object, expected_msg, expected_msg_short)
+    self._TestGetMessageStrings(
+        event_object, expected_message, expected_short_message)
 
 
 if __name__ == '__main__':
