@@ -15,11 +15,13 @@ try:
 except ImportError:
   win32console = None
 
-from dfvfs.helpers import source_scanner
+from dfvfs.lib import definitions as dfvfs_definitions
 
 import plaso
+from plaso import dependencies
 from plaso.cli import extraction_tool
 from plaso.cli import tools as cli_tools
+from plaso.cli import views as cli_views
 from plaso.frontend import log2timeline
 from plaso.lib import definitions
 from plaso.lib import errors
@@ -30,6 +32,10 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
   """Class that implements the log2timeline CLI tool.
 
   Attributes:
+    dependencies_check: a boolean value to indicate the availability and
+                        versions of dependencies should be checked.
+    list_output_modules: a boolean value to indicate information about the
+                         output modules should be shows.
     show_info: a boolean value to indicate information about hashers, parsers,
                plugins, etc. should be shown.
   """
@@ -53,7 +59,7 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
       u'',
       u'Instead of answering questions, indicate some of the options on the',
       u'command line (including data from particular VSS stores).',
-      (u'    log2timeline.py -o 63 --vss_stores 1,2 /cases/plaso_vss.plaso'
+      (u'    log2timeline.py -o 63 --vss_stores 1,2 /cases/plaso_vss.plaso '
        u'image.E01'),
       u'',
       u'And that is how you build a timeline using log2timeline...',
@@ -72,6 +78,7 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
     """
     super(Log2TimelineTool, self).__init__(
         input_reader=input_reader, output_writer=output_writer)
+    self._command_line_arguments = None
     self._enable_sigsegv_handler = False
     self._filter_expression = None
     self._foreman_verbose = False
@@ -83,6 +90,8 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
     self._status_view_mode = u'linear'
     self._output = None
 
+    self.dependencies_check = True
+    self.list_output_modules = False
     self.show_info = False
 
   def _ClearScreen(self):
@@ -105,7 +114,7 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
       console_size = screen_buffer_size.X * screen_buffer_size.Y
 
       screen_buffer.FillConsoleOutputCharacter(
-          b' ', console_size, top_left_coordinate)
+          u' ', console_size, top_left_coordinate)
       screen_buffer.FillConsoleOutputAttribute(
           screen_buffer_attributes, console_size, top_left_coordinate)
       screen_buffer.SetConsoleCursorPosition(top_left_coordinate)
@@ -124,11 +133,16 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
       number_of_events_delta: the number of events since the last status update.
       display_name: the display name of the file last processed.
     """
-    if (number_of_events_delta == 0 and
-        status == definitions.PROCESSING_STATUS_RUNNING):
+    if (process_status and number_of_events_delta == 0 and
+        status in [definitions.PROCESSING_STATUS_RUNNING,
+                   definitions.PROCESSING_STATUS_HASHING,
+                   definitions.PROCESSING_STATUS_PARSING]):
       status = process_status
 
     # This check makes sure the columns are tab aligned.
+    if len(identifier) < 8:
+      identifier = u'{0:s}\t'.format(identifier)
+
     if len(status) < 8:
       status = u'{0:s}\t'.format(status)
 
@@ -145,6 +159,34 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
     return u'{0:s}\t{1:d}\t{2:s}\t{3:s}\t{4:s}'.format(
         identifier, pid, status, events, display_name)
 
+  def _GetMatcher(self, filter_expression):
+    """Retrieves a filter object for a specific filter expression.
+
+    Args:
+      filter_expression: string that contains the filter expression.
+
+    Returns:
+      A filter object (instance of objectfilter.TODO) or None.
+    """
+    try:
+      parser = pfilter.BaseParser(filter_expression).Parse()
+      return parser.Compile(pfilter.PlasoAttributeFilterImplementation)
+
+    except errors.ParseError as exception:
+      logging.error(
+          u'Unable to create filter: {0:s} with error: {1:s}'.format(
+              filter_expression, exception))
+
+  def _ParseExperimentalOptions(self, options):
+    """Parses the experimental plugin options.
+
+    Args:
+      options: the command line arguments (instance of argparse.Namespace).
+    """
+    use_zeromq = getattr(options, u'use_zeromq', False)
+    if use_zeromq:
+      self._front_end.SetUseZeroMQ(use_zeromq)
+
   def _ParseOutputOptions(self, options):
     """Parses the output options.
 
@@ -154,9 +196,13 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
     Raises:
       BadConfigOption: if the options are invalid.
     """
-    self._output_module = getattr(options, u'output_module', None)
+    self._output_module = self.ParseStringOption(options, u'output_module')
+    if self._output_module == u'list':
+      self.list_output_modules = True
 
-    self._text_prepend = getattr(options, u'text_prepend', None)
+    text_prepend = self.ParseStringOption(options, u'text_prepend')
+    if text_prepend:
+      self._front_end.SetTextPrepend(text_prepend)
 
   def _ParseProcessingOptions(self, options):
     """Parses the processing options.
@@ -171,6 +217,8 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
 
     self._foreman_verbose = getattr(options, u'foreman_verbose', False)
 
+    self._number_of_extraction_workers = getattr(options, u'workers', 0)
+
     # TODO: add code to parse the worker options.
 
   def _PrintStatusHeader(self):
@@ -229,13 +277,14 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
 
       status_table.append(status_row)
 
-    status_row = self._FormatStatusTableRow(
-        processing_status.storage_writer.identifier,
-        processing_status.storage_writer.pid,
-        processing_status.storage_writer.status,
-        processing_status.storage_writer.process_status,
-        processing_status.storage_writer.number_of_events,
-        processing_status.storage_writer.number_of_events_delta, u'')
+    if processing_status.storage_writer:
+      status_row = self._FormatStatusTableRow(
+          processing_status.storage_writer.identifier,
+          processing_status.storage_writer.pid,
+          processing_status.storage_writer.status,
+          processing_status.storage_writer.process_status,
+          processing_status.storage_writer.number_of_events,
+          processing_status.storage_writer.number_of_events_delta, u'')
 
     status_table.append(status_row)
 
@@ -275,9 +324,22 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
                 extraction_worker_status.pid,
                 extraction_worker_status.number_of_events,
                 extraction_worker_status.display_name,
-                status == definitions.PROCESSING_STATUS_RUNNING,
+                status in [definitions.PROCESSING_STATUS_RUNNING,
+                           definitions.PROCESSING_STATUS_HASHING,
+                           definitions.PROCESSING_STATUS_PARSING],
                 extraction_worker_status.process_status))
 
+  def AddExperimentalOptions(self, argument_group):
+    """Adds experimental options to the argument group
+
+    Args:
+      argument_group: The argparse argument group (instance of
+                      argparse._ArgumentGroup).
+    """
+    argument_group.add_argument(
+        u'--use_zeromq', action=u'store_true', dest=u'use_zeromq', help=(
+            u'Enables experimental queueing using ZeroMQ'))
+
   def AddOutputOptions(self, argument_group):
     """Adds the output options to the argument group.
 
@@ -286,16 +348,17 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
                       argparse._ArgumentGroup).
     """
     argument_group.add_argument(
-        u'--output', dest=u'output_module', action=u'store', type=unicode,
+        u'--output', dest=u'output_module', action=u'store', type=str,
         default=u'', help=(
             u'Bypass the storage module directly storing events according to '
             u'the output module. This means that the output will not be in the '
             u'pstorage format but in the format chosen by the output module. '
-            u'[Please note this feature is EXPERIMENTAL at this time, use at '
-            u'own risk (eg. sqlite output does not yet work)]'))
+            u'Use "--output list" or "--info" to list the available output '
+            u'modules. Note this feature is EXPERIMENTAL at this time '
+            u'e.g. sqlite output does not yet work.'))
 
     argument_group.add_argument(
-        u'-t', u'--text', dest=u'text_prepend', action=u'store', type=unicode,
+        u'-t', u'--text', dest=u'text_prepend', action=u'store', type=str,
         default=u'', metavar=u'TEXT', help=(
             u'Define a free form text string that is prepended to each path '
             u'to make it easier to distinguish one record from another in a '
@@ -328,30 +391,67 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
   def ListHashers(self):
     """Lists information about the available hashers."""
     hashers_information = self._front_end.GetHashersInformation()
-    self.PrintHeader(u'Hashers')
-    for name, description in sorted(hashers_information):
-      self.PrintColumnValue(name, description)
 
-    self._output_writer.Write(u'\n')
+    table_view = cli_views.ViewsFactory.GetTableView(
+        self._views_format_type, column_names=[u'Name', u'Description'],
+        title=u'Hashers')
+
+    for name, description in sorted(hashers_information):
+      table_view.AddRow([name, description])
+    table_view.Write(self._output_writer)
+
+  def ListOutputModules(self):
+    """Lists the output modules."""
+    table_view = cli_views.ViewsFactory.GetTableView(
+        self._views_format_type, column_names=[u'Name', u'Description'],
+        title=u'Output Modules')
+    for name, output_class in self._front_end.GetOutputClasses():
+      table_view.AddRow([name, output_class.DESCRIPTION])
+    table_view.Write(self._output_writer)
+
+    disabled_classes = list(self._front_end.GetDisabledOutputClasses())
+    if not disabled_classes:
+      return
+
+    table_view = cli_views.ViewsFactory.GetTableView(
+        self._views_format_type, column_names=[u'Name', u'Description'],
+        title=u'Disabled Output Modules')
+    for name, output_class in disabled_classes:
+      table_view.AddRow([name, output_class.DESCRIPTION])
+    table_view.Write(self._output_writer)
 
   def ListParsersAndPlugins(self):
     """Lists information about the available parsers and plugins."""
     parsers_information = self._front_end.GetParsersInformation()
-    self.PrintHeader(u'Parsers')
-    for name, description in sorted(parsers_information):
-      self.PrintColumnValue(name, description)
 
-    plugins_information = self._front_end.GetParserPluginsInformation()
-    self.PrintHeader(u'Parser Plugins')
-    for name, description in sorted(plugins_information):
-      self.PrintColumnValue(name, description)
+    table_view = cli_views.ViewsFactory.GetTableView(
+        self._views_format_type, column_names=[u'Name', u'Description'],
+        title=u'Parsers')
+
+    for name, description in sorted(parsers_information):
+      table_view.AddRow([name, description])
+    table_view.Write(self._output_writer)
+
+    for parser_name in self._front_end.GetNamesOfParsersWithPlugins():
+      plugins_information = self._front_end.GetParserPluginsInformation(
+          parser_filter_string=parser_name)
+
+      table_title = u'Parser plugins: {0:s}'.format(parser_name)
+      table_view = cli_views.ViewsFactory.GetTableView(
+          self._views_format_type, column_names=[u'Name', u'Description'],
+          title=table_title)
+      for name, description in sorted(plugins_information):
+        table_view.AddRow([name, description])
+      table_view.Write(self._output_writer)
 
     presets_information = self._front_end.GetParserPresetsInformation()
-    self.PrintHeader(u'Parsers Presets')
-    for name, description in sorted(presets_information):
-      self.PrintColumnValue(name, description)
 
-    self._output_writer.Write(u'\n')
+    table_view = cli_views.ViewsFactory.GetTableView(
+        self._views_format_type, column_names=[u'Name', u'Parsers and plugins'],
+        title=u'Parser presets')
+    for name, description in sorted(presets_information):
+      table_view.AddRow([name, description])
+    table_view.Write(self._output_writer)
 
   def ParseArguments(self):
     """Parses the command line arguments.
@@ -367,6 +467,9 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
 
     self.AddBasicOptions(argument_parser)
 
+    experimental_group = argument_parser.add_argument_group(u'Experimental')
+    self.AddExperimentalOptions(experimental_group)
+
     extraction_group = argument_parser.add_argument_group(
         u'Extraction Arguments')
 
@@ -385,14 +488,31 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
         u'--info', dest=u'show_info', action=u'store_true', default=False,
         help=u'Print out information about supported plugins and parsers.')
 
+    info_group.add_argument(
+        u'--use_markdown', u'--use-markdown', dest=u'use_markdown',
+        action=u'store_true', default=False, help=(
+            u'Output lists in Markdown format use in combination with '
+            u'"--hashers list", "--parsers list" or "--timezone list"'))
+
+    info_group.add_argument(
+        u'--no_dependencies_check', u'--no-dependencies-check',
+        dest=u'dependencies_check', action=u'store_false', default=True,
+        help=u'Disable the dependencies check.')
+
     self.AddLogFileOptions(info_group)
 
+    # The window status-view mode has an annoying flicker on Windows,
+    # hence we default to linear status-view mode instead.
+    if sys.platform.startswith(u'win'):
+      default_status_view = u'linear'
+    else:
+      default_status_view = u'window'
+
     info_group.add_argument(
         u'--status_view', u'--status-view', dest=u'status_view_mode',
         choices=[u'linear', u'none', u'window'], action=u'store',
-        metavar=u'TYPE', default=u'linear', help=(
-            u'The processing status view mode: "linear", "none" or "window". '
-            u'Note that the "window" mode is still experimental.'))
+        metavar=u'TYPE', default=default_status_view, help=(
+            u'The processing status view mode: "linear", "none" or "window".'))
 
     output_group = argument_parser.add_argument_group(u'Output Arguments')
 
@@ -410,27 +530,27 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
     processing_group.add_argument(
         u'--sigsegv_handler', u'--sigsegv-handler', dest=u'sigsegv_handler',
         action=u'store_true', default=False, help=(
-            u'Enables the SIGSEGV handler. WARNINGN this functionality is '
-            u'expirimental and will a deadlock worker process if a real '
+            u'Enables the SIGSEGV handler. WARNING this functionality is '
+            u'experimental and will a deadlock worker process if a real '
             u'segfault is caught, but not signal SIGSEGV. This functionality '
             u'is therefore primarily intended for debugging purposes'))
 
     argument_parser.add_argument(
         u'output', action=u'store', metavar=u'STORAGE_FILE', nargs=u'?',
-        type=unicode, help=(
+        type=str, help=(
             u'The path to the output file, if the file exists it will get '
             u'appended to.'))
 
     argument_parser.add_argument(
-        u'source', action=u'store', metavar=u'SOURCE', nargs=u'?', type=unicode,
-        help=(
+        self._SOURCE_OPTION, action=u'store', metavar=u'SOURCE', nargs=u'?',
+        default=None, type=str, help=(
             u'The path to the source device, file or directory. If the source '
             u'is a supported storage media device or image file, archive file '
             u'or a directory, the files within are processed recursively.'))
 
     argument_parser.add_argument(
         u'filter', action=u'store', metavar=u'FILTER', nargs=u'?', default=None,
-        type=unicode, help=(
+        type=str, help=(
             u'A filter that can be used to filter the dataset before it '
             u'is written into storage. More information about the filters '
             u'and its usage can be found here: http://plaso.kiddaland.'
@@ -468,11 +588,13 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
       self.ParseOptions(options)
     except errors.BadConfigOption as exception:
       logging.error(u'{0:s}'.format(exception))
+
       self._output_writer.Write(u'\n')
       self._output_writer.Write(argument_parser.format_usage())
-
       return False
 
+    self._command_line_arguments = self.GetCommandLineArguments()
+
     return True
 
   def ParseOptions(self, options):
@@ -486,12 +608,22 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
     """
     # Check the list options first otherwise required options will raise.
     self._ParseExtractionOptions(options)
+    self._ParseOutputOptions(options)
+    # TODO: refactor usage of self._old_preprocess.
+    self._front_end.SetUseOldPreprocess(self._old_preprocess)
     self._ParseTimezoneOption(options)
+    self._ParseExperimentalOptions(options)
 
     self.show_info = getattr(options, u'show_info', False)
 
-    if (self.list_hashers or self.list_parsers_and_plugins or
-        self.list_timezones or self.show_info):
+    if getattr(options, u'use_markdown', False):
+      self._views_format_type = cli_views.ViewsFactory.FORMAT_TYPE_MARKDOWN
+
+    self.dependencies_check = getattr(options, u'dependencies_check', True)
+
+    if (self.list_hashers or self.list_output_modules or
+        self.list_parsers_and_plugins or self.list_timezones or
+        self.show_info):
       return
 
     super(Log2TimelineTool, self).ParseOptions(options)
@@ -509,16 +641,17 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
     else:
       logging_level = logging.INFO
 
-    log_file = getattr(options, u'log_file', None)
+    self.ParseLogFileOptions(options)
     self._ConfigureLogging(
-        filename=log_file, format_string=format_string, log_level=logging_level)
+        filename=self._log_file, format_string=format_string,
+        log_level=logging_level)
 
     if self._debug_mode:
       logging_filter = log2timeline.LoggingFilter()
       root_logger = logging.getLogger()
       root_logger.addFilter(logging_filter)
 
-    self._output = getattr(options, u'output', None)
+    self._output = self.ParseStringOption(options, u'output')
     if not self._output:
       raise errors.BadConfigOption(u'No output defined.')
 
@@ -528,10 +661,10 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
     if self._operating_system:
       self._mount_path = getattr(options, u'filename', None)
 
-    self._filter_expression = getattr(options, u'filter', None)
+    self._filter_expression = self.ParseStringOption(options, u'filter')
     if self._filter_expression:
       # TODO: refactor self._filter_object out the tool into the frontend.
-      self._filter_object = pfilter.GetMatcher(self._filter_expression)
+      self._filter_object = self._GetMatcher(self._filter_expression)
       if not self._filter_object:
         raise errors.BadConfigOption(
             u'Invalid filter expression: {0:s}'.format(self._filter_expression))
@@ -558,22 +691,18 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
     scan_context = self.ScanSource()
     self._source_type = scan_context.source_type
 
-    # TODO: move source_scanner.SourceScannerContext.SOURCE_TYPE_
-    # to definitions.SOURCE_TYPE_.
-    if self._source_type == (
-        source_scanner.SourceScannerContext.SOURCE_TYPE_DIRECTORY):
+    if self._source_type == dfvfs_definitions.SOURCE_TYPE_DIRECTORY:
       self._source_type_string = u'directory'
 
-    elif self._source_type == (
-        source_scanner.SourceScannerContext.SOURCE_TYPE_FILE):
+    elif self._source_type == dfvfs_definitions.SOURCE_TYPE_FILE:
       self._source_type_string = u'single file'
 
     elif self._source_type == (
-        source_scanner.SourceScannerContext.SOURCE_TYPE_STORAGE_MEDIA_DEVICE):
+        dfvfs_definitions.SOURCE_TYPE_STORAGE_MEDIA_DEVICE):
       self._source_type_string = u'storage media device'
 
     elif self._source_type == (
-        source_scanner.SourceScannerContext.SOURCE_TYPE_STORAGE_MEDIA_IMAGE):
+        dfvfs_definitions.SOURCE_TYPE_STORAGE_MEDIA_IMAGE):
       self._source_type_string = u'storage media image'
 
     else:
@@ -593,9 +722,11 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
 
     processing_status = self._front_end.ProcessSources(
         self._source_path_specs, self._source_type,
+        command_line_arguments=self._command_line_arguments,
         enable_sigsegv_handler=self._enable_sigsegv_handler,
         filter_file=self._filter_file,
         hasher_names_string=self._hasher_names_string,
+        number_of_extraction_workers=self._number_of_extraction_workers,
         parser_filter_string=self._parser_filter_string,
         preferred_encoding=self.preferred_encoding,
         single_process_mode=self._single_process_mode,
@@ -617,16 +748,17 @@ class Log2TimelineTool(extraction_tool.ExtractionTool):
 
   def ShowInfo(self):
     """Shows information about available hashers, parsers, plugins, etc."""
-    plugin_list = self._front_end.GetPluginData()
     self._output_writer.Write(
         u'{0:=^80s}\n'.format(u' log2timeline/plaso information '))
 
+    plugin_list = self._front_end.GetPluginData()
     for header, data in plugin_list.items():
-      self.PrintHeader(header)
+      table_view = cli_views.ViewsFactory.GetTableView(
+          self._views_format_type, column_names=[u'Name', u'Description'],
+          title=header)
       for entry_header, entry_data in sorted(data):
-        self.PrintColumnValue(entry_header, entry_data)
-
-    self._output_writer.Write(u'\n')
+        table_view.AddRow([entry_header, entry_data])
+      table_view.Write(self._output_writer)
 
 
 def Main():
@@ -651,6 +783,10 @@ def Main():
     tool.ListParsersAndPlugins()
     have_list_option = True
 
+  if tool.list_output_modules:
+    tool.ListOutputModules()
+    have_list_option = True
+
   if tool.list_timezones:
     tool.ListTimeZones()
     have_list_option = True
@@ -658,6 +794,10 @@ def Main():
   if have_list_option:
     return True
 
+  if tool.dependencies_check and not dependencies.CheckDependencies(
+      verbose_output=False):
+    return False
+
   try:
     tool.ProcessSources()
 
