@@ -15,16 +15,18 @@ import pdb
 import sys
 import time
 
-from plaso import filters
-# TODO: remove after psort options refactor.
-from plaso.analysis import interface as analysis_interface
+# The following import makes sure the filters are registered.
+from plaso import filters  # pylint: disable=unused-import
 from plaso.cli import analysis_tool
 from plaso.cli import tools as cli_tools
+from plaso.cli import views as cli_views
 from plaso.cli.helpers import manager as helpers_manager
+from plaso.filters import manager as filters_manager
+from plaso.frontend import frontend
 from plaso.frontend import psort
 from plaso.output import interface as output_interface
-from plaso.output import manager as output_manager
 from plaso.lib import errors
+from plaso.lib import timelib
 from plaso.winnt import language_ids
 
 
@@ -35,7 +37,7 @@ class PsortOptions(object):
 class PsortTool(analysis_tool.AnalysisTool):
   """Class that implements the psort CLI tool."""
 
-  _URL = u'http://plaso.kiddaland.net/usage/filters'
+  _FILTERS_URL = u'http://plaso.kiddaland.net/usage/filters'
 
   NAME = u'psort'
   DESCRIPTION = (
@@ -57,14 +59,14 @@ class PsortTool(analysis_tool.AnalysisTool):
         input_reader=input_reader, output_writer=output_writer)
     self._analysis_plugins = None
     self._analysis_plugins_output_format = None
+    self._command_line_arguments = None
     self._deduplicate_events = True
     self._filter_expression = None
     self._filter_object = None
     self._front_end = psort.PsortFrontend()
     self._options = None
     self._output_format = None
-    self._time_slice_event_time_string = None
-    self._time_slice_duration = 5
+    self._time_slice = None
     self._use_time_slicer = False
 
     self.list_analysis_plugins = False
@@ -81,9 +83,12 @@ class PsortTool(analysis_tool.AnalysisTool):
     analysis_plugin_info = self._front_end.GetAnalysisPluginInfo()
     analysis_plugin_names = set([
         name.lower() for name, _, _ in analysis_plugin_info])
-    analysis_plugin_string = getattr(options, u'analysis_plugins', u'')
-    if analysis_plugin_string == u'':
+
+    analysis_plugin_string = self.ParseStringOption(
+        options, u'analysis_plugins')
+    if not analysis_plugin_string:
       return
+
     requested_plugin_names = set([
         name.strip().lower() for name in analysis_plugin_string.split(u',')])
 
@@ -96,6 +101,16 @@ class PsortTool(analysis_tool.AnalysisTool):
 
     self._analysis_plugins = analysis_plugin_string
 
+  def _ParseExperimentalOptions(self, options):
+    """Parses the experimental plugin options.
+
+    Args:
+      options: the command line arguments (instance of argparse.Namespace).
+    """
+    use_zeromq = getattr(options, u'use_zeromq', False)
+    if use_zeromq:
+      self._front_end.SetUseZeroMQ(use_zeromq)
+
   def _ParseFilterOptions(self, options):
     """Parses the filter options.
 
@@ -105,24 +120,39 @@ class PsortTool(analysis_tool.AnalysisTool):
     Raises:
       BadConfigOption: if the options are invalid.
     """
-    self._filter_expression = getattr(options, u'filter', None)
+    self._filter_expression = self.ParseStringOption(options, u'filter')
     if self._filter_expression:
-      self._filter_object = filters.GetFilter(self._filter_expression)
+      self._filter_object = filters_manager.FiltersManager.GetFilterObject(
+          self._filter_expression)
       if not self._filter_object:
         raise errors.BadConfigOption(
             u'Invalid filter expression: {0:s}'.format(self._filter_expression))
 
-    self._time_slice_event_time_string = getattr(options, u'slice', None)
-    self._time_slice_duration = getattr(options, u'slice_size', 5)
+    time_slice_event_time_string = getattr(options, u'slice', None)
+    time_slice_duration = getattr(options, u'slice_size', 5)
     self._use_time_slicer = getattr(options, u'slicer', False)
 
     self._front_end.SetFilter(self._filter_object, self._filter_expression)
 
     # The slice and slicer cannot be set at the same time.
-    if self._time_slice_event_time_string and self._use_time_slicer:
+    if time_slice_event_time_string and self._use_time_slicer:
       raise errors.BadConfigOption(
           u'Time slice and slicer cannot be used at the same time.')
 
+    time_slice_event_timestamp = None
+    if time_slice_event_time_string:
+      time_slice_event_timestamp = timelib.Timestamp.FromTimeString(
+          time_slice_event_time_string, timezone=self._timezone)
+      if time_slice_event_timestamp is None:
+        raise errors.BadConfigOption(
+            u'Unsupported time slice event date and time: {0:s}'.format(
+                time_slice_event_time_string))
+
+    if time_slice_event_timestamp is not None or self._use_time_slicer:
+      # Note that time slicer uses the time slice to determine the duration.
+      self._time_slice = frontend.TimeSlice(
+          time_slice_event_timestamp, duration=time_slice_duration)
+
   def _ParseInformationalOptions(self, options):
     """Parses the informational options.
 
@@ -143,7 +173,8 @@ class PsortTool(analysis_tool.AnalysisTool):
     Args:
       options: the command line arguments (instance of argparse.Namespace).
     """
-    preferred_language = getattr(options, u'preferred_language', u'en-US')
+    preferred_language = self.ParseStringOption(
+        options, u'preferred_language', default_value=u'en-US')
 
     if preferred_language == u'list':
       self.list_language_identifiers = True
@@ -154,14 +185,9 @@ class PsortTool(analysis_tool.AnalysisTool):
     """Processes a plaso storage file.
 
     Raises:
+      BadConfigOption: when a configuration parameter fails validation.
       RuntimeError: if a non-recoverable situation is encountered.
     """
-    time_slice = None
-    if self._time_slice_event_time_string is not None or self._use_time_slicer:
-      time_slice = self._front_end.GetTimeSlice(
-          self._time_slice_event_time_string,
-          duration=self._time_slice_duration, timezone=self._timezone)
-
     if self._analysis_plugins:
       read_only = False
     else:
@@ -179,7 +205,10 @@ class PsortTool(analysis_tool.AnalysisTool):
         storage_file, preferred_encoding=self.preferred_encoding,
         timezone=self._timezone)
 
-    if isinstance(output_module, output_interface.LinearOutputModule):
+    # TODO: Remove this check for the xlsx output module when
+    # DynamicOutputModule no longer inherits LinearOutputModule. (issue #393)
+    if (isinstance(output_module, output_interface.LinearOutputModule) and
+        output_module.NAME != u'xlsx'):
       if self._output_filename:
         output_file_object = open(self._output_filename, u'wb')
         output_writer = cli_tools.FileObjectOutputWriter(output_file_object)
@@ -193,11 +222,8 @@ class PsortTool(analysis_tool.AnalysisTool):
     if hasattr(output_module, u'SetFieldsFilter') and self._filter_object:
       output_module.SetFieldsFilter(self._filter_object)
 
-    try:
-      helpers_manager.ArgumentHelperManager.ParseOptions(
-          self._options, output_module)
-    except errors.BadConfigOption as exception:
-      raise RuntimeError(exception)
+    helpers_manager.ArgumentHelperManager.ParseOptions(
+        self._options, output_module)
 
     # Check if there are parameters that have not been defined and need to
     # in order for the output module to continue. Prompt user to supply
@@ -215,11 +241,14 @@ class PsortTool(analysis_tool.AnalysisTool):
               u'Unable to set the missing parameter for: {0:s}'.format(
                   parameter))
           continue
+
         setattr(configuration_object, parameter, value)
+
       helpers_manager.ArgumentHelperManager.ParseOptions(
           configuration_object, output_module)
       missing_parameters = output_module.GetMissingArguments()
 
+    # TODO: fix or remove this comment.
     # Get ANALYSIS PLUGINS AND CONFIGURE!
     get_plugins_and_producers = self._front_end.GetAnalysisPluginsAndEventQueues
     analysis_plugins, event_queue_producers = get_plugins_and_producers(
@@ -230,15 +259,23 @@ class PsortTool(analysis_tool.AnalysisTool):
           self._options, analysis_plugin)
 
     counter = self._front_end.ProcessStorage(
-        output_module, storage_file, analysis_plugins, event_queue_producers,
+        output_module, storage_file, self._storage_file_path,
+        analysis_plugins, event_queue_producers,
+        command_line_arguments=self._command_line_arguments,
         deduplicate_events=self._deduplicate_events,
         preferred_encoding=self.preferred_encoding,
-        time_slice=time_slice, use_time_slicer=self._use_time_slicer)
+        time_slice=self._time_slice, use_time_slicer=self._use_time_slicer)
+
+    storage_file.Close()
 
     if not self._quiet_mode:
-      self.PrintHeader(u'Counter')
+      self._output_writer.Write(u'Processing completed.\n')
+
+      table_view = cli_views.ViewsFactory.GetTableView(
+          self._views_format_type, title=u'Counter')
       for element, count in counter.most_common():
-        self.PrintColumnValue(element, u'{0:d}'.format(count))
+        table_view.AddRow([element, count])
+      table_view.Write(self._output_writer)
 
   def _PromptUserForInput(self, input_text):
     """Prompts user for an input and return back read data.
@@ -269,6 +306,17 @@ class PsortTool(analysis_tool.AnalysisTool):
     helpers_manager.ArgumentHelperManager.AddCommandLineArguments(
         argument_group, u'analysis')
 
+  def AddExperimentalOptions(self, argument_group):
+    """Adds experimental options to the argument group
+
+    Args:
+      argument_group: The argparse argument group (instance of
+                      argparse._ArgumentGroup).
+    """
+    argument_group.add_argument(
+        u'--use_zeromq', u'--use-zeromq', action=u'store_true',
+        dest=u'use_zeromq', help=u'Enables experimental queueing using ZeroMQ')
+
   def AddFilterOptions(self, argument_group):
     """Adds the filter options to the argument group.
 
@@ -285,8 +333,8 @@ class PsortTool(analysis_tool.AnalysisTool):
             u'--slice_size but defaults to 5 minutes.'))
 
     argument_group.add_argument(
-        u'--slice_size', dest=u'slice_size', type=int, default=5,
-        action=u'store', help=(
+        u'--slice_size', u'--slice-size', dest=u'slice_size', type=int,
+        default=5, action=u'store', help=(
             u'Defines the slice size. In the case of a regular time slice it '
             u'defines the number of minutes the slice size should be. In the '
             u'case of the --slicer it determines the number of events before '
@@ -303,10 +351,11 @@ class PsortTool(analysis_tool.AnalysisTool):
 
     argument_group.add_argument(
         u'filter', nargs=u'?', action=u'store', metavar=u'FILTER', default=None,
-        type=unicode, help=(
+        type=str, help=(
             u'A filter that can be used to filter the dataset before it '
             u'is written into storage. More information about the filters '
-            u'and how to use them can be found here: {0:s}').format(self._URL))
+            u'and how to use them can be found here: {0:s}').format(
+                self._FILTERS_URL))
 
   def AddLanguageOptions(self, argument_group):
     """Adds the language options to the argument group.
@@ -317,7 +366,7 @@ class PsortTool(analysis_tool.AnalysisTool):
     """
     argument_group.add_argument(
         u'--language', metavar=u'LANGUAGE', dest=u'preferred_language',
-        default=u'en-US', type=unicode, help=(
+        default=u'en-US', type=str, help=(
             u'The preferred language identifier for Windows Event Log message '
             u'strings. Use "--language list" to see a list of available '
             u'language identifiers. Note that formatting will fall back on '
@@ -341,54 +390,51 @@ class PsortTool(analysis_tool.AnalysisTool):
 
   def ListAnalysisPlugins(self):
     """Lists the analysis modules."""
-    self.PrintHeader(u'Analysis Plugins')
-    format_length = 10
     analysis_plugin_info = self._front_end.GetAnalysisPluginInfo()
 
+    column_width = 10
     for name, _, _ in analysis_plugin_info:
-      if len(name) > format_length:
-        format_length = len(name)
-
-    # TODO: refactor to use type object (class) and add GetTypeString method.
-    for name, description, plugin_type in analysis_plugin_info:
-      if plugin_type == analysis_interface.AnalysisPlugin.TYPE_ANNOTATION:
-        type_string = u'Annotation/tagging plugin'
-      elif plugin_type == analysis_interface.AnalysisPlugin.TYPE_ANOMALY:
-        type_string = u'Anomaly plugin'
-      elif plugin_type == analysis_interface.AnalysisPlugin.TYPE_REPORT:
-        type_string = u'Summary/Report plugin'
-      elif plugin_type == analysis_interface.AnalysisPlugin.TYPE_STATISTICS:
-        type_string = u'Statistics plugin'
-      else:
-        type_string = u'Unknown type'
+      if len(name) > column_width:
+        column_width = len(name)
 
+    table_view = cli_views.ViewsFactory.GetTableView(
+        self._views_format_type, column_names=[u'Name', u'Description'],
+        title=u'Analysis Plugins')
+    # TODO: add support for a 3 column table.
+    for name, description, type_string in analysis_plugin_info:
       description = u'{0:s} [{1:s}]'.format(description, type_string)
-      self.PrintColumnValue(name, description, format_length)
-    self.PrintSeparatorLine()
+      table_view.AddRow([name, description])
+    table_view.Write(self._output_writer)
 
   def ListLanguageIdentifiers(self):
     """Lists the language identifiers."""
-    self.PrintHeader(u'Language identifiers')
-    self.PrintColumnValue(u'Identifier', u'Language')
+    table_view = cli_views.ViewsFactory.GetTableView(
+        self._views_format_type, column_names=[u'Identifier', u'Language'],
+        title=u'Language identifiers')
     for language_id, value_list in sorted(
         language_ids.LANGUAGE_IDENTIFIERS.items()):
-      self.PrintColumnValue(language_id, value_list[1])
+      table_view.AddRow([language_id, value_list[1]])
+    table_view.Write(self._output_writer)
 
   def ListOutputModules(self):
     """Lists the output modules."""
-    self.PrintHeader(u'Output Modules')
-    for name, output_class in sorted(self._front_end.GetOutputClasses()):
-      self.PrintColumnValue(name, output_class.DESCRIPTION, 10)
-    self.PrintSeparatorLine()
+    table_view = cli_views.ViewsFactory.GetTableView(
+        self._views_format_type, column_names=[u'Name', u'Description'],
+        title=u'Output Modules')
+    for name, output_class in self._front_end.GetOutputClasses():
+      table_view.AddRow([name, output_class.DESCRIPTION])
+    table_view.Write(self._output_writer)
 
-    # Assign to an attribute due to line length limitations.
-    disabled_classes = output_manager.OutputManager.GetDisabledOutputClasses
-    if not disabled_classes():
+    disabled_classes = list(self._front_end.GetDisabledOutputClasses())
+    if not disabled_classes:
       return
-    self.PrintHeader(u'Disabled Output Modules')
-    for output_class in disabled_classes():
-      self.PrintColumnValue(output_class.NAME, output_class.DESCRIPTION, 10)
-    self.PrintSeparatorLine()
+
+    table_view = cli_views.ViewsFactory.GetTableView(
+        self._views_format_type, column_names=[u'Name', u'Description'],
+        title=u'Disabled Output Modules')
+    for name, output_class in disabled_classes:
+      table_view.AddRow([name, output_class.DESCRIPTION])
+    table_view.Write(self._output_writer)
 
   def ParseArguments(self):
     """Parses the command line arguments.
@@ -410,10 +456,13 @@ class PsortTool(analysis_tool.AnalysisTool):
 
     analysis_group.add_argument(
         u'--analysis', metavar=u'PLUGIN_LIST', dest=u'analysis_plugins',
-        default=u'', action=u'store', type=unicode, help=(
+        default=u'', action=u'store', type=str, help=(
             u'A comma separated list of analysis plugin names to be loaded '
             u'or "--analysis list" to see a list of available plugins.'))
 
+    experimental_group = argument_parser.add_argument_group(u'Experimental')
+    self.AddExperimentalOptions(experimental_group)
+
     info_group = argument_parser.add_argument_group(u'Informational Arguments')
 
     self.AddLogFileOptions(info_group)
@@ -431,8 +480,8 @@ class PsortTool(analysis_tool.AnalysisTool):
     output_group = argument_parser.add_argument_group(u'Output Arguments')
 
     output_group.add_argument(
-        u'-a', u'--include_all', action=u'store_false', dest=u'dedup',
-        default=True, help=(
+        u'-a', u'--include_all', u'--include-all', action=u'store_false',
+        dest=u'dedup', default=True, help=(
             u'By default the psort removes duplicate entries from the '
             u'output. This parameter changes that behavior so all events '
             u'are included.'))
@@ -549,8 +598,8 @@ class PsortTool(analysis_tool.AnalysisTool):
     super(PsortTool, self).ParseOptions(options)
     self._ParseDataLocationOption(options)
     self._ParseAnalysisPluginOptions(options)
+    self._ParseExperimentalOptions(options)
     self._ParseFilterOptions(options)
-    self._front_end.SetStorageFile(self._storage_file_path)
 
     if self._debug_mode:
       logging_level = logging.DEBUG
@@ -559,8 +608,8 @@ class PsortTool(analysis_tool.AnalysisTool):
     else:
       logging_level = logging.INFO
 
-    log_file = getattr(options, u'log_file', None)
-    self._ConfigureLogging(filename=log_file, log_level=logging_level)
+    self.ParseLogFileOptions(options)
+    self._ConfigureLogging(filename=self._log_file, log_level=logging_level)
 
     self._output_format = getattr(options, u'output_format', None)
     if not self._output_format:
@@ -574,8 +623,6 @@ class PsortTool(analysis_tool.AnalysisTool):
     self._deduplicate_events = getattr(options, u'dedup', True)
 
     self._output_filename = getattr(options, u'write', None)
-    if self._output_filename:
-      self._front_end.SetOutputFilename(self._output_filename)
 
     if self._data_location:
       self._front_end.SetDataLocation(self._data_location)
@@ -584,6 +631,8 @@ class PsortTool(analysis_tool.AnalysisTool):
     else:
       logging.warning(u'Unable to automatically determine data location.')
 
+    self._command_line_arguments = self.GetCommandLineArguments()
+
     # TODO: refactor this.
     self._options = options
 
@@ -600,6 +649,9 @@ class PsortTool(analysis_tool.AnalysisTool):
     except KeyboardInterrupt:
       pass
 
+    except errors.BadConfigOption as exception:
+      logging.error(u'{0:s}'.format(exception))
+
     # Catching every remaining exception in case we are debugging.
     except Exception as exception:
       if not self._debug_mode:
